{
  Name    :   Account.pas

  Purpose :   Classes for managing lists of Delphi Bank Accounts

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit Account;

interface

uses
  Classes,
  Controls,
  DelphiBankServer3_TLB,
  Transactions;

type

  TAccount = class (TCollectionItem)
  private
    FCurrentBalance: Double;
    FMinBalance: Double;
    FAccountType: integer;
    FLastStmnt: TDate;
    FTransactionCollection: TTransactionCollection;
    FAccountID: integer;
    procedure SetAccountType(const Value: integer);
    procedure SetCurrentBalance(const Value: Double);
    procedure SetLastStmnt(const Value: TDate);
    procedure SetMinBalance(const Value: Double);
    procedure SetTransactionCollection(
      const Value: TTransactionCollection);
    function GetTransactionCollection: TTransactionCollection;
    procedure SetAccountID(const Value: integer);
  public
    property AccountID : integer read FAccountID write SetAccountID;
    property AccountType : integer read FAccountType write SetAccountType;
    property MinBalance : Double read FMinBalance write SetMinBalance;
    property CurrentBalance : Double read FCurrentBalance write SetCurrentBalance;
    property LastStmnt : TDate read FLastStmnt write SetLastStmnt;
    Property TransactionCollection : TTransactionCollection read GetTransactionCollection write SetTransactionCollection;
  end;

  TAccountCollection = class (TCollection)
  private
    FCustID: Integer;
  protected
    procedure ReadAccountsArray (ActsArray : OLEVariant);
  public
    procedure Load; virtual;
    Property CustID: Integer read FCustID write FCustID;
  end;

implementation


uses
  SysUtils,
  ClientUtils;

{ TAccountCollection }

procedure TAccountCollection.Load;
var
  ActsArray : OLEVariant;
  StrDebug : WideString;
begin
  if ClientServices.Customers.ListAccounts(ClientServices.ClientKey,
                                          CustID, ActsArray, StrDebug) <> 0 then
    raise Exception.Create ('Problem reading account details: ' + StrDebug)
  else
    ReadAccountsArray (ActsArray);
end;

procedure TAccountCollection.ReadAccountsArray(ActsArray: OLEVariant);
var
  i : integer;
  MaxAccounts : integer;
  ThisAccount : TAccount;
begin
  Clear; // remove list of accounts.

  MaxAccounts := VarArrayHighBound(ActsArray, 2);

  // First row contains field names
  for i := 1 to MaxAccounts do
  begin
    ThisAccount :=  TAccount (self.Add);
    ThisAccount.AccountID := ActsArray[0, i];
    // skip item 1, as it is customer id.
    ThisAccount.AccountType := ActsArray[2, i];
    ThisAccount.MinBalance := ActsArray[3, i];
    ThisAccount.CurrentBalance := ActsArray[4, i];
    ThisAccount.LastStmnt := ActsArray[5, i];
  end;
end;


{ TAccount }

function TAccount.GetTransactionCollection: TTransactionCollection;
begin
  if FTransactionCollection = nil then
  begin
    FTransactionCollection := TTransactionCollection.Create (TTransaction);
    FTransactionCollection.AccountID := AccountID;
  end;

  result := FTransactionCollection;
end;

procedure TAccount.SetAccountID(const Value: integer);
begin
  FAccountID := Value;
end;

procedure TAccount.SetAccountType(const Value: integer);
begin
  FAccountType := Value;
end;

procedure TAccount.SetCurrentBalance(const Value: Double);
begin
  FCurrentBalance := Value;
end;

procedure TAccount.SetLastStmnt(const Value: TDate);
begin
  FLastStmnt := Value;
end;

procedure TAccount.SetMinBalance(const Value: Double);
begin
  FMinBalance := Value;
end;

procedure TAccount.SetTransactionCollection(
  const Value: TTransactionCollection);
begin
  FTransactionCollection := Value;
end;

end.
