{
  Name    :   AccountTypes.pas

  Purpose :   Classes for managing lists of Delphi Bank Accounts Types

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit AccountTypes;

interface

uses
  Classes,
  Controls,
  DelphiBankServer3_TLB;

type

  TAccountType = class (TCollectionItem)
  private
    FAccountTypeID: integer;
    FMinBalance: Double;
    FInterestRate: Double;
    FAccountName: String;
  public
    property AccountTypeID : integer read FAccountTypeID write FAccountTypeID;
    property MinBalance : Double read FMinBalance write FMinBalance;
    property AccountName : String read FAccountName write FAccountName;
    property InterestRate : Double read FInterestRate write FInterestRate;
  end;

  TAccountTypeCollection = class (TCollection)
  protected
    procedure ReadACTypesArray (ACTypesArray : OLEVariant);
  public
    procedure Load; virtual;
  end;

implementation

uses
  SysUtils,
  ClientUtils;

{ TAccountCollection }

procedure TAccountTypeCollection.Load;
var
  AcTypesArray : OLEVariant;
  StrDebug : WideString;
begin
  if ClientServices.AccountsTypes.ListAll (ClientServices.ClientKey, AcTypesArray, StrDebug) <> 0 then
    raise Exception.Create ('Problem reading account details: ' + StrDebug)
  else
    ReadACTypesArray (AcTypesArray);
end;

procedure TAccountTypeCollection.ReadACTypesArray (ACTypesArray : OLEVariant);
var
  i : integer;
  MaxElement : integer;
  ThisAccountType : TAccountType;
begin
  Clear; // remove list of accounts.

  MaxElement := VarArrayHighBound(ACTypesArray, 2);

  // First row contains field names
  for i := 1 to MaxElement do
  begin
    ThisAccountType :=  TAccountType (self.Add);
    ThisAccountType.AccountTypeID := ACTypesArray[0, i];
    ThisAccountType.AccountName := ACTypesArray[1, i];
    ThisAccountType.InterestRate := ACTypesArray[2, i];
    ThisAccountType.MinBalance := ACTypesArray[3, i];
  end;
end;

end.

