{
  Name    :   AccountListForm.pas

  Purpose :   Handles user interaction for Delphi Bank Accounts

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit AccountsListForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CustomerInfo, ComCtrls, ActnList, ToolWin, Grids, StdCtrls, ExtCtrls,
  BankWinMessages;

type
  TAccountsList = class(TForm)
    ToolBar1: TToolBar;
    ActionList1: TActionList;
    New: TAction;
    AccountsListViewer: TListView;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    CustNameEdit: TEdit;
    CustNotesEdit: TEdit;
    Transfer: TAction;
    ToolButton3: TToolButton;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure NewUpdate(Sender: TObject);
    procedure AccountsListViewerDblClick(Sender: TObject);
    procedure TransferExecute(Sender: TObject);
    procedure TransferUpdate(Sender: TObject);
  private
    FCust: TCustomer;
    procedure SetCust(const Value: TCustomer);
    { Private declarations }
  protected
    procedure UpdateAccountWindows;
    procedure BWMCust (var Message : TMessage); message BWM_CUST;

  public
    { Public declarations }
    class procedure ShowAccounts (Cust : TCustomer);
    procedure ShowCustomer;
    procedure LoadAccounts;
    property Cust : TCustomer read FCust write SetCust;
  end;


implementation

uses
  DelphiBankServer3_TLB,
  ClientUtils,
  Account, TransactionsFrm, Transactions, TransferMoneyDialog;

{$R *.DFM}

{ TAccountsList }

class procedure TAccountsList.ShowAccounts(Cust: TCustomer);
var
  AccountsList: TAccountsList;
begin
  AccountsList := TAccountsList.Create (Application);
  AccountsList.Cust := Cust;
  AccountsList.Visible := True;
end;

procedure TAccountsList.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := true;
end;

procedure TAccountsList.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TAccountsList.SetCust(const Value: TCustomer);
begin
  FCust := Value;
  ShowCustomer;
  LoadAccounts;
end;

procedure TAccountsList.LoadAccounts;
var
  i : integer;
  ListItem : TListItem;
  Account : TAccount;
begin
  Assert (Cust <> nil);
  Assert (Cust.AccountCollection <> nil);

  AccountsListViewer.Items.Clear;

  for i := 0 to Cust.AccountCollection.Count -1 do
  begin
    ListItem := AccountsListViewer.Items.Add;
    Account := Cust.AccountCollection.items[i] as TAccount;
    ListItem.Data := Account;
    ListItem.Caption :=
      ClientServices.AccountType [Account.AccountType].AccountName +
      ' (' + Trim(Format ('%f', [ClientServices.AccountType [Account.AccountType].InterestRate])) + '%)';
    ListItem.SubItems.Add (CurrencyString + CurrToStr (Account.CurrentBalance));
    ListItem.SubItems.Add (DateToStr (Account.LastStmnt));
    ListItem.SubItems.Add (CurrencyString + CurrToStr (Account.MinBalance));
  end;
end;

procedure TAccountsList.ShowCustomer;
begin
  CustNameEdit.Text := Cust.UserName;
  CustNotesEdit.Text := Cust.CustNotes;
  Caption := Cust.UserName;
end;

procedure TAccountsList.NewUpdate(Sender: TObject);
begin
  New.Enabled := FCust <> nil;
end;

procedure TAccountsList.AccountsListViewerDblClick(Sender: TObject);
var
  MyListItem : TListItem;
  MyAccount : TAccount;

begin
  if AccountsListViewer.Selected <> nil then
  begin
    MyListItem := AccountsListViewer.Selected;
    assert (MyListItem.data <> nil);
    MyAccount  := TAccount (MyListItem.Data);
    MyAccount.TransactionCollection.Load;

    TTransactionsForm.ShowAccount(MyAccount, Cust.UserName, Cust.CustID);
  end;
end;

procedure TAccountsList.TransferExecute(Sender: TObject);
begin
  if TTxMoneyDialog.TransferMoney(Cust) then
  begin
    FCust.AccountCollection.Load;
    LoadAccounts;
    UpdateAccountWindows;
  end;
end;

procedure TAccountsList.TransferUpdate(Sender: TObject);
begin
  Transfer.Enabled := FCust.AccountCollection.Count > 1;
end;


procedure TAccountsList.UpdateAccountWindows;
var
  i : integer;
  Account : TAccount;
begin
  Assert (Cust <> nil);
  Assert (Cust.AccountCollection <> nil);
  for i := 0 to Cust.AccountCollection.Count -1 do
  begin
    Account := Cust.AccountCollection.items[i] as TAccount;
    PostMessage (Application.MainForm.Handle, BWM_ACCOUNT, Account.AccountID, 0);
  end;

end;

procedure TAccountsList.BWMCust(var Message: TMessage);
var
  BWMCust : TBWMCust;
begin
  BWMCust :=  TBWMCust(Message);
  if (BWMCust.CustId = FCust.CustID) then
  begin
    FCust.AccountCollection.Load;
    LoadAccounts;
    UpdateAccountWindows;
  end;
end;

end.
