{
  Name    :   CustomerInfo.pas

  Purpose :   Classes for managing lists of Delphi Bank Customers

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit CustomerInfo;

interface

uses
  Classes,
  DelphiBankServer3_TLB,
  Account;

type
  TCustomer = class (TCollectionItem)
  private
    FCustID: Integer;
    FPassword: String;
    FLogonID: String;
    FCustNotes: String;
    FUserName: String;
    FAccountCollection: TAccountCollection;
  protected
    procedure SetAccountCollection(const Value: TAccountCollection); virtual;
  public
    constructor Create (Collection: TCollection); override;
    destructor Destroy; override;
    procedure Load;

    Property UserName : String read FUserName write FUserName;
    Property LogonID: String read FLogonID write FLogonID;
    Property Password: String read FPassword write FPassword;
    Property CustNotes: String read FCustNotes write FCustNotes;
    Property CustID: Integer read FCustID write FCustID;

    property AccountCollection : TAccountCollection read FAccountCollection write SetAccountCollection;
  end;

  TCustomerCollection = class(TCollection)
  end;

implementation

{ TCustomerInfo }

constructor TCustomer.Create;
begin
  inherited;
  AccountCollection := TAccountCollection.Create (TAccount)
end;


destructor TCustomer.Destroy;
begin
  AccountCollection.Free;
  inherited;
end;


procedure TCustomer.Load;
begin
  Assert (AccountCollection <> nil);
  AccountCollection.CustID := CustID;
  AccountCollection.Load;
end;

procedure TCustomer.SetAccountCollection(const Value: TAccountCollection);
begin
  FAccountCollection := Value;
end;


end.
