{
  Name    :   PayMoneyDialog.pas

  Purpose :   Handles user interaction for paying and depositing money into
               a Delphi Bank Account.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit PayMoneyDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActnList, StdCtrls,
  Account;

type
  TPayMoneyDlg = class(TForm)
    Pay: TButton;
    Cancel: TButton;
    Label2: TLabel;
    Label3: TLabel;
    DescriptionEdit: TEdit;
    AmountEdit: TEdit;
    ActionList1: TActionList;
    PayNow: TAction;
    procedure PayNowExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FAccount : TAccount;
    FPaying  : Boolean;
    procedure SetAccount(const Value: TAccount);
    { Private declarations }
  public
    { Public declarations }
    class function PayMoney (AAccount : TAccount) : boolean;
    class function DepositMoney (AAccount : TAccount) : boolean;
    property Account : TAccount read FAccount write SetAccount;
  end;

implementation

uses
  ClientUtils;

{$R *.DFM}

class function TPayMoneyDlg.PayMoney(AAccount: TAccount): boolean;
var
  PayMoneyDlg: TPayMoneyDlg;
begin
  Result := False;

  PayMoneyDlg := TPayMoneyDlg.Create (Application);
  try
    PayMoneyDlg.Account := AAccount;
    PayMoneyDlg.ShowModal;
    if PayMoneyDlg.ModalResult = mrOK then
      Result := True;
  finally
    PayMoneyDlg.Free;
  end;
end;

class function TPayMoneyDlg.DepositMoney(AAccount: TAccount): boolean;
var
  PayMoneyDlg: TPayMoneyDlg;
begin
  Result := False;

  PayMoneyDlg := TPayMoneyDlg.Create (Application);
  try
    with PayMoneyDlg do begin
      Account     := AAccount;
      FPaying     := False;
      Pay.Caption := 'Deposit';
      Caption     := 'Deposit Money';
      ShowModal;
    end;
    if PayMoneyDlg.ModalResult = mrOK then
      Result := True;
  finally
    PayMoneyDlg.Free;
  end;
end;

procedure TPayMoneyDlg.PayNowExecute(Sender: TObject);
var
  StrDebug : WideString;
  AcctId : integer;
  Description : WideString;
  Amount : Currency;

begin
  AcctId := Account.AccountID;
  Description := DescriptionEdit.Text;

  { Paying or depositing? }
  if FPaying then
    Amount := 0 - StrToCurr (AmountEdit.Text)
  else
    Amount := StrToCurr (AmountEdit.Text);

  if ClientServices.Accounts.AddTransaction(ClientServices.ClientKey,
                                            AcctId, Description, Amount,
                                            StrDebug) <> 0 then
    raise Exception.Create ('Pay Money: ' + StrDebug)
  else
    ModalResult := mrOk
end;

procedure TPayMoneyDlg.SetAccount(const Value: TAccount);
begin
  FAccount := Value;
end;

procedure TPayMoneyDlg.FormCreate(Sender: TObject);
begin
  FPaying     := True;
  Pay.Caption := 'Pay';
  Caption     := 'Pay Money';
end;

end.
