{
  Name    :   Transactions.pas

  Purpose :   Classes for managing lists of Delphi Bank Transactions

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit Transactions;

interface

uses
  Classes,
  Controls,
  DelphiBankServer3_TLB;

type

  TTransaction = class (TCollectionItem)
  private
    FAmount: Currency;
    FTransactionID: integer;
    FDescription: string;
    FTimeStamp: TDateTime;
    procedure SetAmount(const Value: Currency);
    procedure SetDescription(const Value: string);
    procedure SetTransactionID(const Value: integer);
    procedure SetTimeStamp(const Value: TDateTime);
  public
    property TransactionID : integer read FTransactionID write SetTransactionID;
    property Description : string read FDescription write SetDescription;
    property Amount : Currency read FAmount write SetAmount;
    property TimeStamp : TDateTime read FTimeStamp write SetTimeStamp;
  end;

  TTransactionCollection = class (TCollection)
  private
    FAccountID: integer;
    procedure SetAccountID(const Value: integer);
  protected
    procedure ReadArray (TheArray : OLEVariant);
  public
    procedure Load; virtual;
    property AccountID : integer read FAccountID write SetAccountID;
  end;

implementation

uses
  SysUtils,
  ClientUtils;

procedure TTransactionCollection.Load;
var
  MyArray : OLEVariant;
  StrDebug : WideString;
begin

  if ClientServices.Transactions.ListByAcctID(ClientServices.ClientKey,
                                             AccountId, MyArray, StrDebug) <> 0 then
    raise Exception.Create ('Problem reading account details: ' + StrDebug)
  else
    ReadArray (MyArray);
end;

procedure TTransactionCollection.ReadArray (TheArray : OLEVariant);
var
  i : integer;
  MaxElement : integer;
  ThisTransaction : TTransaction;

begin
  Clear; // remove list of transactions.

  MaxElement := VarArrayHighBound(TheArray, 2);

  // First row contains field names
  for i := 1 to MaxElement do
  begin
    ThisTransaction  := TTransaction (self.Add);
    ThisTransaction.TransactionID := TheArray [1,i];
    ThisTransaction.Description := TheArray [2,i];
    ThisTransaction.Amount := TheArray [3,i];
    ThisTransaction.TimeStamp := TheArray [4,i];
  end;
end;


procedure TTransactionCollection.SetAccountID(const Value: integer);
begin
  FAccountID := Value;
end;

{ TTransaction }
procedure TTransaction.SetAmount(const Value: Currency);
begin
  FAmount := Value;
end;

procedure TTransaction.SetDescription(const Value: string);
begin
  FDescription := Value;
end;

procedure TTransaction.SetTimeStamp(const Value: TDateTime);
begin
  FTimeStamp := Value;
end;

procedure TTransaction.SetTransactionID(const Value: integer);
begin
  FTransactionID := Value;
end;

end.

