{
  Name    :   TransactionsFrm.pas

  Purpose :   Handles user interaction for Delphi Bank Transactions on an Account.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit TransactionsFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ToolWin,
  Account,
  ActnList,
  Transactions,
  BankWinMessages;


type
  TTransactionsForm = class(TForm)
    TransactionsListViewer: TListView;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    TransactionActions: TActionList;
    PayMoney: TAction;
    ToolButton2: TToolButton;
    DepositMoney: TAction;
    procedure PayMoneyExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DepositMoneyExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FTransactionCollection: TTransactionCollection;
    FAccount: TAccount;
    FCustID: integer;
    FCustName: string;
    procedure SetTransactionCollection(
      const Value: TTransactionCollection);
    procedure SetAccount(const Value: TAccount);
    procedure ShowTxs;
    procedure UpdateCustomer;
    procedure SetCustID(const Value: integer);
    procedure SetCustName(const Value: string);
    { Private declarations }
  protected
    procedure BWMAccount (var Message : TMessage); message BWM_ACCOUNT;
  public
    { Public declarations }
    class procedure ShowAccount (AAccount : TAccount;
                              CustName : string;
                              CustID : integer);

    constructor Create (AOwner: TComponent;
                        AAccount : TAccount;
                        ACustName : string;
                        ACustID : integer); reintroduce;
    property TransactionCollection : TTransactionCollection read FTransactionCollection write SetTransactionCollection;
    property Account : TAccount read FAccount write SetAccount;
    property CustName : string read FCustName write SetCustName;
    property CustID : integer read FCustID write SetCustID;

  end;


implementation

uses PayMoneyDialog, ClientUtils;

{$R *.DFM}

{ TTransactionsForm }

procedure TTransactionsForm.ShowTxs;
var
  i : integer;
  ListItem : TListItem;
  Transaction : TTransaction;
begin
  TransactionsListViewer.Items.BeginUpdate;
  try
    TransactionsListViewer.Items.Clear;

    for i := 0 to TransactionCollection.Count -1 do
    begin
      ListItem := TransactionsListViewer.Items.Add;
      Transaction := TTransaction (TransactionCollection.Items[i]);
      ListItem.Caption := DateTimeToStr (Transaction.TimeStamp);
      ListItem.Data := Transaction;
      ListItem.SubItems.Add (Transaction.Description);
      ListItem.SubItems.Add (Format ('%f', [Transaction.Amount]));
    end;

  finally
    TransactionsListViewer.Items.EndUpdate;
  end;

end;


procedure TTransactionsForm.SetTransactionCollection(
  const Value: TTransactionCollection);
begin
  FTransactionCollection := Value;
  ShowTxs;
end;

procedure TTransactionsForm.SetAccount(const Value: TAccount);
begin
  FAccount := Value;
end;

class procedure TTransactionsForm.ShowAccount(AAccount: TAccount;
                              CustName : string;
                              CustID : integer);
var
  TransactionsForm: TTransactionsForm;
begin
  TransactionsForm := TTransactionsForm.Create (Application, AAccount, CustName, CustID);
  TransactionsForm.TransactionCollection := AAccount.TransactionCollection;
  TransactionsForm.Visible := True;
end;

procedure TTransactionsForm.FormShow(Sender: TObject);
begin
  Assert (Account <> nil);
  Caption := CustName + ' ' + ClientServices.GetAccountType(Account.AccountType).AccountName;
end;

procedure TTransactionsForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := True;
end;

procedure TTransactionsForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TTransactionsForm.PayMoneyExecute(Sender: TObject);
begin
  if TPayMoneyDlg.PayMoney (Account) then
  begin
    TransactionCollection.Load;
    ShowTxs;
  end;
  UpdateCustomer;
end;

procedure TTransactionsForm.DepositMoneyExecute(Sender: TObject);
begin
  if TPayMoneyDlg.DepositMoney (Account) then
  begin
    TransactionCollection.Load;
    ShowTxs;
  end;
  UpdateCustomer;
end;


procedure TTransactionsForm.UpdateCustomer;
begin
  PostMessage (Application.MainForm.Handle, BWM_CUST,  CustID, 0);
end;

procedure TTransactionsForm.BWMAccount(var Message: TMessage);
var
  BWMAccount : TBWMAccount;
begin
  BWMAccount := TBWMAccount (Message);
  if (BWMAccount.AccountID = Account.AccountID) then
  begin
    TransactionCollection.Load;
    ShowTxs;
  end;
end;

procedure TTransactionsForm.SetCustID(const Value: integer);
begin
  FCustID := Value;
end;

procedure TTransactionsForm.SetCustName(const Value: string);
begin
  FCustName := Value;
end;


constructor TTransactionsForm.Create(AOwner: TComponent;
  AAccount: TAccount; ACustName: string; ACustID: integer);
begin
  inherited Create (AOwner);
  Account := AAccount;
  CustName := ACustName;
  CustID := ACustID;
end;

end.
