{
  Name    :   TransferMoneyDialog.pas

  Purpose :   Handles user interaction for money transfer between two Delphi
               Bank Accounts.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, September 1998. paul@c-s-c.demon.co.uk
              (Based on code kindly donated by Mark Smith)
}
unit TransferMoneyDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  CustomerInfo, ActnList;

type
  TTxMoneyDialog = class(TForm)
    OK: TButton;
    Cancel: TButton;
    FromActList: TListBox;
    ToActList: TListBox;
    Label1: TLabel;
    DescriptionEdit: TEdit;
    AmountEdit: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    ActionList1: TActionList;
    Transfer: TAction;
    procedure TransferUpdate(Sender: TObject);
    procedure TransferExecute(Sender: TObject);
  private
    { Private declarations }
    FCust : TCustomer;
  protected
    procedure Populate;
  public
    { Public declarations }
    class function TransferMoney(Cust: TCustomer) : boolean;
  end;


implementation

uses
  ClientUtils,
  Account,
  DelphiBankServer3_TLB;

{$R *.DFM}

{ TTxMoneyDialog }

class function TTxMoneyDialog.TransferMoney(Cust: TCustomer) : boolean;
var
  TxDialog : TTxMoneyDialog;
begin
  TxDialog := TTxMoneyDialog.Create (Application);
  try
    TxDialog.FCust := Cust;
    TxDialog.Populate;
    TxDialog.ShowModal;

    Result := ( TxDialog.ModalResult = mrOK );

  finally
    TxDialog.Free;
  end;
end;

procedure TTxMoneyDialog.Populate;
var
  i : integer;
  ThisAccount : TAccount;

  function BuildDisplayName : string;
  begin
    result := ClientServices.AccountType[ThisAccount.AccountType].AccountName +
      ' (' + CurrencyString + Format ('%f', [ThisAccount.CurrentBalance]) + ')';
  end;

begin
  FromActList.Clear;
  for i := 0 to FCust.AccountCollection.Count -1 do
  begin
    ThisAccount := TAccount (FCust.AccountCollection.items[i]);
    FromActList.Items.AddObject (BuildDisplayName, ThisAccount);
  end;

  ToActList.Items.Assign (FromActList.Items);

  FromActList.ItemIndex := 0;
  ToActList.ItemIndex := 1;
end;

procedure TTxMoneyDialog.TransferUpdate(Sender: TObject);
begin
  Transfer.Enabled :=
    (FromActList.ItemIndex <> ToActList.ItemIndex) and
    (DescriptionEdit.Text <> '') and
    (AmountEdit.text <> '');
end;

procedure TTxMoneyDialog.TransferExecute(Sender: TObject);
var
  StrDebug : WideString;
  SourceAcctId : integer;
  DestAcctId : integer;
  Description : WideString;
  Amount : Currency;

  function GetAcctID ( ListBox : TListBox) : integer;
  var
    Account : TAccount;
  begin
    Account := TAccount ( ListBox.items.Objects[ListBox.ItemIndex]);
    Result := Account.AccountID;
  end;

begin
  SourceAcctId := GetAcctID (FromActList);
  DestAcctId := GetAcctID (ToActList);
  Description := DescriptionEdit.Text;
  Amount := StrToCurr (AmountEdit.Text);

  if ClientServices.Accounts.Transfer(ClientServices.ClientKey,
                                      SourceAcctId, DestAcctId,
                                      Description, Amount,
                                      False, StrDebug) <> 0 then
    raise Exception.Create ('Transfer money: ' + StrDebug)
  else
    ModalResult := mrOk
end;

//getobjectcontext

end.

