unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, ToolWin, ComCtrls, xwModExpt, ImgList;

type
  TMainForm = class(TForm)
    mmMain: TMainMenu;
    mmiFile: TMenuItem;
    mmiExit: TMenuItem;
    mmiHelp: TMenuItem;
    mmiAbout: TMenuItem;
    mmiHowHelp: TMenuItem;
    mmiContents: TMenuItem;
    cbMain: TCoolBar;
    tbShell: TToolBar;
    tbcMain: TTabControl;
    stsbrMain: TStatusBar;
    pnlMain: TPanel;
    mmiViewSelect: TMenuItem;
    mmiActions: TMenuItem;
    tbtn1: TToolButton;
    tbtn2: TToolButton;
    imMain: TImageList;
    pnlModule: TPanel;
    pnlViews: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tbcMainChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mmiExitClick(Sender: TObject);
    procedure mmiAboutClick(Sender: TObject);
  private
    { Private declarations }
    FModuleManager: TXWModuleManager;

    procedure OpenView(const AViewNum: Integer);
    {: After load module event handler }
    procedure AfterLoadModule(Sender: TObject);
    {: Before Unload module event handler }
    procedure BeforeUnloadModule(Sender: TObject);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  { Create the FModuleManager and assign to it's properties the menus and panels that will
    house the module and view level menus and toolbars. }
  FModuleManager := TXWModuleManager.Create;
  FModuleManager.ModuleLevelMenuParent := mmiViewSelect;
  FModuleManager.ModuleLevelTBParent   := pnlModule;

  FModuleManager.ViewParentWindow := pnlMain;
  FModuleManager.ViewMenu := mmiActions;
  FModuleManager.ViewTBParent := pnlViews;


  // Merge the Module Manager's menu
  mmMain.Merge(FModuleManager.ModuleManagerMenu);
  // Hook up the event handlers for After and Before loading modules
  FModuleManager.AfterLoadModule := AfterLoadModule;
  FModuleManager.BeforeUnloadModule := BeforeUnloadModule;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  // Unmerge the Module Manager's menu and free it.
  mmMain.UnMerge(FModuleManager.ModuleMainMenu);
  FModuleManager.Free;
end;

procedure TMainForm.OpenView(const AViewNum: Integer);
begin
  { When a new view (Form) is loaded, set it's parent window to that of the Module
   Manager's parent window. }
  if FModuleManager.CurrentModule <> nil then
    FModuleManager.OpenView(AViewNum, True);
end;

procedure TMainForm.tbcMainChange(Sender: TObject);
begin
  OpenView(tbcMain.TabIndex)
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FModuleManager.UnloadModule;
end;

procedure TMainForm.AfterLoadModule(Sender: TObject);
begin
  { When a module is loaded, retrieve its caption and create its tabs
    on the TTabControl component based on its captions. Also, get it's icon }
  Caption := FModuleManager.CurrentModule.ModuleDescripton;
  tbcMain.Tabs.Assign(FModuleManager.CurrentModule.ViewCaptions);
  // Invoke the call to open the first view.
  tbcMainChange(nil);
  Icon := FModuleManager.CurrentModule.ModuleIcon;
end;

procedure TMainForm.BeforeUnloadModule(Sender: TObject);
var
  M: TMemoryBasicInformation;
  i: integer;
begin
  { Before unloading a module, we must free any hanging component provided by the
    module. }
  if FModuleManager.CurrentModule <> nil then
  begin
    for i := Application.ComponentCount - 1 downto 0 do
    begin
      VirtualQuery(GetClass(Application.Components[i].ClassName), M, SizeOf(M));
      if (FModuleManager.CurrentModuleHandle = 0) or
         (HMODULE(M.AllocationBase) = FModuleManager.CurrentModuleHandle) then
         Application.Components[i].Free;
    end;
    Caption := 'Xapware Shell Application';
    tbcMain.Tabs.Clear;
    Icon := Application.Icon;
  end;
end;

procedure TMainForm.mmiExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.mmiAboutClick(Sender: TObject);
begin
  cbMain.Bands[2].Control := nil;
end;

end.
