unit GradBtn;
{This component is freeware.  Use, abuse it, modify it, improve it, take out
 parts you don't need/want.  You don't need to credit the author if you use
 this in an application.  However, you can communicate with me at
 harmans@uswest.net }

{Version 1.0 11/97 Initial release to the Public
 Version 1.1 12/97 Added Presets.  Add KeyPress to allow Enter key to act as
              Click. Also added btnFrameSize property, but I don't like it much
	     'cause it overlays the gradient, and I'm too lazy to figure
             out a solution.
 Version 1.2 01/98
             Tony BenBrahim (startech@neosoft.com) made a VAST improvement by
             adding drawing and handling for accelerator keys(&).  Thanks, Tony!
             Fixed bug - made tsRaised the default TextStyle.
             Added code to indicate if button has Focus.
             Added two events - OnMouseEnter and OnMouseLeave.
             Cleaned up code a little.
 Version 1.3 07/98
             Added Justification property for alignment of caption, per a
             suggestion from Yves MailHot. Thanks!
 Version 1.4 02/99
             Added code to Re-Draw Button back to its original state if
             mouse is down and moved away, to more closely resemble a typical
             button behavior.  Suggestion and code from Andy Daxer, Munich.
             Also modified the code for the Elliptic style of gradient to use
             integer math instaed of real numbers, greatly improving the speed
             of redraw.  The code was contributed by Dr. Allan Harkness, from
             Paisley, Scotland.  Thanks a great deal to you both, this component
             keeps getting better all the time, with insightful and generous
             help from people like you. Dr. Harkin has also shared other
             improvements as follows:  When button is disabled, it looks that
             way.  Drop-down menu support, you can associate a popup menu with
             the button (the little triangle thingy).  A new preset called
             Brassed Off.  NOTE - Drop down menu support only available with D4.
             I also changed the code so that the button is always re-drawn on
             a click.  There was a bug where if a user program error occured
             in the OnClick code, the button was left in a 'down' state.  I
             _think_ I fixed this...}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, menus;

{I learned the basic gradient technique from a component by John Baumbach.
 I added several more types of gradient and 3D text.  His component was
 derived from a SpeedButton.  I found his component at the Delphi Deli,
 (www.delphideli.com), a great place on the Web.  There are a lot of
 comments in here, which kind of trashes up the code, but I wanted to
 make this understandable to just about anybody, so have tried to explain
 everything that's going on.}

{These are the six different gradient styles available.}
type
  TGradientStyle = (gsHorizontal, gsVertical, gsElliptic, gsRectangle,
                    gsVertCenter, gsHorizCenter);

{Text can be normal, or 3D raised/inset}
type
  TTextStyle = (tsNormal, tsRaised, tsInset);

{ShadeType refers to the 3D text effect.}
type
  TShadeType = (stLight, stHeavy);

{Added some presets to make things easier}
type
  TPreset = (None, GoldCoast, LimeBox, Aluminum, SunRise, BrassedOff);

{Added justification for caption.}
type
  TJustification = (jLeft, jCenter, jRight);

type
  TGradBtn = class(TCustomControl)
  private
    { Private declarations }
    {$IFDEF VER120}
    FMenuPoppedUp : Boolean;  //Added AH 13/2/99
    FDropdownMenu: TPopupMenu;//Added AH 13/2/99
    FMarkerOffset : Integer;  //Added AH 13/2/99
    {$ENDIF}
    FDisabledColor : TColor;  //Added AH 13/2/99 Color when control is disabled
    FBtnHiliteClr : TColor;   //Hilite around button
    FBtnShadowClr : TColor;   //Shadow around button
    FBtnFrameSize : Integer;  //Size of 3D Frame
    FTxtHiliteClr : TColor;  //Text Hilite
    FTxtShadowClr : TColor;  //Text Shadow
    FBeginClr : TColor;    //Start color for gradient
    FEndClr : TColor;     //End color for gradient
    FSwapClr : Boolean;    //Swap start-end colors on button press?
    FShowFocus : Boolean;  //Show indicator rectangle when we have focus?
    FGradientStyle : TGradientStyle;  //One of six choices
    FTextStyle : TTextStyle;  //Raised, Inset, or None
    FShadeType : TShadeType; //Light or heavy text shading
    FOnMouseEnter: TNotifyEvent;  //Make the Mouse Enter and Leave available
    FOnMouseLeave: TNotifyEvent;
    FJustify : TJustification;   //Allow for left/center/right justify
    mUp : Boolean;    //Flags for Mouse Up or Down
    Clicking : Boolean;   //To catch a real button click...
    bm : TBitmap;          //WorkHorse internal bmp
    TmpClr : TColor;       //Holding area for Color during Mouse Down.
    FPreset : TPreset;   //Variable for presets
    Caption2: string; //ABB caption w/out ampersand!
    function Muldv(a,b,c : integer) : longint;
    procedure WMSetFocus(Var Msg : TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(Var Msg : TWMKillFocus); message WM_KILLFOCUS;
    //I had to add this to get the Caption to change while designing
    procedure CMMouseEnter(var Msg:TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg:TMessage); message CM_MOUSELEAVE;
    //Processing these messages allows events to fire when mouse enters or leaves
    procedure CMTextChanged(var Msg:TMessage); message CM_TEXTCHANGED;
    {$IFNDEF VER120}
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    {$ENDIF}
    procedure WMEraseBkgnd(Var Msg : TMessage); message WM_ERASEBKGND;
    //I added this to help reduce flicker and speed things up.
    procedure KeyAccel(var Message: TCMDialogChar);message CM_DIALOGCHAR;
    procedure SetDisabledColor(Value: TColor); //Added AH 13/2/99 Set color when control is disabled
    procedure SetBtnHilite(Value: TColor);  //Set color of top and left of frame
    procedure SetBtnShadow(Value: TColor);  //Set Color of right and bottom of frame
    procedure SetBtnFrameSize(Value: Integer); //Set size of 3D frame
    procedure SetTxtHilite(Value: TColor);  //Set color for text 3D effect hilite
    procedure SetTxtShadow(Value: TColor);  //Set color for text 3D effect shadow
    procedure SetBeginClr(Value : TColor);  //Set start color of gradient
    procedure SetEndClr(Value : TColor);    //Set end color of gradient
    procedure SetSwap(Value : Boolean);  //Swap the start-end true-false
    procedure SetShowFocus(Value : Boolean);  //Draw Rect when we have focus T/F
    procedure SetStyle(Value: TTextStyle);  //Set Caption 3D style
    procedure SetShade(Value: TShadeType);  //Set Caption shading type
    procedure SetPreset(Value: TPreset);   //Set preset values
    procedure DoRaise(offset: Integer);   //This draws the 3D Caption 'raised'
    procedure DoInset(offset: Integer);  //This draws te 3D caption 'inset'
    procedure DoNorm(offset: Integer);   //This draws the caption with no 3D
    procedure SetGradient(Value : TGradientStyle);  //Choose gradient fill style
    procedure SetJustify(Value : TJustification);   //Select caption justification
    procedure DoHorizontal(fr, fg, fb, dr, dg, db : Integer); //These all draw
    procedure DoVertical(fr, fg, fb, dr, dg, db : Integer); // the gradients.
    procedure DoElliptic(fr, fg, fb, dr, dg, db : Integer); // The fr fg fb etc
    procedure DoRectangle(fr, fg, fb, dr, dg, db : Integer); // are color values.
    procedure DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
    procedure DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
    {$IFDEF VER120}
    procedure SetDropdownMenu (Value: TPopupMenu); //Added AH 13/2/99
    procedure DrawPopupMark;			   //Added AH 13/2/99
    {$ENDIF}
  protected
    { Protected declarations }
    {I added the following to allow the Enter key to behave as a Click.}
    procedure KeyPress(var Key: Char); override;
    {$IFDEF VER120}
    procedure Notification (AComponent: TComponent; Operation: TOperation); override; //Added AH 13/2/99
    {$ENDIF}
    procedure SetEnabled(value: Boolean); {$IFDEF VER120}override;{$ENDIF} //Added AH 13/2/99
  public
    { Public declarations }
    {Almost all components override the constructor, so that default values for
     new properties can be set.  The destructor is usually overridden so cleanup
     can be done - like releasing resources so they don't cause memory leaks.}
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;  //Override Paint - entire control is painted.
    procedure MouseDown(Button:TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState;
      X, Y: Integer); override;
    //Mouse up and down are what I use to swap gradient colors
    procedure Click; override;
  published
    { Published declarations }
    {This first batch of stuff is all the new properties I added for this
     component.  As long as the properties are part of the usual types, you
     don't have to define your own property editor. I love Delphi.}
    property DisabledColor : TColor read FDisabledColor
       write SetDisabledColor default clGrayText; // Added by AH 13/2/99
    property BtnHiliteClr : TColor read FBtnHiliteClr
       write SetBtnHilite default clBtnHighLight;
    property BtnShadowClr : TColor read FBtnShadowClr
       write SetBtnShadow default clBtnShadow;
    property BtnFrameSize : Integer read FBtnFrameSize
       write SetBtnFrameSize default 1;
    property TxtHiliteClr : TColor read FTxtHiliteClr
       write SetTxtHilite default clBtnHighLight;
    property TxtShadowClr : TColor read FTxtShadowClr
       write SetTxtShadow default clBtnShadow;
    property BeginColor : TColor read FBeginClr
       write SetBeginClr default clNavy;
    property EndColor : TColor read FEndClr
       write SetEndClr default clAqua;
    property SwapColors : Boolean read FSwapClr
       write SetSwap default TRUE;
    property ShowFocus : Boolean read FShowFocus
       write SetShowFocus default TRUE;
    property GradientStyle : TGradientStyle read FGradientStyle
       write SetGradient default gsVertCenter;
    property TextStyle : TTextStyle read FTextStyle
       write SetStyle default tsRaised;
    property TextShadeType : TShadeType read FShadeType
       write SetShade default stLight;
    property Presets : TPreset read FPreset
       write SetPreset default None;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter
        write FOnMouseEnter;   //These show up in events
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave
        write FOnMouseLeave;
    property Justification : TJustification read FJustify
       write SetJustify default jCenter;
    {$IFDEF VER120}
    property DropdownMenu: TPopupMenu read FDropdownMenu write SetDropdownMenu; // Added by AH 13/2/99
    {$ENDIF}
    property Caption;      //Added these so they show up in the Obj Insp.
    property OnClick;      // When deriving from a Custom type control,
    property OnMouseDown;  // these don't normally show up unless you specifically
    property OnMouseUp;    // declare them yourself.  You can find them by
    property Visible;      // going through the browser.
    property Enabled;
    property Font;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseMove;
  end;

procedure Register;

implementation

constructor TGradBtn.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  {$IFDEF VER120}
  FMenuPoppedUp := False; // Added by AH 13/2/99 
  FMarkerOffset := 0; // Added by AH 13/2/99 offset if drop down menu used
  {$ENDIF}
  FDisabledColor := clGrayText;   // Added by AH 13/2/99 Default disabled Color
  FBtnHiliteClr := clBtnHighLight;   //Default Hilite Color
  FBtnShadowClr := clBtnShadow;      //Default Shadow Color
  FBtnFrameSize := 1;                //Default frame size
  FTxtHiliteClr := clBtnHighLight;   //Default Text Hilite
  FTxtShadowClr := clBtnShadow;      //Default Text Shadow
  FBeginClr := clNavy;            //Start Color is at the Top of button
  FEndClr := clAqua;             //Stop Color is at the Bottom of button
  Height := 25;                   //Default button size
  Width := 75;
  bm := TBitmap.Create;          //Create the internal BMP
  bm.Canvas.Pen.Style:= psSolid; //These may already be the default,
  bm.Canvas.Pen.Mode:= pmCopy;   // but just in case...
  mUp := TRUE;                   //Mouse is UP to start with
  Font.Color := clBlack;         //Default Font Color
  ControlStyle := ControlStyle + [csOpaque];   //This reduces flicker
  FSwapClr := TRUE;               //By default, we swap colors on Mouse Down
  FShowFocus := TRUE;             //Draw focusrect when we have focus
  FShadeType := stLight;          //Light text shading by default
  FTextStyle := tsRaised;         //Raised '3d' text
  FGradientStyle := gsVertCenter;   //Top & Bottom to Center for gradient Fill default
  FPreset := None;
  TabStop := TRUE;      //Made this the default so button can have Focus
  FJustify := jCenter;  //Default is center justify
end;

function TGradBtn.Muldv(a,b,c : integer) : longint;
ASM

  MOV EAX, a
  IMUL b
  IDIV c

end;


{Everything pretty much happens in the Paint procedure; a lot of it is farmed
 out to other procedures just so the logic in the paint procedure can be
 understood a little better.  Here's the basic process for creating a gradient:
 Get the separate R, G, and B values of the BeginClr Color.  Then, find the
 difference between that and the R, G, and B values of the EndClr Color.  Using
 these two sets of values, draw a series of 'shapes' using a series of colors
 between the begin and end.  After the gradient is drawn, add a frame and the
 Caption.}
procedure TGradBtn.Paint;
var
  FromR, FromG, FromB : Integer; //These are the separate color values for RGB
  DiffR, DiffG, DiffB : Integer; // of color values.
  rct : TRect;                   //Rectangle used to draw frame around button
  offset : Integer;              //Used for Caption location during Mouse Down
  i: integer; 			 //loop variable to iterate Caption for &. ABB
{To speed things up and reduce flicker, I use a Bitmap to draw the button in
 its entirety, then BitBlt it to the canvas of the control.}
begin
  //Harm removed this 2/15/99
  //inherited;                  // PROBABLY DON'T NEED THIS
  bm.Canvas.Font.Color := Font.Color; //Keep Bitmap in synch with settings.
  bm.Canvas.Font := Font;     // Set the Bitmap font to match control font
  bm.Width := Width;          //Set BMP dimensions to match control's
  bm.Height := Height;
  rct := Rect(0,0,Width,Height);  //Set rectangle size for later use
  FromR := FBeginClr and $000000ff;  //Strip out separate RGB values
  FromG := (FBeginClr shr 8) and $000000ff;
  FromB := (FBeginClr shr 16) and $000000ff;
  DiffR := (FEndClr and $000000ff) - FromR;   //Find the difference
  DiffG := ((FEndClr shr 8) and $000000ff) - FromG;
  DiffB := ((FEndClr shr 16) and $000000ff) - FromB;
  //Depending on gradient style selected, go draw it on the Bitmap canvas.
  case FGradientStyle of
    gsHorizontal  : DoHorizontal(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertical    : DoVertical(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsElliptic    : DoElliptic(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsRectangle   : DoRectangle(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertCenter  : DoVertCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsHorizCenter : DoHorizCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  end;

  // Added by AH 13/2/99
  {$IFDEF VER120}
  if Assigned (DropDownMenu) then DrawPopupMark;
  {$ENDIF}

  //By setting the Brush style to Clear, it will draw without overlaying bkgrnd
  bm.Canvas.Brush.Style := bsClear;  //Gradient is done, time for Hilite-Shadow
  if mUp = TRUE then begin       //If Mouse up do normal button
    offset := 0;  //offset is used to move the Caption down a notch when the
                  // button is pressed.
    Frame3D(bm.Canvas,rct,FBtnHiliteClr,FBtnShadowClr,FBtnFrameSize); //Draw frame
  end
  else begin
    offset := 1; //If mouse down, we want caption to 'move' and reverse the frame
    Frame3D(bm.Canvas,rct,FBtnShadowClr,FBtnHiliteClr,FBtnFrameSize);
  end;
  //Added by ABB
  Caption2:='';
  if Length(Caption) > 0 then begin
    for i:=1 to Length(Caption)-1 do if (Caption[i]<>'&') or (Caption[i+1]='&') then Caption2:=Caption2+Caption[i];
    Caption2:=Caption2+Caption[Length(Caption)];
  end;
  //End ABB

  // Added by AH 13/2/99
  {$IFDEF VER120}
  if Assigned (DropDownMenu) then
    FMarkerOffset := -6
  else
    FMarkerOffset := 0;
  {$ENDIF}

  // disabled button is inset - Altered by AH 13/2/99
  if not Enabled then DoInset(offset) 
  else
    Case FTextStyle of
      tsInset  : DoInset(offset);
      tsRaised :DoRaise(offset);
      tsNormal :DoNorm(offset);
    end;

  //If we have the Focus, draw a little rectangular indicator.
  if not (csDesigning in ComponentState) then
    if FShowFocus = TRUE then
      if Focused then
        bm.Canvas.DrawFocusRect(RECT(2,  2, Width - 2, Height - 2));
  {Finally, the button is all painted on the bitmap canvas.  Now we just need
   to copy it to the canvas of our control.  BitBlt is one method; there are
   several others.}
  BitBlt(Canvas.Handle,0,0,Width,Height,bm.Canvas.Handle,0,0,SRCCOPY);

  // Added by AH 13/2/99
  {$IFDEF VER120}
  if mUp then FMenuPoppedUp := False; // if button now up, menu must have disappeared
  {$ENDIF}
end;

//Added handler for accelerator key
procedure TGradBtn.KeyAccel(var Message: TCMDialogChar);
begin
  with Message do begin
    if IsAccel(CharCode, Caption) and Enabled then begin
      Click;
      Result := 1;
    end
    else
     inherited;
  end;
end;

procedure TGradBtn.SetDisabledColor(Value: TColor);
// Added by AH 13/2/99
begin
  if FDisabledColor <> Value then begin
    FDisabledColor := Value;
    Invalidate;  //Invalidate tells windows to do a Paint.  Our control
  end;           // goes through its Paint procedure.
end;

procedure TGradBtn.SetBtnHilite(Value: TColor);
begin
  if FBtnHiliteClr <> Value then begin
    FBtnHiliteClr := Value;
    Invalidate;  //Invalidate tells windows to do a Paint.  Our control
  end;           // goes through its Paint procedure.
end;

procedure TGradBtn.SetBtnShadow(Value: TColor);
begin
  if FBtnShadowClr <> Value then begin
    FBtnShadowClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetBtnFrameSize(Value: Integer);
begin
  if FBtnFrameSize <> Value then begin
    if Value > ((Width Div 2) - 5) then exit;
    if Value > ((Height Div 2) - 5) then exit;
    FBtnFrameSize := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetTxtHilite(Value: TColor);
begin
  if FTxtHiliteClr <> Value then begin
    FTxtHiliteClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetTxtShadow(Value: TColor);
begin
  if FTxtShadowClr <> Value then begin
    FTxtShadowClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetBeginClr(Value : TColor);
begin
  if FBeginClr <> Value then begin
    FBeginClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetEndClr(Value : TColor);
begin
  if FEndClr <> Value then begin
    FEndClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetSwap(Value : Boolean);
begin
  if FSwapClr <> Value then
    FSwapClr := Value;
end;

procedure TGradBtn.SetShowFocus(Value : Boolean);
begin
  if FShowFocus <> Value then
    FShowFocus := Value;
end;

procedure TGradBtn.MouseDown(Button:TMouseButton; Shift:TShiftState;
      X, Y: Integer);
{$IFDEF VER120}
var
  PopupPoint : TPoint;
{$ENDIF}
begin
  inherited;
  if not mUp then exit; // don't do more than once
  if Button = mbLeft then mUp := FALSE;
  SetFocus;
  if FSwapClr = TRUE then begin    //If SwapClrs is true, we need to save
    TmpClr := FBeginClr;           // the original stuff.  Hence, TmpClr
    FBeginClr := FEndClr;
    FEndClr := TmpClr;
  end;
  Invalidate;
  {$IFDEF VER120}
  if (not FMenuPoppedUp) and Assigned (DropDownMenu) and (Button = mbLeft) then
  begin
    PopupPoint := ClientToScreen(Point(0, Height));
    FMenuPoppedUp := True;
    with DropdownMenu do
    begin
      PopupComponent := Self;
      TrackButton := tbLeftButton;
      Popup (PopupPoint.X, PopupPoint.Y);
    end;
    MouseUp (Button, Shift, X, Y);
    ControlState := ControlState - [csLButtonDown];
end;
  {$ENDIF}
end;

procedure TGradBtn.MouseUp(Button:TMouseButton; Shift:TShiftState;
      X, Y: Integer);
begin
  inherited;
  if Clicking then begin
    Clicking := FALSE;
    exit;
  end;
  if mUp then exit; // don't do more than once
  if (FSwapClr = TRUE) and (mUp = FALSE) then begin
    TmpClr := FBeginClr;
    FBeginClr := FEndClr;
    FEndClr := TmpClr;
  end;
  mUp := TRUE;
  Invalidate;
end;

procedure TGradBtn.Click;   //Added this so I can re-draw button
begin
  Clicking := TRUE;
  if FSwapClr = TRUE then begin
    TmpClr := FBeginClr;
    FBeginClr := FEndClr;
    FEndClr := TmpClr;
  end;
  mUp := TRUE;
  Invalidate;
  Application.ProcessMessages;
  inherited;
end;

procedure TGradBtn.CMTextChanged(var Msg:TMessage);
begin
  Inherited;
  Invalidate;
end;

{$IFNDEF VER120}
procedure TGradBtn.CMEnabledChanged(var Message: TMessage);
begin
  Inherited;
  Invalidate;
end;
{$ENDIF}

procedure TGradBtn.WMEraseBkgnd(var Msg:TMessage);
begin
  Msg.Result := 0;  //We intercept this message and tell
end;                // Windows it processed OK.

procedure TGradBtn.WMSetFocus(var Msg:TWMSetFocus);
begin
  inherited;
  if csDesigning in ComponentState then exit;
  Invalidate;
end;

procedure TGradBtn.WMKillFocus(var Msg:TWMKillFocus);
begin
  inherited;
  if csDesigning in ComponentState then exit;
  Invalidate;
end;

procedure TGradBtn.SetGradient(Value : TGradientStyle);
begin
  if FGradientStyle <> Value then begin
    FGradientStyle := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetJustify(Value : TJustification);
begin
  if FJustify <> Value then begin
    FJustify := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetStyle(Value: TTextStyle);
begin
  {When user selects a style, set the variable to new style and do
   an invalidate to force a re-paint of the component.}
  if FTextStyle <> Value then begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetShade(Value: TShadeType);
begin
  {When user selects a shade, set the variable to new type and do
   an invalidate to force a re-paint of the component.}
  if FShadeType <> Value then begin
    FShadeType := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetPreset(Value: TPreset);
begin
 if csDesigning in ComponentState then begin
  case Value of
    None : begin
      FDisabledColor := clGray;
      FBeginClr := clNavy;
      FEndClr := clAqua;
      Font.Size := 8;
      Font.Name := 'MS Sans Serif';
      Font.Color := clBlack;
      Font.Style := [];
      FGradientStyle := gsVertCenter;
      FShadeType := stLight;
      FTextStyle := tsRaised;
      FTxtHiliteClr := clBtnHighLight;
      FTxtShadowClr := clBtnShadow;
      FBtnHiliteClr := clBtnHighlight;
      FBtnShadowClr := clBtnShadow;
    end;
    GoldCoast : begin
      FDisabledColor := $0000bbbb;
      FBeginClr := clYellow;
      FEndClr := clOlive;
      Font.Size := 10;
      Font.Name := 'Arial';
      Font.Color := $0000bbbb;
      Font.Style := [fsBold];
      FGradientStyle := gsVertical;
      FShadeType := stHeavy;
      FTextStyle := tsRaised;
      FTxtHiliteClr := $00bfffff;
      FTxtShadowClr := $00007171;
      FBtnHiliteClr := $00b3ffff;
      FBtnShadowClr := $00616161;
    end;
    LimeBox : begin
      FDisabledColor := clOlive;
      FBeginClr := clBlack;
      FEndClr := clLime;
      Font.Size := 8;
      Font.Name := 'Courier';
      Font.Color := clGreen;
      Font.Style := [];
      FGradientStyle := gsRectangle;
      FShadeType := stLight;
      FTextStyle := tsRaised;
      FTxtHiliteClr := $00acffac;
      FTxtShadowClr := $00004000;
      FBtnHiliteClr := clBtnHighLight;
      FBtnShadowClr := clBtnShadow;
    end;
    Aluminum : begin
      FDisabledColor := clGray;
      FBeginClr := $00cecece;
      FEndClr := $005f5f5f;
      Font.Size := 10;
      Font.Name := 'Times New Roman';
      Font.Color := clBlack;
      Font.Style := [fsBold];
      FGradientStyle := gsVertical;
      FShadeType := stHeavy;
      FTextStyle := tsInset;
      FTxtHiliteClr := clBtnHighlight;
      FTxtShadowClr := clBtnShadow;
      FBtnHiliteClr := clWhite;
      FBtnShadowClr := clBlack;
    end;
    SunRise : begin
      FDisabledColor := clGray;
      FBeginClr := clYellow;
      FEndClr := clRed;
      Font.Size := 8;
      Font.Name := 'MS Sans Serif';
      Font.Color := $00000040;
      Font.Style := [fsBold];
      FGradientStyle := gsHorizontal;
      FShadeType := stHeavy;
      FTextStyle := tsRaised;
      FTxtHiliteClr := $00cfe7e6;
      FTxtShadowClr := $000058b0;
      FBtnHiliteClr := clBtnHighlight;
      FBtnShadowClr := clBtnShadow;
    end;
    BrassedOff : begin
      FDisabledColor := clOlive;
      FBeginClr := $00CEFFFF;
      FEndClr := $0092C2C2;
      Font.Size := 9;
      Font.Name := 'Verdana';
      Font.Color := clMaroon;
      Font.Style := [];
      FGradientStyle := gsElliptic;
      FShadeType := stLight;
      FTextStyle := tsNormal;
      FSwapClr := False;
      FTxtHiliteClr := $00EFFFFF;
      FTxtShadowClr := $0092C2C2;
      FBtnHiliteClr := $00EFFFFF;
      FBtnShadowClr := $0092C2C2;
    end;
    else
      Exit;
  end;
  FPreset := Value;
  Invalidate;
 end;
end;

{The following two procedures process user code that can be specified in the
 events for the component.  These are often used to change the color when the
 mouse enters a control, or something like that.  I made these into events so
 the user can trigger whatever code they wish without modifying/creating a
 component.}
procedure TGradBtn.CMMouseEnter(var Msg:TMessage);
begin
  inherited;
  if csLButtonDown in ControlState then begin
    Self.MouseDown(mbLeft,[ssLeft],0,0); //Passed values are ignored
  end;
  if Assigned (FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TGradBtn.CMMouseLeave(var Msg:TMessage);
begin
  inherited;
  if csLButtonDown in ControlState then begin
    Self.MouseUp(mbLeft,[ssLeft],0,0);  //Passed values are ignored
  end;
  if Assigned (FonMouseLeave) then FOnMouseLeave(Self);
end;

{I'll explain a little about the Horizontal gradient, the other styles are all
 consistent with their logic.  The six R, G, and B values are passed to us.
 We define some local variables we'll need: a rectangle, a FOR loop counter,
 and our own RGB numbers.  For a horizontal gradient, we'll draw a series of
 rectangles, each one a little closer in color to the EndClr value.  A horizontal
 gradient rectangle will always be from the top to the bottom of the canvas,
 so we set top to 0 and bottom to however tall our control is.  Then, we draw
 a series of 255 rectangles.  The starting point and width of each will depend
 on the actual width of our control.  It starts out on the left, draws the
 first rectangle in a color that's a percentage of the difference plus the
 starting color.  As I increments through the loop, the rectangles move to the
 right and the color gets closer and closer to the EndClr.}
procedure TGradBtn.DoHorizontal(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Top:= 0;                //Set rectangle top
  ColorRect.Bottom := Height;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Left:= Muldv (I, Width, 256);    //Find left for this color
    ColorRect.Right:= Muldv (I + 1, Width, 256);   //Find Right
    R := fr + Muldv(I, dr, 255);    //Find the RGB values
    G := fg + Muldv(I, dg, 255);
    B := fb + Muldv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bm.Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TGradBtn.DoVertical(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Left:= 0;                //Set rectangle left&right
  ColorRect.Right:= Width;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Top:= Muldv (I, Height, 256);    //Find top for this color
    ColorRect.Bottom:= Muldv (I + 1, Height, 256);   //Find Bottom
    R := fr + Muldv(I, dr, 255);    //Find the RGB values
    G := fg + Muldv(I, dg, 255);
    B := fb + Muldv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bm.Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TGradBtn.DoElliptic(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph, Dw, Dh : integer;
  x1,y1,x2,y2 : integer;
{The elliptic is a bit different, since I had to use real numbers. I cut down
 on the number (to 155 instead of 255) of iterations in an attempt to speed
 things up, to no avail.  I think it just takes longer for windows to draw an
 ellipse as opposed to a rectangle.}
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := bm.Width div -3;
  x2 := bm.Width + (bm.Width div 3);
  y1 := bm.Height div -3;
  y2 := bm.Height + (bm.Height div 3);
  Pw := x2 - x1;
  Ph := y2 - y1;
  for I := 0 to 50 do begin         //Make ellipses of color
    R := fr + Muldv(I, dr, 50);    //Find the RGB values
    G := fg + Muldv(I, dg, 50);
    B := fb + Muldv(I, db, 50);
    bm.Canvas.Brush.Color := R or (G shl 8) or (b shl 16);   //Plug colors into brush
    Dw := Pw * i div 100;
    Dh := Ph * i div 100;
    bm.Canvas.Ellipse(x1 + Dw,y1 + Dh,x2 - Dw,y2 - Dh);
  end;
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TGradBtn.DoRectangle(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph : Real;
  x1,y1,x2,y2 : Real;
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := 0;
  x2 := bm.Width+2;
  y1 := 0;
  y2 := bm.Height+2;
  Pw := (bm.Width / 2) / 255;
  Ph := (bm.Height / 2) / 255;
  for I := 0 to 255 do begin         //Make rectangles of color
    x1 := x1 + Pw;
    x2 := X2 - Pw;
    y1 := y1 + Ph;
    y2 := y2 - Ph;
    R := fr + Muldv(I, dr, 255);    //Find the RGB values
    G := fg + Muldv(I, dg, 255);
    B := fb + Muldv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bm.Canvas.FillRect(Rect(Trunc(x1),Trunc(y1),Trunc(x2),Trunc(y2)));
  end;
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TGradBtn.DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := bm.Height Div 2;
  ColorRect.Left := 0;
  ColorRect.Right := bm.Width;
  for I := 0 to Haf do begin
    ColorRect.Top := Muldv (I, Haf, Haf);
    ColorRect.Bottom := Muldv (I + 1, Haf, Haf);
    R := fr + Muldv(I, dr, Haf);
    G := fg + Muldv(I, dg, Haf);
    B := fb + Muldv(I, db, Haf);
    bm.Canvas.Brush.Color := RGB(R, G, B);
    bm.Canvas.FillRect(ColorRect);
    ColorRect.Top := bm.Height - (Muldv (I, Haf, Haf));
    ColorRect.Bottom := bm.Height - (Muldv (I + 1, Haf, Haf));
    bm.Canvas.FillRect(ColorRect);
  end;
end;

procedure TGradBtn.DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := bm.Width Div 2;
  ColorRect.Top := 0;
  ColorRect.Bottom := bm.Height;
  for I := 0 to Haf do begin
    ColorRect.Left := Muldv (I, Haf, Haf);
    ColorRect.Right := Muldv (I + 1, Haf, Haf);
    R := fr + Muldv(I, dr, Haf);
    G := fg + Muldv(I, dg, Haf);
    B := fb + Muldv(I, db, Haf);
    bm.Canvas.Brush.Color := RGB(R, G, B);
    bm.Canvas.FillRect(ColorRect);
    ColorRect.Left := bm.Width - (Muldv (I, Haf, Haf));
    ColorRect.Right := bm.Width - (Muldv (I + 1, Haf, Haf));
    bm.Canvas.FillRect(ColorRect);
  end;
end;

procedure TGradBtn.DoRaise(offset: Integer);
{This draws 'raised' 3D text for the Caption.  Inset is the same principal,
 the colors are just reversed.  The brush style is set to Clear so the
 background is not erased.  The highlight colored text is drawn on the canvas
 at -1 up and left of center.  If heavy shading is set, the shadow colored
 text is drawn at +1 right and down of center.  Finally, the Font.Color colored
 text is drawn in exact center.  The offset parameter is used to 'move' the
 whole shebang down and right 1 if the Mouse is Down.}
var
  r: TRect;
  J : integer;    //For justification
begin
  bm.Canvas.Brush.Style := bsClear;
  bm.Canvas.Font.Color := FTxtHiliteClr;
  J := DT_CENTER;    //Initialize to avoid warnings
  case FJustify of
    jLeft   : begin
                J := DT_LEFT;
                r:=Rect(1+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,0,0);
              end;
    JCenter : begin
                J := DT_CENTER;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)-1+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,0,0);
              end;
    jRight  : begin
                J := DT_RIGHT;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)-1+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,0,0);
              end;
  end;

  DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,J+DT_TOP);
  if FShadeType = stHeavy then begin
    bm.Canvas.Font.Color := FTxtShadowClr;

    case FJustify of
      jLeft   : begin
                  J := DT_LEFT;
                  r:=Rect(3+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,0,0);
                end;
      jCenter : begin
                  J := DT_CENTER;
                  r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+1+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,0,0);
                end;
      jRight  : begin
                  J := DT_RIGHT;
                  r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 1)+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,0,0);
                end;
    end;

    DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+DT_LEFT+DT_TOP);
    DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,DT_CENTER+DT_TOP);
  end;
  bm.Canvas.Font.Color := Font.Color;
  case FJustify of
    jLeft   : begin
                J := DT_LEFT;
                r:=Rect(2+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
    JCenter : begin
                J := DT_CENTER;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
    jRight  : begin
                J := DT_RIGHT;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
  end;

  DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,J+DT_TOP);
  bm.Canvas.Brush.Style := bsSolid;
end;

procedure TGradBtn.DoInset(offset: Integer);
var
  r: TRect;
  J : integer;         //For justification
begin
  bm.Canvas.Brush.Style := bsClear;
  J := DT_CENTER;
  if FShadeType = stHeavy then begin
    bm.Canvas.Font.Color := FTxtShadowClr;

    case FJustify of
      jLeft   : begin
                  J := DT_LEFT;
                  r:=Rect(1+offset,
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,0,0);
                end;
      JCenter : begin
                  J := DT_CENTER;
                  r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)-1+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,0,0);
                end;
      jRight  : begin
                  J := DT_RIGHT;
                  r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)-1+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,0,0);
                end;
    end;

    DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+J+DT_TOP);
    DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,J+DT_TOP);
  end;

  bm.Canvas.Font.Color := FTxtHiliteClr;

  case FJustify of
    jLeft   : begin
                J := DT_LEFT;
                r:=Rect(3+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,0,0);
              end;
    jCenter : begin
                J := DT_CENTER;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+1+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,0,0);
              end;
    jRight  : begin
                J := DT_RIGHT;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 1)+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,0,0);
              end;
  end;

  DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,J+DT_TOP);

  // Added by AH 13/2/99
  if Enabled then
    bm.Canvas.Font.Color := Font.Color
  else
    bm.Canvas.Font.Color := FDisabledColor;

  case FJustify of
    jLeft   : begin
                J := DT_LEFT;
                r:=Rect(2+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
    JCenter : begin
                J := DT_CENTER;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
    jRight  : begin
                J := DT_RIGHT;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
  end;


  DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+J+DT_TOP);
  DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,J+DT_TOP);
  bm.Canvas.Brush.Style := bsSolid;
end;

procedure TGradBtn.DoNorm(offset: Integer);
var
  r: TRect;
  J : integer;  //For justification
begin           //If no 3D effect is set, we just draw text
  bm.Canvas.Brush.Style := bsClear;
  bm.Canvas.Font.Color := Font.Color;
  J := DT_CENTER;     //Initialized to avoid warnings
  case FJustify of
    jLeft   : begin
                J := DT_LEFT;
                r:=Rect(2+offset,
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
    JCenter : begin
                J := DT_CENTER;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) div 2)+offset{$IFDEF VER120}+FMarkerOffset{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
    jRight  : begin
                J := DT_RIGHT;
                r:=Rect(((bm.Width-bm.Canvas.TextWidth(Caption2)) - 2)+offset{$IFDEF VER120}+FMarkerOffset-6{$ENDIF},
                  ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,0,0);
              end;
  end;

    DrawText(bm.Canvas.Handle,PChar(Caption2),-1,r,DT_CALCRECT+J+DT_TOP);
    DrawText(bm.Canvas.Handle,PChar(Caption),-1,r,J+DT_TOP);
    bm.Canvas.Brush.Style := bsSolid;
end;

destructor TGradBtn.Destroy;
begin
  bm.Free;       //Cleanup the bitmap - no memory leaks
  inherited Destroy;
end;

{I added the following to allow the Enter key to behave as a Click.}
 {If return key was pressed, do a 'Click'.  This only happens if
  we have the 'Focus'.}
procedure TGradBtn.KeyPress(var Key: Char);
begin
  inherited;
  if Key = Chr(13) then Click;
end;

{$IFDEF VER120}
procedure TGradBtn.DrawPopupMark;
// Added by AH 13/2/99
var
  y : Integer;
begin
	y := (Height - 4) div 2;
  with bm.Canvas do
  begin
    Pen.Color := Font.Color;
    Brush.Color := Font.Color;
    Brush.Style := bsSolid;
    Polygon ([Point (Width - 14, y), Point (Width - 8, y), Point (Width - 11, y + 3)]);
  end;
end;

procedure TGradBtn.SetDropdownMenu (Value: TPopupMenu);
// Added by AH 13/2/99
begin
  if FDropdownMenu <> Value then
  begin
    FDropdownMenu := Value;
    if Assigned(Value) then Value.FreeNotification (Self);
    Invalidate;
  end;
end;

procedure TGradBtn.Notification (AComponent: TComponent; Operation: TOperation);
// Added by AH 13/2/99
begin
  inherited;
  if (Operation = opRemove) and (AComponent = DropdownMenu) then DropdownMenu := nil;
end;
{$ENDIF}

procedure TGradBtn.SetEnabled(value: Boolean);
// Added by AH 13/2/99
begin
  inherited;
  invalidate;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TGradBtn]);
end;

end.
