/////////////////////////////////////////////////////////////////////////
// Main unit of the IntsDemo project
//-----------------------------------------------------------------------
// Copyright (c) 1999, Marcel Martin. All rights reserved.
/////////////////////////////////////////////////////////////////////////
unit IntsDemoMain;

interface
{$A+,H+}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, Menus;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Memo1: TMemo;
    AbortBtn: TSpeedButton;
    ClearBtn: TSpeedButton;
    CloseBtn: TSpeedButton;
    HelpBtn: TSpeedButton;
    MenuBtn: TSpeedButton;
    PopupMenu1: TPopupMenu;
    PopItem0: TMenuItem;
    PopItem1: TMenuItem;
    PopItem2: TMenuItem;
    PopItem3: TMenuItem;
    PopItem4: TMenuItem;
    procedure AbortBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure MenuBtnClick(Sender: TObject);
    procedure PopItem0Click(Sender: TObject);
    procedure PopItem1Click(Sender: TObject);
    procedure PopItem2Click(Sender: TObject);
    procedure PopItem3Click(Sender: TObject);
    procedure PopItem4Click(Sender: TObject);
    procedure Run0(Sender: TObject);
    procedure Run1(Sender: TObject);
    procedure Run2(Sender: TObject);
    procedure Run3(Sender: TObject);
    procedure Run4(Sender: TObject);
    procedure Runx(Sender: TObject; Num: Integer);
  end;

var Form1: TForm1;


implementation
//=======================================================================
{$C-,I-,O+,Q-,R-,W-}
{$R *.DFM}
//-----------------------------------------------------------------------
uses Integers;

const
  CHECKMSG : array[Boolean] of string[5] = ('error','ok');
  CRLF = #13#10; //Carriage Return + Line Feed


/////////////////////////////////////////////////////////////////////////
// TForm1 class
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// Abort the running of some routines of Integers
//=======================================================================
procedure TForm1.AbortBtnClick(Sender: TObject);
begin
  IAbort;
end;

//=======================================================================
// Clear the editor
//=======================================================================
procedure TForm1.ClearBtnClick(Sender: TObject);
begin
  Memo1.Lines.Clear;
end;

//=======================================================================
// Terminate the application
//=======================================================================
procedure TForm1.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

//=======================================================================
// Create the form
//=======================================================================
procedure TForm1.FormCreate(Sender: TObject);
begin
  Application.HintColor := clYELLOW;
  //----- String format (Used by the procedure IStr)
  //IStrFormat.SpaceAsPlus := false; //'+' instead of a space
  //IStrFormat.LowerCased := false;  //For hexadecimal strings
end;

//=======================================================================
// Open the INTEGERS.HLP file
//=======================================================================
procedure TForm1.HelpBtnClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;


/////////////////////////////////////////////////////////////////////////
// Example #0
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// Generate and display a random integer
//=======================================================================
procedure TForm1.Run0(Sender: TObject);
  var A : TInteger;
begin
  //----- Create A equal to 0
  A := TInteger.Create;
  try
    //----- Random signed integer of at most 166 bits (~ 50 decimal digits)
    IRnd(A,166,false,true);
    //----- Display A
    Memo1.Lines.Add('A (base 10) =' + IStr(A,10,5) + CRLF);
    Memo1.Lines.Add('A (base 16) =' + IStr(A,16,4) + CRLF);
    Memo1.Lines.Add('A (base 2)  =' + IStr(A,2,16) + CRLF);
  finally
    //----- Free the integer
    A.Free;
  end;
end;

/////////////////////////////////////////////////////////////////////////
// Example #1
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// Check whether (A - B)C = AC - BC
//=======================================================================
procedure TForm1.Run1(Sender: TObject);
  const BSIZE = 33200; //About 10000 decimal digits
  var A, B, C, X : TInteger;
begin
  //----- Set to nil so that the "finally" part works okay in any case
  A := nil;
  B := nil;
  C := nil;
  X := nil;
  try
    //----- Create integers equal to 0
    A := TInteger.Create;
    B := TInteger.Create;
    C := TInteger.Create;
    //----- Random signed integers
    IRnd(A,BSIZE,false,true);
    IRnd(B,BSIZE,false,true);
    IRnd(C,BSIZE,false,true);
    //----- Create X equal to A
    X := TInteger.Copy(A);
    //----- Display A, B and C
    // To "see" the integers, activate the 3 following instructions
    {
    Memo1.Lines.Add('A =' + IStr(A) + CRLF);
    Memo1.Lines.Add('B =' + IStr(B) + CRLF);
    Memo1.Lines.Add('C =' + IStr(C) + CRLF);
    }
    //----- Display the decimal sizes of A, B and C
    Memo1.Lines.Add('Decimal size of A = ' + IntToStr(A.DecimalSize));
    Memo1.Lines.Add('Decimal size of B = ' + IntToStr(B.DecimalSize));
    Memo1.Lines.Add('Decimal size of C = ' + IntToStr(C.DecimalSize) + CRLF);
    //----- Compute (A - B)C
    ISub(A,B);
    IMul(A,C);
    //----- Compute AC - BC
    IMul(X,C);
    IMul(B,C);
    ISub(X,B);
    //----- Here, we should have A = X
    Memo1.Lines.Add('Check = ' + CHECKMSG[ICmp(A,X) = 0]);
  finally
    //----- Free the integers
    A.Free;
    B.Free;
    C.Free;
    X.Free;
  end;
end;

/////////////////////////////////////////////////////////////////////////
// Example #2
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// Compute 2^(A-1) modulo A (A is a 332-bit random integer)
//=======================================================================
procedure TForm1.Run2(Sender: TObject);
  var
    A, E, X   : TInteger;
    chrono    : Longword;
    BSize, Hw : Longint;
begin
  //----- Set to nil so that the "finally" part works okay in any case
  A := nil;
  E := nil;
  X := nil;
  with Memo1.Lines do
  try
    //----- Create integers equal to 0
    A := TInteger.Create;
    E := TInteger.Create;
    X := TInteger.Create;
    //----- Compute a 332-bit random unsigned integer
    BSize := 332;  //332 bits ~ 100 decimal digits
    IRnd(A,BSize);
    A.Odd := true; //So that the number is odd
    //----- Display A
    Add('A =' + IStr(A,10,5)); //Base 10, 5-digit blocks
    //----- Display the different sizes of the number
    Add('Size (base 10)   = ' + IntToStr(A.DecimalSize));
    Add('Size (base 2)    = ' + IntToStr(A.BitSize));
    Add('Size (base 2^8)  = ' + IntToStr(A.ByteSize));
    Add('Size (base 2^16) = ' + IntToStr(A.WordSize));
    Add('Size (base 2^32) = ' + IntToStr(A.LongSize));
    //----- Display the size of the memory allocated to the digits
    Add('Precision        = ' + IntToStr(A.Precision(dtByte)) + ' bytes');
    Add('');
    //----- E = A-1
    ISet(E,A); //Copy A to E
    IDec(E);   //E := E - 1
    //----- X = 2
    ISet(X,2);
    //----- Calculate X := 2^(A-1) mod A
    chrono := GetTickCount;
    IPowMod(X,E,A);
    chrono := GetTickCount - chrono;
    //----- Display
    Add('2^(A-1) mod A =' + IStr(X,10,5));
    Add('Size (base 10)   = ' + IntToStr(X.DecimalSize));
    Add('Size (base 2)    = ' + IntToStr(X.BitSize));
    Add('Size (base 2^8)  = ' + IntToStr(X.ByteSize));
    Add('Size (base 2^16) = ' + IntToStr(X.WordSize));
    Add('Size (base 2^32) = ' + IntToStr(X.LongSize));
    Add('Precision        = ' + IntToStr(X.Precision(dtByte)) + ' bytes');
    //----- Rare event: Proba(X = 1) ~ 2/N.BitSize
    if X.Is1 then Add('N is pseudoprime to the base 2.');
    Add('');
    //----- The work done depends on the size and on the Hamming weight
    Dec(BSize);
    Hw := E.Weight-1;
    Add('The calculations required ' +
         IntToStr(BSize) + ' squares, ' +
         IntToStr(Hw) + ' multiplications and ' +
         IntToStr(BSize+Hw) + ' divisions.');
    //----- Display the running time
    Add('Running time = ' + IntToStr(chrono) + ' ms.' + CRLF);
  finally
    //----- Free the integers
    A.Free;
    E.Free;
    X.Free;
  end;
end;

/////////////////////////////////////////////////////////////////////////
// Example #3
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// Saving and loading integers
//=======================================================================
procedure TForm1.Run3(Sender: TObject);
  const FILENAME = 'Number.i';
  var A : TInteger;
begin
  //----- Create A equal to 0
  A := TInteger.Create;
  try
    //----- Random signed integer of at most 500 decimal digits
    IRnd(A,1660,false,true);
    //----- Display A
    Memo1.Lines.Add('A =' + IStr(A,10,5));
    //----- Save A to the FILENAME file
    A.SaveToFile(FILENAME);
    //----- Modify A (set to 0)
    A.Set0;
    //----- Display A
    Memo1.Lines.Add('A =' + IStr(A,10,5));
    //----- Load A from the FILENAME file
    A.LoadFromFile(FILENAME);
    //----- Display A
    Memo1.Lines.Add('A =' + IStr(A,10,5) + CRLF);
  finally
    A.Free;
  end;
end;

/////////////////////////////////////////////////////////////////////////
// Example #4
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// GCD (Greatest Common Divisor)
//=======================================================================
procedure TForm1.Run4(Sender: TObject);
  var
    A, B, C, D : TInteger;
    Chrono     : Longword;
begin
  //----- Set to nil so that the "finally" part works okay in any case
  A := nil;
  B := nil;
  C := nil;
  D := nil;
  //----- Allow to abort the running by a click on the "Abort" button.
  //      To "see" how it slows down the running of routines, you can
  //      disable this instruction.
  IBreakWith(Application.ProcessMessages);
  try
    AbortBtn.Enabled := IBreakOn;
    //----- Create integers equal to 0
    A := TInteger.Create;
    B := TInteger.Create;
    //----- Random unsigned integers
    IRnd(A,20000,false);
    IRnd(B,30000,false);
    //----- Create C and D equal to A and B
    C := TInteger.Copy(A);
    D := TInteger.Copy(B);
    //----- Display the decimal sizes of A and B
    Memo1.Lines.Add('Decimal size of A = ' + IntToStr(A.DecimalSize));
    Memo1.Lines.Add('Decimal size of B = ' + IntToStr(B.DecimalSize) + CRLF);

    //----- Compute the GCD of A and B (Euclidean algorithm)
    Chrono := GetTickCount;
    while not A.Is0 do
    begin
      //----- (A', B') <- (B mod A, A)
      IMod(B,A);
      ISwp(B,A);
    end;
    Chrono := GetTickCount - Chrono;
    //----- Display B (= GCD(A,B))
    Memo1.Lines.Add('GCD(A,B) =' + IStr(B,10,0));
    //----- Display the running time
    Memo1.Lines.Add('Running time = ' + IntToStr(Chrono) + ' ms.' + CRLF);

    //----- Compute the GCD of A and B (with the IGCD routine)
    Chrono := GetTickCount;
    IGCD(C,D);
    Chrono := GetTickCount - Chrono;
    //----- Display C (= GCD(A,B))
    Memo1.Lines.Add('GCD(A,B) =' + IStr(C,10,0));
    //----- Display the running time
    Memo1.Lines.Add('Running time = ' + IntToStr(Chrono) + ' ms.' + CRLF);

    // The 2 previous running times show that IGCD is much more efficient
    // than the simple Euclidean algorithm. That's right. Integers uses
    // the Lehmer-Collins variant.

  finally
    //----- Default status
    AbortBtn.Enabled := false;
    INoBreak;
    //----- Free the integers
    A.Free;
    B.Free;
    C.Free;
    D.Free;
  end;
end;

/////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////

//=======================================================================
// Open the Popup menu
//=======================================================================
procedure TForm1.MenuBtnClick(Sender: TObject);
  var P : TPoint;
begin
  MenuBtn.Enabled := false;
  try
    GetCursorPos(P);
    PopupMenu1.Popup(P.X,P.Y);
  finally
    MenuBtn.Enabled := true;
  end;
end;

//=======================================================================
// Call one of the methods Run
//=======================================================================
procedure TForm1.Runx(Sender: TObject; Num: Integer);
begin
  MenuBtn.Enabled := false;
  ClearBtn.Enabled := false;
  CloseBtn.Enabled := false;
  HelpBtn.Enabled := false;
  PopupMenu1.AutoPopup := false;
  try
    Memo1.Lines.Add(#13#10'Example ' + IntToStr(Num));
    Memo1.Lines.Add('---------'#13#10);
    case Num of
      0: Run0(Sender);
      1: Run1(Sender);
      2: Run2(Sender);
      3: Run3(Sender);
      4: Run4(Sender);
    end;
  finally
    MenuBtn.Enabled := true;
    ClearBtn.Enabled := true;
    CloseBtn.Enabled := true;
    HelpBtn.Enabled := true;
    PopupMenu1.AutoPopup := true;
  end;
end;

//=======================================================================
// Call Run0 method
//=======================================================================
procedure TForm1.PopItem0Click(Sender: TObject);
begin
  Runx(Sender,0);
end;

//=======================================================================
// Call Run1 method
//=======================================================================
procedure TForm1.PopItem1Click(Sender: TObject);
begin
  Runx(Sender,1);
end;

//=======================================================================
// Call Run2 method
//=======================================================================
procedure TForm1.PopItem2Click(Sender: TObject);
begin
  Runx(Sender,2);
end;

//=======================================================================
// Call Run3 method
//=======================================================================
procedure TForm1.PopItem3Click(Sender: TObject);
begin
  Runx(Sender,3);
end;

//=======================================================================
// Call Run4 method
//=======================================================================
procedure TForm1.PopItem4Click(Sender: TObject);
begin
  Runx(Sender,4);
end;

/////////////////////////////////////////////////////////////////////////
end.
/////////////////////////////////////////////////////////////////////////

