unit ResFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons;

type
  TCheckResForm = class(TForm)
    Memo1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    LOriginal: TLabel;
    LActual: TLabel;
    LInfo: TLabel;
    BitBtn1: TBitBtn;
    Bevel1: TBevel;
    Button1: TButton;
    procedure FormActivate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  CheckResForm: TCheckResForm;

implementation

{$R *.DFM}

uses IniFiles, Hash;

{yes, this enumerate over all Resources and calculate the Hash-Digest}
function EnumChaining(Module: hModule; Name: PChar; Hash: Integer): Bool; stdcall;

  function EnumNames(Module: hModule; ResType, ResName: PChar; Hash: THash): Bool; stdcall;
  var
    ResInfo,ResHandle: THandle;
    ResData: Pointer;
  begin
    Result := True;
    ResInfo := FindResource(Module, ResName, ResType);
    if ResInfo = 0 then Exit;
    ResHandle := LoadResource(Module, ResInfo);
    if ResHandle = 0 then Exit;
    ResData := LockResource(ResHandle);
    if ResData <> nil then
      Hash.Calc(ResData^, SizeOfResource(Module, ResInfo));
    UnlockResource(ResHandle);
    FreeResource(ResHandle);
  end;

begin
  Result := True;
  EnumResourceNames(Module, Name, @EnumNames, Hash);
end;

function EnumSeparate(Module: hModule; Name: PChar; Sum: Integer): Bool; stdcall;

  function EnumNames(Module: hModule; ResType, ResName: PChar; Sum: TMemoryStream): Bool; stdcall;
  var
    ResInfo,ResHandle: THandle;
    ResData: Pointer;
    Digest: String;
  begin
    Result := True;
    ResInfo := FindResource(Module, ResName, ResType);
    if ResInfo = 0 then Exit;
    ResHandle := LoadResource(Module, ResInfo);
    if ResHandle = 0 then Exit;
    ResData := LockResource(ResHandle);
    if ResData <> nil then
    begin
{allocate Memory for the Digest}
      SetLength(Digest, THash_RipeMD160.DigestKeySize);
{calc a Digest from the Resource} 
      THash_RipeMD160.CalcBuffer(PChar(Digest), ResData^, SizeOfResource(Module, ResInfo));
{write the Digest to Sum}
      Sum.Write(PChar(Digest)^, THash_RipeMD160.DigestKeySize);
    end;
    UnlockResource(ResHandle);
    FreeResource(ResHandle);
  end;

begin                 
  Result := True;
  EnumResourceNames(Module, Name, @EnumNames, Sum);
end;

procedure TCheckResForm.FormActivate(Sender: TObject);
var
  Hash: THash;
begin
{Disable Memo without changing Textcolor}
  SetWindowLong(Memo1.Handle, gwl_Style, GetWindowLong(Memo1.Handle, gwl_Style) or ws_Disabled);
{load the Original Digest from INI-File}
  with TIniFile.Create(ChangeFileExt(ParamStr(0), '.INI')) do
  try
    LOriginal.Caption := ReadString('System', 'Digest', '');
  finally
    Free;
  end;
  Update;

{this check a modification in chaining mode}
  Hash := THash_Haval224.Create;
  try
    Hash.Init;
    EnumResourceTypes(MainInstance, @EnumChaining, Integer(Hash));
    Hash.Done;
    LActual.Caption := Hash.DigestBase64;
  finally                        
    Hash.Free;
  end;
{this check all Resources separate}
{
  Sum := TMemoryStream.Create;
  try
    EnumResourceTypes(MainInstance, @EnumSeparate, Integer(Sum));
//  Sum contains all Digest from all Resources

    LActual.Caption := THash_SHA1.CalcStream(nil, Sum, -1);
  finally
    Sum.Free;
  end;
}
{Message for You}
  if LActual.Caption = LOriginal.Caption then LInfo.Caption := 'You have NOT modified'
    else LInfo.Caption := 'You have modified';
end;

procedure TCheckResForm.Button1Click(Sender: TObject);
begin
{Save the current calculated Digest as original Digest}
  with TIniFile.Create(ChangeFileExt(ParamStr(0), '.INI')) do
  try
    WriteString('System', 'Digest', LActual.Caption);
  finally
    Free;
  end;
  LOriginal.Caption := LActual.Caption;
  LInfo.Caption := 'Calculated Digest is saved as Original Digest !';  
end;

end.
