unit SearchF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
    TLongArray = array[1..10000000] of Longint;
    PLongArray = ^TLongArray;

  TSearchForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    txtNumItems: TEdit;
    Label2: TLabel;
    lblMaxValue: TLabel;
    cmdCreateList: TButton;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    txtTarget: TEdit;
    txtNumTrials: TEdit;
    cmdSearch: TButton;
    chkExhaustive: TCheckBox;
    lblExhaustivePosition: TLabel;
    lblExhaustiveSearches: TLabel;
    lblExhaustiveTime: TLabel;
    chkBinary: TCheckBox;
    lblBinaryPosition: TLabel;
    lblBinarySearches: TLabel;
    lblBinaryTime: TLabel;
    chkInterpolation: TCheckBox;
    lblInterpolationPosition: TLabel;
    lblInterpolationSearches: TLabel;
    lblInterpolationTime: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cmdCreateListClick(Sender: TObject);
    procedure cmdSearchClick(Sender: TObject);

  private
    { Private declarations }
    List     : PLongArray;
    NumItems : Longint;

    procedure TestExhaustive(target, num_trials : Longint);
    function ExhaustiveSearch(target : Longint; List : PLongArray; min, max : Longint; var num_searches : Longint) : Longint;
    procedure TestBinary(target, num_trials : Longint);
    function BinarySearch(target : Longint; List : PLongArray; min, max : Longint; var num_searches : Longint) : Longint;
    procedure TestInterpolation(target, num_trials : Longint);
    function InterpolationSearch(target : Longint; List : PLongArray; min, max : Longint; var num_searches : Longint) : Longint;

  public
    { Public declarations }
  end;

var
  SearchForm: TSearchForm;

implementation

{$R *.DFM}

// Initialize the random number generator.
procedure TSearchForm.FormCreate(Sender: TObject);
begin
    Randomize;
end;

// Destroy the list if it is allocated.
procedure TSearchForm.FormDestroy(Sender: TObject);
begin
    if (NumItems > 0) then FreeMem(List);
end;

// Create the list to search later.
procedure TSearchForm.cmdCreateListClick(Sender: TObject);
var
    i, new_value : Longint;
begin
    Screen.Cursor := crHourGlass;
    Refresh;

    // Destroy the old list if it has been allocated.
    if (NumItems > 0) then FreeMem(List);

    // Allocate the new List array.
    NumItems := StrToInt(txtNumItems.Text);
    GetMem(List, NumItems * SizeOf(Longint));

    // Create random items.
    new_value := 0;
    for i := 1 to NumItems do
    begin
        new_value := new_value + 1 + Trunc(Random(5));
        List^[i] := new_value;
    end;

    lblMaxValue.Caption := IntToStr(new_value);
    cmdSearch.Enabled := True;
    Screen.Cursor := crDefault;
end;

// Search using the correct methods.
procedure TSearchForm.cmdSearchClick(Sender: TObject);
var
    target, num_trials : Longint;
begin
    Screen.Cursor := crHourGlass;

    // Clear previous results.
    lblExhaustivePosition.Caption := '';
    lblExhaustiveSearches.Caption := '';
    lblExhaustiveTime.Caption := '';

    lblBinaryPosition.Caption := '';
    lblBinarySearches.Caption := '';
    lblBinaryTime.Caption := '';

    lblInterpolationPosition.Caption := '';
    lblInterpolationSearches.Caption := '';
    lblInterpolationTime.Caption := '';

    target := StrToInt(txtTarget.Text);
    num_trials := StrToInt(txtNumTrials.Text);

    if (chkExhaustive.Checked)    then TestExhaustive   (target, num_trials);
    if (chkBinary.Checked)        then TestBinary       (target, num_trials);
    if (chkInterpolation.Checked) then TestInterpolation(target, num_trials);

    Screen.Cursor := crDefault;
end;

// Perform exhaustive search tests.
procedure TSearchForm.TestExhaustive(target, num_trials : Longint);
var
    hours, minutes, seconds, milliseconds : Word;
    start_time, ellapsed_time             : TDateTime;
    trial, index, num_searches            : Longint;
begin
    index := 0;
    start_time := Time;
    for trial := 1 to num_trials do
    begin
        index := ExhaustiveSearch(target, List, 1, NumItems, num_searches);
    end;
    ellapsed_time := Time - start_time;

    lblExhaustivePosition.Caption := IntToStr(index);
    lblExhaustiveSearches.Caption := IntToStr(num_searches);
    DecodeTime(ellapsed_time, hours, minutes, seconds, milliseconds);
    lblExhaustiveTime.Caption := Format('%.2f',
        [milliseconds / 1000.0 + seconds +
            60 * (minutes + hours * 60)]);
end;

// Perform an exhaustive search of a sorted list.
function TSearchForm.ExhaustiveSearch(target : Longint; list : PLongArray; min, max : Longint; var num_searches : Longint) : Longint;
var
    i : Longint;
begin
    num_searches := 0;
    for i := min to max do
    begin
        num_searches := num_searches + 1;
        if (list^[i] >= target) then break;
    end;

    if (i > max) then
        Result := 0    // Not found.
    else if (list^[i] = target) then
        Result := i    // Found.
    else
        Result := 0;   // Not found.
end;

// Perform binary search tests.
procedure TSearchForm.TestBinary(target, num_trials : Longint);
var
    hours, minutes, seconds, milliseconds : Word;
    start_time, ellapsed_time             : TDateTime;
    trial, index, num_searches            : Longint;
begin
    index := 0;
    start_time := Time;
    for trial := 1 to num_trials do
    begin
        index := BinarySearch(target, List, 1, NumItems, num_searches);
    end;
    ellapsed_time := Time - start_time;

    lblBinaryPosition.Caption := IntToStr(index);
    lblBinarySearches.Caption := IntToStr(num_searches);
    DecodeTime(ellapsed_time, hours, minutes, seconds, milliseconds);
    lblBinaryTime.Caption := Format('%.2f',
        [milliseconds / 1000.0 + seconds +
            60 * (minutes + hours * 60)]);
end;

// Perform an binary search of a sorted list.
function TSearchForm.BinarySearch(target : Longint; list : PLongArray; min, max : Longint; var num_searches : Longint) : Longint;
var
    mid : Longint;
begin
    num_searches := 0;

    // The target's index is always between min and max.
    while (min <= max) do
    begin
        num_searches := num_searches + 1;

        mid := Round((max + min) / 2);
        if (target = list^[mid]) then // Here it is.
        begin
            Result := mid;
            Exit;
        end else if (target < list^[mid]) then
            // Search the left half.
            max := mid - 1
        else
            // Search the right half.
            min := mid + 1;
    end;

    // If we get here, the target is not present.
    Result := 0;
end;

// Perform interpolation search tests.
procedure TSearchForm.TestInterpolation(target, num_trials : Longint);
var
    hours, minutes, seconds, milliseconds : Word;
    start_time, ellapsed_time             : TDateTime;
    trial, index, num_searches            : Longint;
begin
    index := 0;
    start_time := Time;
    for trial := 1 to num_trials do
    begin
        index := InterpolationSearch(target, List, 1, NumItems, num_searches);
    end;
    ellapsed_time := Time - start_time;

    lblInterpolationPosition.Caption := IntToStr(index);
    lblInterpolationSearches.Caption := IntToStr(num_searches);
    DecodeTime(ellapsed_time, hours, minutes, seconds, milliseconds);
    lblInterpolationTime.Caption := Format('%.2f',
        [milliseconds / 1000.0 + seconds +
            60 * (minutes + hours * 60)]);
end;

// Perform an interpolation search of a sorted list.
function TSearchForm.InterpolationSearch(target : Longint; list : PLongArray; min, max : Longint; var num_searches : Longint) : Longint;
var
    mid : Longint;
begin
    num_searches := 0;

    while (min <= max) do
    begin
        // Watch for division by zero.
        if (list^[min] = list^[max]) then
        begin
            // If this isn't the item, it's not here.
            if (list^[min] = target) then
                Result := min
            else
                Result := 0;
            Exit;
        end;

        // Compute the dividing index.
        mid := Round(min + ((target - list^[min]) *
            ((max - min) / (list^[max] - list^[min]))));

        // Check that we are in bounds.
        if ((mid < min) or (mid > max)) then
        begin
            // The target isn't here.
            Result := 0;
            Exit;
        end;

        num_searches := num_searches + 1;
        if (target = list^[mid]) then // The target is here.
        begin
            Result := mid;
            Exit;
        end else if (target < list^[mid]) then
            // Search the left half.
            max := mid - 1
        else
            // Search the right half.
            min := mid + 1;
    end; // End while (min <= max) do loop

    // If we got to this point, the target isn't here.
    Result := 0;
end;

end.
