unit Sample;
{ Copyright (c) 1999 by Hagen Reddmann
  mailto:HaReddmann@AOL.COM

  This Unit include Sample Classes, Delphi Encryption Compendium Part I}

interface

uses Hash, Cipher;

type
  THash_Sample = class(THash)
  private
    FDigest: array[0..1] of Integer;
  protected
    class function TestVector: Pointer; override;
  public
    class function DigestKeySize: Integer; override;
    procedure Init; override;
    procedure Calc(const Data; DataSize: Integer); override;
    function DigestKey: Pointer; override;
  end;

  TCipher_Sample = class(TCipher)
  protected
    class procedure GetContext(var ABufSize, AKeySize, AUserSize: Integer); override;
    class function TestVector: Pointer; override;
    procedure Encode(Data: Pointer); override;
    procedure Decode(Data: Pointer); override;
  public
    procedure Init(const Key; Size: Integer; IVector: Pointer); override;
    procedure Done; override;
  end;

implementation

uses HCMngr;

type
  PByte    = ^Byte;
  PInteger = ^Integer;

{this Code is automaticly generated,
 click 'File\Generate Testvector for Hash' and Paste from Clipboard here}  
class function THash_Sample.TestVector: Pointer;
asm
         MOV   EAX,OFFSET @Vector
         RET
@Vector: DB    0F3h,0C5h,05Ch,05Ch,005h,000h,000h,000h
end;

class function THash_Sample.DigestKeySize: Integer;
begin
  Result := SizeOf(Integer) * 2;
end;

procedure THash_Sample.Init;
begin
  FillChar(FDigest, SizeOf(FDigest), 0);
end;

procedure THash_Sample.Calc(const Data; DataSize: Integer);
// calculate a Crosssum !??, in german Quersumme
var
  B: PInteger;
  T: Integer;
begin
  B := @Data;
  while DataSize >= SizeOf(Integer) do
  begin
    T := FDigest[0];
    Inc(FDigest[0], B^);
    if FDigest[0] < T then Inc(FDigest[1]);
    Inc(B);
    Dec(DataSize, SizeOf(Integer));
  end;
  if DataSize > 0 then
  begin
    T := 0;
    Move(B^, T, DataSize);
    Inc(FDigest[0], T);
  end;
end;

function THash_Sample.DigestKey: Pointer;
begin
  Result := @FDigest;
end;

{Borland's String Encryption Method}
class procedure TCipher_Sample.GetContext(var ABufSize, AKeySize, AUserSize: Integer);
begin
  ABufSize := 128;
  AKeySize := 6 * SizeOf(Integer);
  AUserSize := 6 * SizeOf(Integer);
end;

class function TCipher_Sample.TestVector: Pointer;
asm
         MOV   EAX,OFFSET @Vector
         RET
@Vector: DB    0F5h,076h,09Ch,08Fh,04Dh,065h,0BEh,0ACh
         DB    019h,08Ch,0B9h,08Ch,000h,0F9h,06Eh,072h
         DB    0D0h,072h,0C7h,018h,03Eh,0C4h,0BEh,007h
         DB    0AFh,013h,0A8h,0B9h,017h,01Ch,05Fh,077h
end;

type
  PSampleKey = ^TSampleKey;
  TSampleKey = packed record
                 StartKey: Integer;
                 MultKey: Integer;
                 AddKey: Integer;
               end;
  PSaveKey = ^TSaveKey;
  TSaveKey = packed record
               WorkKey: TSampleKey;
               SaveKey: TSampleKey;
             end;

procedure TCipher_Sample.Encode(Data: Pointer);
var
  I,T: Integer;
  B: PByte;
begin
  B := Data;
  with PSampleKey(User)^ do
    for I := 1 to BufSize do
    begin
      T  := B^;
      T  := (T xor (StartKey shr 8)) and $FF;
      B^ := T;
      StartKey := (StartKey + T) * MultKey + AddKey;
      Inc(B);
    end;
end;

procedure TCipher_Sample.Decode(Data: Pointer);
var
  I,T: Integer;
  B: PByte;
begin
  B := Data;
  with PSampleKey(User)^ do
    for I := 1 to BufSize do
    begin
      T  := B^;
      B^ := B^ xor StartKey shr 8;
      StartKey := (StartKey + T) * MultKey + AddKey;
      Inc(B);
    end;
end;

procedure TCipher_Sample.Init(const Key; Size: Integer; IVector: Pointer);
const
  DefaultKey : TSampleKey = (StartKey: 981; MultKey: 12674; AddKey: 35891);
begin
  InitBegin(Size);
  Move(DefaultKey, User^, SizeOf(DefaultKey));
  Move(Key, User^, Size);
  InitEnd(IVector);
{Save the Key to SaveKey}
  with PSaveKey(User)^ do Move(WorkKey, SaveKey, SizeOf(WorkKey));
end;

procedure TCipher_Sample.Done;
begin
  inherited Done;
{Restore the WorkKey}
  with PSaveKey(User)^ do Move(SaveKey, WorkKey, SizeOf(SaveKey));
end;

{the conventional using from THash by a specific Class, this is default
  without using THashManager

  ResultBase64String := THash_RipeMD160.CalcFile(nil, 'C:\MyFile.dat');

next use a TStream and calculate from Position 60 to 160, 100 Bytes

  MyStream := TFileStream.Create('c:\myFile', fmOpenRead or fmShareDenyNone);
  MyStream.Position := 60;
  ResultBase64String := THash_MD5.CalcStream(nil, MyStream, 100);
  MyStream.Free;


 next calculate a CRC32 - Checksum over a String

var
  Digest: String;

  DefaultDigestStringFormat := 16;
  SetLength(Digest, THash_CRC32.DigestKeySize);
  ResultHexString := THash_CRC32.CalcString(PChar(Digest), 'Hagen Reddmann');
  ResultBase64String := StrToBase64(PChar(Digest), THash_CRC32.DigestKeySize);

 next use a Instance of Hash

 with THash_SHA1.Create do
 try
   Init;
// init with a other Digestvalue
   Move(MyDigestInitValue, Digest^, DigestKeySize);
//Calculate over 3 various Data Buffers
   Calc(MyBuffer_A, MyBufferSize_A);
   Calc(MyBuffer_B, MyBufferSize_B);
   Calc(MyBuffer_C, MyBufferSize_C);
   Done;
//give me the Results
   Result64Str                 := DigestBase64;
   Result16Str                 := DigestBase16;
   ResultBinaryDataStr         := DigestKeyStr
   ResultDigestByDefaultFormat := DigestString;
 finally
   Free;
 end;


Hash Class-Methods

  DigestKeySize: Integer     // give back SizeOf Digest

  ResultDefaultStringFormt := CalcBuffer(Digest, Buffer, BufferSize);
  ResultDefaultStringFormt := CalcFile(Digest, File);
  ResultDefaultStringFormt := CalcStream(Digest, Stream, StreamSize);
  ResultDefaultStringFormt := CalcString(Digest, String);

Hash Object Methods

  Init;                     // initialize the Hash Object
  Calc(Buffer, BufferSize)  // calculate
  Done;                     // calculate the Rest and the Digest

  property DigestBase16: String  // a HEX-String Representation from Digest^
  property DigestBase64: String  // a Base 64 MIME Repr. from Digest
  property DigestString: String  // default Format
  property DigestKeyStr: String  // binary Digest but as String from the Length DigestKeySize

}

{ the conventional using from TCipher by a specific Class, this is default
  without using TCipherManager

 with TCipher_SAFER.Create do
 try
// set to other Rounds
   Rounds := 13;            // various Parmeters for SAFER


// set to other Hash, default is THash_SHA1
   HashClass := THash_MD5;

// init the Key with other InitVector
   InitKey('Hagen Reddmann', @MyInitVectorArray);
or
// specific for SAFER  - smK40, smSK40, smK64, smSK64, smK128, smSK128, smStrong, smDefault
   InitNew(MyKeyArray, MyKeyArraySize, smSK64)

// en/decode a String
   ResultString := EncodeString('Hagen Reddmann');

// protect the security relevant Data's
   Protect;

// and now back
   InitKey('Hagen Reddmann', @MyInitVectorArray);
   OriginalString := DecodeString(ResultString);

 finally
   Free;
 end;

// this en/decode a Part from the TStream MyStream into same Stream

  with TCipher_FROG.Create do
  try
    InitKey('Key', nil);
    MyStream.Position := 100;
    EncodeStream(MyStream, nil, 100);       // 100 Bytes from MyStream to MyStream
or
    EncodeStream(MyStream, MyStream, 100);  // has same Effect
    Done;                                   // a new Process will begin with the same Key
    MyStream.Position := 100;
    DecodeStream(MyStream, ToOtherStream, 100);
or
    DecodeStream(MyStream, nil, 100);
  finally
    Free;
  end;

en/decode Methods

  EncodeBuffer(Source, Dest, Size);
  DecodeBuffer(Source, Dest, Size);

  EncodeFile(SourceFile, DestFile); // Source and Dest can be equal or Dest = ''
  DecodeFile(SourceFile, DestFile);

  EncodeStream(SourceStream, DestStream, Size);
  DecodeStream(SourceStream, DestStream, Size);

  ResultString := EncodeString(String);
  ResultString := DecodeString(String);

  Done;                             // for a new Process

  Protect;                          // for protect the internal Datas
}

initialization
  THashManager.RegisterHash(THash_Sample, 'Sample Hash', 'see Unit "Sample"');
  TCipherManager.RegisterCipher(TCipher_Sample, 'Sample Cipher', 'see Unit "Sample"')
finalization
  THashManager.UnregisterHash(THash_Sample);
  TCipherManager.UnregisterCipher(TCipher_Sample);
end.
 