{
  Name    :   AccountTypes2.pas

  Purpose :   MTS TAccountTypes2 object implementing IAccountTypes2 and operating on
               the ACCTTYPE table.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, October 1998. paul@c-s-c.demon.co.uk
}
unit AccountTypes2;

interface

uses
  BDEMTS, AccountTypesDM, ActiveX, MtsObj, ComObj, DelphiBankServer3_TLB;

type
  TAccountTypes3 = class(TMtsAutoObject, IAccountTypes3, IAccountTypesAdmin)
  private
    dmAccountTypes : TdmAccountTypes;
  protected
    function ListAll(ClientKey: Integer; var varResultSet: OleVariant; var strDebug: WideString): Integer; safecall;
    function AddNew(ClientKey: Integer; const ATName: WideString; ATInterestRate: Single;
                    ATDefMinBalance: Currency; var strDebug: WideString): Integer; safecall;
    function Delete(ClientKey: Integer; ATID: Integer; var strDebug: WideString): Integer; safecall;
    function ListAccounts(ClientKey: Integer; ATID: Integer; var varResultSet: OleVariant;
                          var strDebug: WideString): Integer; safecall;
    function GetAcctTypeByID(ClientKey: Integer; ATID: Integer; var ATName: WideString;
                             var ATInterestRate: Single; var ATDefMinBalance: Currency;
                             var StrDebug: WideString): Integer; safecall;
    { Protected declarations }
    procedure OnActivate; override;
    procedure OnDeactivate; override;
    procedure CheckDbOpen;
  end;

implementation

uses SysUtils, ComServ, ServerUtils, DelphiBankCommonDM, Windows;

{ ---------------------------------------------------------------------------- }
procedure TAccountTypes3.OnActivate;
begin
  TServerUtils.WriteToDelphiBankLog('TAccountTypes2 - OnActivate entered.');

  try
    { Create our own datamodule. }
    dmAccountTypes := TdmAccountTypes.Create(nil);
    if Assigned(dmAccountTypes) then
      dmAccountTypes.DbDelphiBank.Open;
  except
    on E: Exception do
      TServerUtils.WriteToDelphiBankLog('TAccountTypes2.OnActivate - ' + E.Message);
  end;
end;

{ ---------------------------------------------------------------------------- }
procedure TAccountTypes3.OnDeactivate;
begin
  TServerUtils.WriteToDelphiBankLog('TAccountTypes2 - OnDeactivate entered.');

  try
    if Assigned(dmAccountTypes) then begin
      dmAccountTypes.DbDelphiBank.Close;
      dmAccountTypes.Free;
    end;
    dmAccountTypes := nil;
  except
  end;
end;

{ ---------------------------------------------------------------------------- }
procedure TAccountTypes3.CheckDbOpen;
begin
  if not Assigned(dmAccountTypes)then
    raise Exception.Create('Datamodule not instantiated.');
  if not dmAccountTypes.DbDelphiBank.Connected then
    raise Exception.Create('Could not establish connection to database.');
end;

{ ---------------------------------------------------------------------------- }
function TAccountTypes3.ListAll(ClientKey: Integer;
                                var varResultSet: OleVariant;
                                var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }

  varResultSet := UnAssigned;
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccountTypes do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAcctTypeReadAll.Open;

      { Package the result. }
      CreateVarArrayFromDataset(varResultSet, QryAcctTypeReadAll);
      QryAcctTypeReadAll.Close;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      dmAccountTypes.QryAcctTypeReadAll.Close;
      strDebug := 'TAccountTypes2.ListAll() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccountTypes3.AddNew(ClientKey: Integer; const ATName: WideString;
                               ATInterestRate: Single; ATDefMinBalance: Currency;
                               var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccountTypes do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAcctTypeInsert.ParamByName(cFieldATID).Value := GetTickCount;
      QryAcctTypeInsert.ParamByName(cFieldATName).Value := ATName;
      QryAcctTypeInsert.ParamByName(cFieldATInterestRate).Value := ATInterestRate;
      QryAcctTypeInsert.ParamByName(cFieldATDefMinBalance).Value := ATDefMinBalance;
      QryAcctTypeInsert.ExecSQL;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      strDebug := 'TAccountTypes2.AddNew() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccountTypes3.Delete(ClientKey: Integer; ATID: Integer;
                               var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccountTypes do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAcctTypeDelete.ParamByName(cFieldATID).Value := ATID;
      QryAcctTypeDelete.ExecSQL;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      strDebug := 'TAccountTypes2.Delete() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccountTypes3.ListAccounts(ClientKey: Integer; ATID: Integer;
                                     var varResultSet: OleVariant;
                                     var strDebug: WideString): Integer;
var
  AccountsIntf   : IAccounts2;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccountTypes do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { We can delegate this to the Accounts object. }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_Accounts2, IAccounts2, AccountsIntf));
      except
        raise Exception.Create('Could not create Accounts object');
      end;

      { Make the call. }
      if AccountsIntf.ListByAcctTypeID(ClientKey, ATID, varResultSet, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);
    end;

    SetComplete; { OK. }
    AccountsIntf := nil;
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      AccountsIntf := nil;
      strDebug := 'TAccountTypes2.ListAccounts() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccountTypes3.GetAcctTypeByID(ClientKey: Integer;
                                        ATID: Integer;
                                        var ATName: WideString;
                                        var ATInterestRate: Single;
                                        var ATDefMinBalance: Currency;
                                        var StrDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccountTypes do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      with QryAcctTypeSelectiveRead do begin
        ParamByName(cFieldATID).Value := ATID;
        Open;

        First;
        if EOF then raise Exception.Create('Account Type ID not found');

        ATName          := FieldByName(cFieldATName).Value;
        ATInterestRate  := FieldByName(cFieldATInterestRate).Value;

        Close;
      end;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      dmAccountTypes.QryAcctTypeSelectiveRead.Close;
      strDebug := 'TAccountTypes2.GetAcctTypeByID() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
initialization
  TAutoObjectFactory.Create(ComServer, TAccountTypes3, Class_AccountTypes3,
    ciMultiInstance, tmApartment);

finalization
  TServerUtils.WriteToDelphiBankLog('AccountTypes2 module finalization...');

end.
