{
  Name    :   Accounts.pas

  Purpose :   MTS TAccounts object implementing IAccounts and operating on
               the ACCOUNTS table.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, October 1998. paul@c-s-c.demon.co.uk
}
unit Accounts;

interface

uses
  BDEMTS, ActiveX, MtsObj, ComObj, DelphiBankServer3_TLB, AccountsDM;

type
  TAccounts2 = class(TMtsAutoObject, IAccounts2, IAccountsAdmin)
  private
    dmAccounts : TdmAccounts;
  protected
    function AddNew(ClientKey, ATID, CustID: Integer;
      var strDebug: WideString): Integer; safecall;
    function AddTransaction(ClientKey, AcctID: Integer;
      const TrnRef: WideString; TrnAmount: Currency;
      var strDebug: WideString): Integer; safecall;
    function Delete(ClientKey, AcctID: Integer;
      var strDebug: WideString): Integer; safecall;
    function GetAcctByID(ClientKey, AcctID: Integer; var ATID, CustID: Integer;
      var AcctMinBalance, AcctBalance: Currency;
      var AcctLastStatement: TDateTime; var strDebug: WideString): Integer;
      safecall;
    function ListAll(ClientKey: Integer; var varResultSet: OleVariant;
      var strDebug: WideString): Integer; safecall;
    function ListByAcctTypeID(ClientKey, ATID: Integer;
      var varResultSet: OleVariant; var strDebug: WideString): Integer;
      safecall;
    function ListByCustID(ClientKey, CustID: Integer;
      var varResultSet: OleVariant; var strDebug: WideString): Integer;
      safecall;
    function ListStatementTransByAcctID(ClientKey, AcctID: Integer;
      var varResultSet: OleVariant; var strDebug: WideString): Integer;
      safecall;
    function ListTransByAcctID(ClientKey, AcctID: Integer;
      var varResultSet: OleVariant; var strDebug: WideString): Integer;
      safecall;
    function Transfer(ClientKey, SrcAcctID, DestAcctID: Integer;
      const TrnRef: WideString; TrnAmount: Currency;
      ForceFailure: WordBool; var strDebug: WideString): Integer; safecall;
    { Protected declarations }
    function  AddToBalance(ClientKey, AcctID : Integer;
                           TrnAmount : Currency; dmAccounts : TdmAccounts;
                           var strDebug : WideString) : Boolean;
    procedure OnActivate; override;
    procedure OnDeactivate; override;
    procedure CheckDbOpen;
  end;

implementation

uses ComServ, ServerUtils, {DMDelphiBank, }SysUtils, DelphiBankCommonDM, dbtables, Windows;

{ ---------------------------------------------------------------------------- }
procedure TAccounts2.OnActivate;
begin
  TServerUtils.WriteToDelphiBankLog('TAccounts - OnActivate entered.');

  { Create our own datamodule. }
  try
    dmAccounts := TdmAccounts.Create(nil);
    if Assigned(dmAccounts) then
      dmAccounts.DbDelphiBank.Open;
  except
    on E: Exception do
      TServerUtils.WriteToDelphiBankLog('TAccounts.OnActivate - ' + E.Message);
  end;
end;

{ ---------------------------------------------------------------------------- }
procedure TAccounts2.OnDeactivate;
begin
  TServerUtils.WriteToDelphiBankLog('TAccounts - OnDeactivate entered.');

  try
    if Assigned(dmAccounts) then begin
      dmAccounts.DbDelphiBank.Close;
      dmAccounts.Free;
    end;
    dmAccounts := nil;
  except
  end;
end;

{ ---------------------------------------------------------------------------- }
procedure TAccounts2.CheckDbOpen;
begin
  if not Assigned(dmAccounts)then
    raise Exception.Create('Datamodule not instantiated.');
  if not dmAccounts.DbDelphiBank.Connected then 
    raise Exception.Create('Could not establish connection to database.');
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.AddNew(ClientKey, ATID, CustID: Integer;
  var strDebug: WideString): Integer;
var
  AccountTypesIntf     : IAccountTypes3;
  ATName               : WideString;
  ATInterestRate       : Single;
  ATDefMinBalance      : Currency;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { We need to ask an AccountType2 object for its details to get the
         default minimum balance. }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_AccountTypes3, IAccountTypes3, AccountTypesIntf));
      except
        raise Exception.Create('Could not create AccountTypes2 object');
      end;

      { Make the call. }
      if AccountTypesIntf.GetAcctTypeByID(ClientKey, ATID, ATName, ATInterestRate,
                                          ATDefMinBalance, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);

      { Run the query. }
      QryAccountInsert.ParamByName(cFieldAcctID).Value := GetTickCount;
      QryAccountInsert.ParamByName(cFieldATID).Value := ATID;
      QryAccountInsert.ParamByName(cFieldCustID).Value := CustID;
      QryAccountInsert.ParamByName(cFieldAcctMinBalance).Value := ATDefMinBalance;
      QryAccountInsert.ParamByName(cFieldAcctBalance).Value := 0.0;
      QryAccountInsert.ParamByName(cFieldAcctLastStatement).Value := Now;
      QryAccountInsert.ExecSQL;
    end;

    SetComplete; { OK. }
    AccountTypesIntf := nil;
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      AccountTypesIntf := nil;
      strDebug := 'TAccounts.AddNew() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.AddTransaction(ClientKey, AcctID: Integer;
  const TrnRef: WideString; TrnAmount: Currency;
  var strDebug: WideString): Integer;
var
  TransactionsIntf : ITransactions2;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;
    
    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { We can delegate this to the Transactions object. }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_Transactions2, ITransactions2, TransactionsIntf));
      except
        raise Exception.Create('Could not create Transactions object');
      end;

      { Add the amount to the balance. }
      if not AddToBalance(ClientKey, AcctID, TrnAmount, dmAccounts, strDebug) then
        raise Exception.Create(strDebug);

      { Make the transaction call. }
      if TransactionsIntf.AddNew(ClientKey, AcctID, TrnRef, TrnAmount, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);
    end;

    SetComplete; { OK. }
    TransactionsIntf := nil;
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      TransactionsIntf := nil;
      strDebug := 'TAccounts.AddTransaction() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.Delete(ClientKey, AcctID: Integer;
  var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAccountDelete.ParamByName(cFieldAcctID).Value := AcctID;
      QryAccountDelete.ExecSQL;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      strDebug := 'TAccounts.Delete() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.GetAcctByID(ClientKey, AcctID: Integer; var ATID,
  CustID: Integer; var AcctMinBalance, AcctBalance: Currency;
  var AcctLastStatement: TDateTime; var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      with QryAccountSelectiveRead do begin
        ParamByName(cFieldAcctID).Value := AcctID;
        Open;

        First;
        if EOF then raise Exception.Create('Account ID not found');

        ATID              := FieldByName(cFieldATID).Value;
        CustID            := FieldByName(cFieldCustID).Value;
        AcctMinBalance    := FieldByName(cFieldAcctMinBalance).Value;
        AcctBalance       := FieldByName(cFieldAcctBalance).Value;
        AcctLastStatement := FieldByName(cFieldAcctLastStatement).Value;
        Close;
      end;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      dmAccounts.QryAccountSelectiveRead.Close;
      SetAbort; { Problem encountered. }
      strDebug := 'TAccounts.GetAcctByID() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.ListAll(ClientKey: Integer;
  var varResultSet: OleVariant; var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }

  varResultSet := UnAssigned;
  strDebug     := '';

  try
    CheckDbOpen;
    
    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAccountReadAll.Open;

      { Package the result. }
      CreateVarArrayFromDataset(varResultSet, QryAccountReadAll);
      QryAccountReadAll.Close;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      dmAccounts.QryAccountReadAll.Close;
      strDebug := 'TAccounts.ListAll() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.ListByAcctTypeID(ClientKey, ATID: Integer;
  var varResultSet: OleVariant; var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }

  varResultSet := UnAssigned;
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAccountReadByAcctTypeID.ParamByName(cFieldATID).Value := ATID;
      QryAccountReadByAcctTypeID.Open;

      { Package the result. }
      CreateVarArrayFromDataset(varResultSet, QryAccountReadByAcctTypeID);
      QryAccountReadByAcctTypeID.Close;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      dmAccounts.QryAccountReadByAcctTypeID.Close;
      strDebug := 'TAccounts.ListByAcctTypeID() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.ListByCustID(ClientKey, CustID: Integer;
  var varResultSet: OleVariant; var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }

  varResultSet := UnAssigned;
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryAccountReadByCustID.ParamByName(cFieldCustID).Value := CustID;
      QryAccountReadByCustID.Open;

      { Package the result. }
      CreateVarArrayFromDataset(varResultSet, QryAccountReadByCustID);
      QryAccountReadByCustID.Close;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      dmAccounts.QryAccountReadByCustID.Close;
      strDebug := 'TAccounts.ListByCustID() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.ListStatementTransByAcctID(ClientKey, AcctID: Integer;
  var varResultSet: OleVariant; var strDebug: WideString): Integer;
var
  TransactionsIntf : ITransactions2;
begin
  Result := cErrResult; { Default. }

  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { We can delegate this to the Transactions object. }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_Transactions2, ITransactions2, TransactionsIntf));
      except
        raise Exception.Create('Could not create Transactions object');
      end;

      { Make the call. }
      if TransactionsIntf.ListStatementByAcctID(ClientKey, AcctID, varResultSet, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);
    end;

    SetComplete; { OK. }
    TransactionsIntf := nil;
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      TransactionsIntf := nil;
      strDebug := 'TAccounts.ListStatementTransByAcctID() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.ListTransByAcctID(ClientKey, AcctID: Integer;
                                     var varResultSet: OleVariant;
                                     var strDebug: WideString): Integer;
var
  TransactionsIntf : ITransactions2;
begin
  Result := cErrResult; { Default. }

  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { We can delegate this to the Transactions object. }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_Transactions2, ITransactions2, TransactionsIntf));
      except
        raise Exception.Create('Could not create Transactions object');
      end;

      { Make the call. }
      if TransactionsIntf.ListByAcctID(ClientKey, AcctID, varResultSet, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);
    end;

    SetComplete; { OK. }
    TransactionsIntf := nil;
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      TransactionsIntf := nil;
      strDebug := 'TAccounts.ListTransByAcctID() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.AddToBalance(ClientKey, AcctID : Integer;
                                TrnAmount : Currency; dmAccounts : TdmAccounts;
                                var strDebug : WideString) : Boolean;
var
  ATID, CustID                : Integer;
  AcctMinBalance, AcctBalance : Currency;
  AcctLastStatement           : TDateTime;
begin
  Result := False;
  try
    CheckDbOpen;

    with dmAccounts do begin
      { We must add the transaction amount to the Account's balance.
        We cannot let it go below the minimum balance so first we
         need to get the Account's details.
      }
      if GetAcctByID(ClientKey, AcctID, ATID, CustID, AcctMinBalance,
                     AcctBalance, AcctLastStatement, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);

      {TServerUtils.WriteToDelphiBankLog('AcctID=' + IntToStr(AcctID) + ',' +
                                         'AcctBalance=' + FloatToStr(AcctBalance) + ',' +
                                         'TrnAmount=' + FloatToStr(TrnAmount) + ',' +
                                         'AcctMinBalance=' + FloatToStr(AcctMinBalance));}

      if (AcctBalance + TrnAmount) < AcctMinBalance then
        raise Exception.Create('Transaction would leave account ' + IntToStr(AcctID) +
                               '''s balance below minimum allowed.');

      { Enforce business rules. }
      { Disabled. Before enabling this code you should enable security
         on the System package, add yourself to the Administrator role
         then create three roles for "The Delpi Bank" package - Managers,
         "Tellers" and "Customers". Add user accounts for users who will
         login and add them to the relevant roles above. }
      {if (TrnAmount > 2000) or (TrnAmount < 2000) then begin
        if not (IsSecurityEnabled and IsCallerInRole('Managers')) then
          raise Exception.Create('Transactions of more than 2000 dollars only allowed by Managers');
      end;}

      { Update the Account's balance. }
      QryAccountAddToBalance.Close;
      QryAccountAddToBalance.ParamByName(cFieldAcctID).Value    := AcctID;
      QryAccountAddToBalance.ParamByName(cFieldTrnAmount).Value := TrnAmount;
      QryAccountAddToBalance.ExecSQL;
    end;

    Result := True;
  except
    on E: Exception do begin
      { Write out to event log. }
      strDebug := E.Message;
      TServerUtils.WriteToDelphiBankLog('TAccounts.AddToBalance() - ' + strDebug);
    end;
  end;
end;

{ ---------------------------------------------------------------------------- }
function TAccounts2.Transfer(ClientKey, SrcAcctID, DestAcctID: Integer;
  const TrnRef: WideString; TrnAmount: Currency; ForceFailure: WordBool;
  var strDebug: WideString): Integer;
var
  TransactionsIntf : ITransactions2;
begin
  Result := cErrResult; { Default. }

  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmAccounts do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;
      { We need to delegate this to the Transactions object by adding
         two new transactions. One to deduct the money from the source
         account and the other to deposit it in the destination account.
        We don't worry about only part of this operation succeeding since
         we call SetAbort() if anthing goes wrong and MTS takes care of
         the rest! }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_Transactions2, ITransactions2, TransactionsIntf));
      except
        raise Exception.Create('Could not create Transactions object');
      end;

      { Deposit the amount to the destination balance. }
      if not AddToBalance(ClientKey, DestAcctID, TrnAmount, dmAccounts, strDebug) then
        raise Exception.Create(strDebug);
      { Deposit the money from the destination. }
      if TransactionsIntf.AddNew(ClientKey, DestAcctID, TrnRef, TrnAmount, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);

      { If we are testing a transaction failure, do it now. }
      if ForceFailure then
        raise Exception.Create('Forced failure - Amount deposited but not deducted.');

      { Deduct the amount from the source balance. }
      if not AddToBalance(ClientKey, SrcAcctID, -TrnAmount, dmAccounts, strDebug) then
        raise Exception.Create(strDebug);
      { Deduct the money from the source. }
      if TransactionsIntf.AddNew(ClientKey, SrcAcctID, TrnRef, -TrnAmount, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);
    end;

    SetComplete; { OK. }
    Result := cOKResult;
    TransactionsIntf := nil;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      TransactionsIntf := nil;
      strDebug := 'TAccounts.Transfer() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

initialization
  TAutoObjectFactory.Create(ComServer, TAccounts2, Class_Accounts2,
    ciMultiInstance, tmApartment);

finalization
  TServerUtils.WriteToDelphiBankLog('Accounts module finalization...');

end.
