{
  Name    :   Customers.pas

  Purpose :   MTS TCustomers object implementing ICustomers and operating on
               the CUSTOMER table.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, October 1998. paul@c-s-c.demon.co.uk
}
unit Customers;

interface

uses
  BDEMTS, CustomersDM, ActiveX, MtsObj, ComObj, DelphiBankServer3_TLB;

type
  TCustomers2 = class(TMtsAutoObject, ICustomers2, ICustomersAdmin)
  private
    dmCustomers  : TdmCustomers;
  protected
    function AddNew(ClientKey: Integer; const CustName, CustLogonID,
      CustPassword, CustNotes: WideString;
      var strDebug: WideString): Integer; safecall;
    function Delete(ClientKey, CustID: Integer;
      var strDebug: WideString): Integer; safecall;
    function ListAccounts(ClientKey, CustID: Integer;
      var varResultSet: OleVariant; var strDebug: WideString): Integer;
      safecall;
    function ListAll(ClientKey: Integer; var varResultSet: OleVariant;
      var strDebug: WideString): Integer; safecall;
    function Logon(const CustLogonID, CustPassword: WideString;
      var CustFullName, CustNotes: WideString; var CustID,
      ClientKey: Integer; var strDebug: WideString): Integer; safecall;
    { Protected declarations }
    procedure OnActivate; override;
    procedure OnDeactivate; override;
    procedure CheckDbOpen;
  end;

implementation

uses ComServ, ServerUtils, {DMDelphiBank, }SysUtils, DelphiBankCommonDM, Windows;

{ ---------------------------------------------------------------------------- }
procedure TCustomers2.OnActivate;
begin
  TServerUtils.WriteToDelphiBankLog('TCustomers - OnActivate entered.');

  try
    { Create our own datamodule. }
    dmCustomers := TdmCustomers.Create(nil);
    if Assigned(dmCustomers) then
      dmCustomers.DbDelphiBank.Open;
  except
    on E: Exception do
      TServerUtils.WriteToDelphiBankLog('TCustomers.OnActivate - ' + E.Message);
  end;
end;

{ ---------------------------------------------------------------------------- }
procedure TCustomers2.OnDeactivate;
begin
  TServerUtils.WriteToDelphiBankLog('TCustomers - OnDeactivate entered.');

  try
    if Assigned(dmCustomers) then begin
      dmCustomers.DbDelphiBank.Close;
      dmCustomers.Free;
    end;
    dmCustomers := nil;
  except
  end;
end;

{ ---------------------------------------------------------------------------- }
procedure TCustomers2.CheckDbOpen;
begin
  if not Assigned(dmCustomers)then
    raise Exception.Create('Datamodule not instantiated.');
  if not dmCustomers.DbDelphiBank.Connected then
    raise Exception.Create('Could not establish connection to database.');
end;

{ ---------------------------------------------------------------------------- }
function TCustomers2.AddNew(ClientKey: Integer; const CustName, CustLogonID,
  CustPassword, CustNotes: WideString; var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmCustomers do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryCustomerInsert.ParamByName(cFieldCustID).Value       := GetTickCount;
      QryCustomerInsert.ParamByName(cFieldCustName).Value     := CustName;
      QryCustomerInsert.ParamByName(cFieldCustLogonID).Value  := UpperCase(CustLogonID);
      QryCustomerInsert.ParamByName(cFieldCustPassword).Value := CustPassword;
      QryCustomerInsert.ParamByName(cFieldCustNotes).Value    := CustNotes;
      QryCustomerInsert.ExecSQL;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      strDebug := 'TCustomers.AddNew() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TCustomers2.Delete(ClientKey, CustID: Integer;
  var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmCustomers do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryCustomerDelete.ParamByName(cFieldCustID).Value := CustID;
      QryCustomerDelete.ExecSQL;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      strDebug := 'TCustomers.Delete() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TCustomers2.ListAccounts(ClientKey, CustID: Integer;
                                 var varResultSet: OleVariant;
                                 var strDebug: WideString): Integer;
var
  AccountsIntf : IAccounts2;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmCustomers do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { We can delegate this to the Accounts object. }
      try
        OleCheck(ObjectContext.CreateInstance(CLASS_Accounts2, IAccounts2, AccountsIntf));
      except
        raise Exception.Create('Could not create Accounts object');
      end;

      { Make the call. }
      if AccountsIntf.ListByCustID(ClientKey, CustID, varResultSet, strDebug) <> cOKResult then
        raise Exception.Create(strDebug);
    end;

    SetComplete; { OK. }
    AccountsIntf := nil;
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      AccountsIntf := nil;
      strDebug := 'TCustomers.ListAccounts() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TCustomers2.ListAll(ClientKey: Integer;
  var varResultSet: OleVariant; var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }

  varResultSet := UnAssigned;
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmCustomers do begin
      { Security. }
      if not IsClientKeyValid(ClientKey, strDebug) then Exit;

      { Run the query. }
      QryCustomerReadAll.Open;

      { Package the result. }
      CreateVarArrayFromDataset(varResultSet, QryCustomerReadAll);

      QryCustomerReadAll.Close;
    end;

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      dmCustomers.QryCustomerReadAll.Close;
      SetAbort; { Problem encountered. }
      strDebug := 'TCustomers.ListAll() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
function TCustomers2.Logon(const CustLogonID, CustPassword: WideString;
  var CustFullName, CustNotes: WideString; var CustID, ClientKey: Integer;
  var strDebug: WideString): Integer;
begin
  Result := cErrResult; { Default. }
  strDebug     := '';

  try
    CheckDbOpen;

    { Set up the query. }
    with dmCustomers do begin
      with QryCustomerMatchLogon do begin
        ParamByName(cFieldCustLogonID).Value  := UpperCase(CustLogonID);
        ParamByName(cFieldCustPassword).Value := CustPassword;
        Open;

        First;

        if EOF then raise Exception.Create('Unknown logon id or password');

        CustFullName := FieldByName(cFieldCustName).Value;
        CustNotes    := FieldByName(cFieldCustNotes).Value;
        CustID       := FieldByName(cFieldCustID).Value;
        Close;
      end;

      { We now generate a new Client Key for this logon session. }
      ClientKey := CreateClientKey;
    end;

    TServerUtils.WriteToDelphiBankLog('Customer ''' + CustLogonID + ''' logged on. Client Key = ' + IntToStr(ClientKey));

    SetComplete; { OK. }
    Result := cOKResult;
  except on E: Exception do begin
      SetAbort; { Problem encountered. }
      dmCustomers.QryCustomerMatchLogon.Close;
      strDebug := 'TCustomers.Logon() - ' + E.Message;
      Result := cErrResult;
    end;
  end;

  { Write out to event log. }
  if strDebug <> '' then
    TServerUtils.WriteToDelphiBankLog(strDebug);
end;

{ ---------------------------------------------------------------------------- }
initialization
  TAutoObjectFactory.Create(ComServer, TCustomers2, Class_Customers2,
    ciMultiInstance, tmApartment);

finalization
  TServerUtils.WriteToDelphiBankLog('Customers module finalization...');

end.
