{
  Name    :   ServerUtils.pas

  Purpose :   Provides support code for the DelphiBankServer2 component.
              Includes exception handling and log writing code.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.
              You may copy, modify or use any part of this source code
               provided the author is credited for his work.

  Author  :   Paul M. Fairhurst, October 1998. paul@c-s-c.demon.co.uk
}
unit ServerUtils;

interface

uses SysUtils;

const
  cOKResult  : Integer = 0;
  cErrResult : Integer = -1;

  cLoggingEnabled : Boolean = True; { Turn logging on or off. }

type  
  TServerUtils = class (TObject)
  public
    class procedure HandleException(Sender: TObject; E: Exception);
    class procedure WriteToDelphiBankLog(AString : String);
  end;

implementation

uses Forms;

const
  cLogFile : String = 'C:\DelphiBankServer3Log.txt';

class procedure TServerUtils.WriteToDelphiBankLog(AString : String);
var
  F : TextFile;
begin
  if not cLoggingEnabled then Exit;
  
  { Write out to event log. }
  AssignFile(F, cLogFile);
  try
    Append(F); { Open file if it exists. }
  except
    Rewrite(F); { Create new one. }
  end;

  try
    Writeln(F, DateTimeToStr(Now)+ ' - ' + AString);
  except
  end;

  try
    CloseFile(F);
  except
  end;
end;

class procedure TServerUtils.HandleException(Sender: TObject; E: Exception);
begin
  WriteToDelphiBankLog('TServerUtils.HandleException - ' + E.Message);
end;

initialization
  Application.OnException := TServerUtils.HandleException;

finalization
  Application.OnException := nil;

end.
