unit ShellNotifyTestForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, kbShellNotify;

type
  TfmShellNotifyTestForm = class(TForm)
    snNotifier: TkbShellNotify;
    lvwEventList: TListView;
    procedure snNotifierAnyEvent(Sender: TObject;
      EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
      IsInterrupt: Boolean);
    procedure snNotifierAssociationChanged(Sender: TObject;
      IsInterrupt: Boolean);
    procedure snNotifierAttributesChanged(Sender: TObject;
      PIDL: Pointer; Path: String; IsInterrupt: Boolean);
    procedure snNotifierDiskEvent(Sender: TObject;
      EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
      IsInterrupt: Boolean);
    procedure snNotifierDriveAdded(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierDriveRemoved(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierExtendedEvent(Sender: TObject;
      EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
      IsInterrupt: Boolean);
    procedure snNotifierFolderCreated(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierFolderDeleted(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierFolderRenamed(Sender: TObject;
      OldPIDL: Pointer; OldPath: String; NewPIDL: Pointer;
      NewPath: String; IsInterrupt: Boolean);
    procedure snNotifierFolderUpdated(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierFreespaceChanged(Sender: TObject;
      PIDL: Pointer; Path: String; IsInterrupt: Boolean);
    procedure snNotifierGlobalEvent(Sender: TObject;
      EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
      IsInterrupt: Boolean);
    procedure snNotifierImageUpdated(Sender: TObject; Index: Integer;
      IsInterrupt: Boolean);
    procedure snNotifierItemCreated(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierItemDeleted(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierItemRenamed(Sender: TObject; OldPIDL: Pointer;
      OldPath: String; NewPIDL: Pointer; NewPath: String;
      IsInterrupt: Boolean);
    procedure snNotifierItemUpdated(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierMediaInserted(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierMediaRemoved(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierNetworkDriveAdded(Sender: TObject;
      PIDL: Pointer; Path: String; IsInterrupt: Boolean);
    procedure snNotifierResourceShared(Sender: TObject; PIDL: Pointer;
      Path: String; IsInterrupt: Boolean);
    procedure snNotifierResourceUnshared(Sender: TObject;
      PIDL: Pointer; Path: String; IsInterrupt: Boolean);
    procedure snNotifierServerDisconnected(Sender: TObject;
      PIDL: Pointer; Path: String; IsInterrupt: Boolean);
  private
    procedure InsertEventIntoList(Event: String; FirstArgument: String; SecondArgument: String; IsInterrupt: Boolean);
  public
    { Public declarations }
  end;

var
  fmShellNotifyTestForm: TfmShellNotifyTestForm;

implementation

{$R *.DFM}


procedure TfmShellNotifyTestForm.InsertEventIntoList(Event: String; FirstArgument: String; SecondArgument: String; IsInterrupt: Boolean);
var
  ListItem: TListItem;
begin
  ListItem := Self.lvwEventList.Items.Add;
  ListItem.Caption := Event;
  ListItem.SubItems.Add(FirstArgument);
  ListItem.SubItems.Add(SecondArgument);
  if (IsInterrupt) then begin
    ListItem.SubItems.Add('Yes');
  end  {if}
  else begin
    ListItem.SubItems.Add('No');
  end; {else}
end;


procedure TfmShellNotifyTestForm.snNotifierAnyEvent(Sender: TObject;
  EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
  IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Any Event', Format('%p', [PIDL1]), Format('%p', [PIDL2]), IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierAssociationChanged(Sender: TObject;
  IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Association Changed', EmptyStr, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierAttributesChanged(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Attributes Changed', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierDiskEvent(Sender: TObject;
  EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
  IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Disk Event', Format('%p', [PIDL1]), Format('%p', [PIDL2]), IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierDriveAdded(Sender: TObject; PIDL: Pointer;
  Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Drive Added', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierDriveRemoved(Sender: TObject; PIDL: Pointer;
  Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Drive Removed', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierExtendedEvent(Sender: TObject;
  EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
  IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Extended Event', Format('%p', [PIDL1]), Format('%p', [PIDL2]), IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierFolderCreated(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Folder Created', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierFolderDeleted(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Folder Deleted', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierFolderRenamed(Sender: TObject;
  OldPIDL: Pointer; OldPath: String; NewPIDL: Pointer;
  NewPath: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Folder Renamed', OldPath, NewPath, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierFolderUpdated(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Folder Updated', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierFreespaceChanged(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Freespace Changed', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierGlobalEvent(Sender: TObject;
  EventType: TkbShellNotifyEventType; PIDL1, PIDL2: Pointer;
  IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Global Event', Format('%p', [PIDL1]), Format('%p', [PIDL2]), IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierImageUpdated(Sender: TObject; Index: Integer;
  IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Image Updated', 'Index ' + IntToStr(Index), EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierItemCreated(Sender: TObject; PIDL: Pointer;
  Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Item Created', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierItemDeleted(Sender: TObject; PIDL: Pointer;
  Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Item Deleted', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierItemRenamed(Sender: TObject;
  OldPIDL: Pointer; OldPath: String; NewPIDL: Pointer;
  NewPath: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Item Renamed', OldPath, NewPath, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierItemUpdated(Sender: TObject; PIDL: Pointer;
  Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Item Updated', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierMediaInserted(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Media Inserted', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierMediaRemoved(Sender: TObject; PIDL: Pointer;
  Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Media Removed', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierNetworkDriveAdded(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Network Drive Added', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierResourceShared(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Resource Shared', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierResourceUnshared(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Resource Unshared', Path, EmptyStr, IsInterrupt);
end;

procedure TfmShellNotifyTestForm.snNotifierServerDisconnected(Sender: TObject;
  PIDL: Pointer; Path: String; IsInterrupt: Boolean);
begin
  Self.InsertEventIntoList('Server Disconnected', Path, EmptyStr, IsInterrupt);
end;

end.
