{Copyright:      Hagen Reddmann  mailto:HaReddmann@AOL.COM
 Author:         Hagen Reddmann
 Remarks:        freeware, but this Copyright must included
 known Problems: none
 Version:        2.1,  Part I from Delphi Encryption Compendium
                 Delphi 2-4, designed and testet under D3 and D4
 Description:    Low Level API Routines for Cipher and Hash
                 }
unit DEC_api;

interface

uses Windows;

type
  TCipherHandle = Integer;
  THashHandle   = Integer;
  TEnumProc     = function(ID: Integer; Name: PChar; MaxKeySize: Integer): Bool; stdcall;

function Cipher_GetID(Name: PChar): Integer; stdcall;
function Cipher_Create(ID: Integer): TCipherHandle; stdcall;
function Cipher_Delete(Handle: TCipherHandle): Integer; stdcall;
function Cipher_Encode(Handle: TCipherHandle; Source, Dest: PChar; Size: Integer): Integer; stdcall;
function Cipher_Decode(Handle: TCipherHandle; Source, Dest: PChar; Size: Integer): Integer; stdcall;
function Cipher_Init(Handle: TCipherHandle; Key: Pointer; KeyLen: Integer; IVector: Pointer): Integer; stdcall;
function Cipher_InitKey(Handle: TCipherHandle; Key: PChar; IVector: Pointer): Integer; stdcall;
function Cipher_Done(Handle: TCipherHandle): Integer; stdcall;
function Cipher_Protect(Handle: TCipherHandle): Integer; stdcall;
function Cipher_GetMaxKeySize(Handle: TCipherHandle): Integer; stdcall;
function Cipher_SetHash(Handle: TCipherHandle; Hash_ID: Integer): Integer; stdcall;
function Cipher_GetHash(Handle: TCipherHandle): Integer; stdcall;
procedure Cipher_EnumNames(Proc: TEnumProc); stdcall;

function Hash_GetID(Name: PChar): Integer; stdcall;
function Hash_Create(ID: Integer): THashHandle; stdcall;
function Hash_Delete(Handle: THashHandle): Integer; stdcall;
function Hash_Init(Handle: THashHandle): Integer; stdcall;
function Hash_Done(Handle: THashHandle; Digest: PChar; DigestSize: Integer): Integer; stdcall;
function Hash_Update(Handle: THashHandle; Source: PChar; SourceLen: Integer): Integer; stdcall;
function Hash_GetMaxDigestSize(Handle: THashHandle): Integer; stdcall;
procedure Hash_EnumNames(Proc: TEnumProc); stdcall;

{using Hash

var
  Handle: THashHandle;
  Digest: Pointer;
  DigestSize: Integer;
begin
  Handle := Hash_Create(2);  // for SHA1
  try
    DigestSize := Hash_GetMaxDigestSize(Handle);
    Digest := GetMem(DigestSize);
    Hash_Init(Handle);
    Hash_Update(Handle, Data, DataSize);
    Hash_Done(Handle, Digest, DigestSize);
  finally
    Hash_Delete(Handle);
    FreeMem(Handle, DigestSize);
  end;
end;

using Cipher

var
  Handle: TCipherHandle;
begin
  Handle := Cipher_Create(1); // for Blowfish
  try
    Cipher_SetHash(Handle, 13);  // for Tiger, default is SHA1
    Cipher_InitKey(Handle, 'Password', nil);
    Cipher_Encode(Handle, Data, Data, DataSize);
  finally
    Cipher_Delete(Handle);
  end;
end;

}
implementation

uses Cipher, Hash, Classes;

const
  FCipherList: TStringList = nil;
  FHashList: TStringList = nil;

function IsObject(AObject: Integer; AClass: TClass): Boolean;
var
  E: Pointer;
begin
  Result := False;
  if AObject = 0 then Exit;
  E := ExceptionClass;
  ExceptionClass := nil;
  try
    if (PInteger(PChar(AObject) - SizeOf(Integer))^ and $00000002 = $00000002) and
       (TObject(AObject) is AClass) then Result := True;
  except
  end;
  ExceptionClass := E;
end;

function Cipher_GetID(Name: PChar): Integer;
begin
  Result := FCipherList.IndexOf(Name);
end;

function Cipher_Create(ID: Integer): TCipherHandle;
begin
  if (ID >= 0) and (ID < FCipherList.Count) then
  begin
    if TCipherClass(FCipherList.Objects[ID]).SelfTest then
      Result := Integer(TCipherClass(FCipherList.Objects[ID]).Create)
    else Result := -2;
  end else Result := -1;
end;

function Cipher_Delete(Handle: TCipherHandle): Integer;
begin
  Result := 0;
  if IsObject(Handle, TCipher) then TCipher(Handle).Free else Result := -1
end;

function Cipher_Encode(Handle: TCipherHandle; Source, Dest: PChar; Size: Integer): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    if TCipher(Handle).Initialized then
    begin
      TCipher(Handle).EncodeBuffer(Source^, Dest^, Size);
      Result := 0;
    end else Result := -2;
  end else Result := -1;
end;

function Cipher_Decode(Handle: TCipherHandle; Source, Dest: PChar; Size: Integer): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    if TCipher(Handle).Initialized then
    begin
      TCipher(Handle).DecodeBuffer(Source^, Dest^, Size);
      Result := 0;
    end else Result := -2;
  end else Result := -1;
end;

function Cipher_Init(Handle: TCipherHandle; Key: Pointer; KeyLen: Integer; IVector: Pointer): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    TCipher(Handle).Init(Key^, KeyLen, IVector);
    Result := 0;
  end else Result := -1;
end;

function Cipher_InitKey(Handle: TCipherHandle; Key: PChar; IVector: Pointer): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    TCipher(Handle).InitKey(Key, IVector);
    Result := 0;
  end else Result := -1;
end;

function Cipher_Done(Handle: TCipherHandle): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    TCipher(Handle).Done;
    Result := 0;
  end else Result := -1;
end;

function Cipher_Protect(Handle: TCipherHandle): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    TCipher(Handle).Protect;
    Result := 0;
  end else Result := -1;
end;

function Cipher_GetMaxKeySize(Handle: TCipherHandle): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then Result := TCipher(Handle).MaxKeySize
    else Result := -1;
end;

function Cipher_SetHash(Handle: TCipherHandle; Hash_ID: Integer): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
  begin
    if (Hash_ID >= 0) and (Hash_ID < FHashList.Count) then
      TCipher(Handle).HashClass := THashClass(FHashList.Objects[Hash_ID]);
  end else Result := -1;
end;

function Cipher_GetHash(Handle: TCipherHandle): Integer;
begin
  Result := -3;
  if IsObject(Handle, TCipher) then
    Result := FHashList.IndexOfObject(Pointer(TCipher(Handle).HashClass))
  else Result := -1;
end;

procedure Cipher_EnumNames(Proc: TEnumProc);
var
  I: Integer;
begin
  for I := 0 to FCipherList.Count-1 do
    Proc(I, PChar(FCipherList[I]), TCipherClass(FCipherList.Objects[I]).MaxKeySize);
end;

function Hash_GetID(Name: PChar): Integer;
begin
  Result := FHashList.IndexOf(Name);
end;

function Hash_Create(ID: Integer): THashHandle;
begin
  if (ID >= 0) and (ID < FHashList.Count) then
  begin
    if THashClass(FHashList.Objects[ID]).SelfTest then
      Result := Integer(THashClass(FHashList.Objects[ID]).Create)
    else Result := -2;
  end else Result := -1;
end;

function Hash_Delete(Handle: THashHandle): Integer;
begin
  Result := 0;
  if IsObject(Handle, THash) then THash(Handle).Free else Result := -1
end;

function Hash_Init(Handle: THashHandle): Integer;
begin
  Result := -3;
  if IsObject(Handle, THash) then
  begin
    THash(Handle).Init;
    Result := 0;
  end else Result := -1
end;

function Hash_Done(Handle: THashHandle; Digest: PChar; DigestSize: Integer): Integer;
begin
  Result := -3;
  if IsObject(Handle, THash) then
  begin
    THash(Handle).Done;
    FillChar(Digest^, DigestSize, 0);
    if DigestSize > THash(Handle).DigestKeySize then DigestSize := THash(Handle).DigestKeySize;
    Move(THash(Handle).DigestKey^, Digest^, DigestSize);
    Result := 0;
  end else Result := -1
end;

function Hash_Update(Handle: THashHandle; Source: PChar; SourceLen: Integer): Integer;
begin
  Result := -3;
  if IsObject(Handle, THash) then
  begin
    THash(Handle).Calc(Source^, SourceLen);
    Result := 0;
  end else Result := -1
end;

function Hash_GetMaxDigestSize(Handle: THashHandle): Integer;
begin
  Result := -3;
  if IsObject(Handle, THash) then Result := THash(Handle).DigestKeySize
    else Result := -1
end;

procedure Hash_EnumNames(Proc: TEnumProc);
var
  I: Integer;
begin
  for I := 0 to FHashList.Count-1 do
    Proc(I, PChar(FHashList[I]), THashClass(FHashList.Objects[I]).DigestKeySize);
end;


exports
  Cipher_GetID,
  Cipher_Create,
  Cipher_Delete,
  Cipher_Encode,
  Cipher_Decode,
  Cipher_Init,
  Cipher_InitKey,
  Cipher_Done,
  Cipher_Protect,
  Cipher_GetMaxKeySize,
  Cipher_SetHash,
  Cipher_GetHash,
  Cipher_EnumNames,

  Hash_GetID,
  Hash_Create,
  Hash_Delete,
  Hash_Init,
  Hash_Done,
  Hash_Update,
  Hash_GetMaxDigestSize,
  Hash_EnumNames;

initialization
  FCipherList := TStringList.Create;
  FHashList := TStringList.Create;
  with FCipherList do
  begin
    AddObject('3Way', Pointer(TCipher_3Way));
    AddObject('Blowfish', Pointer(TCipher_Blowfish));
    AddObject('Cast128', Pointer(TCipher_Cast128));
    AddObject('Cast256', Pointer(TCipher_Cast256));
    AddObject('FROG', Pointer(TCipher_FROG));
    AddObject('Gost', Pointer(TCipher_Gost));
    AddObject('IDEA', Pointer(TCipher_IDEA));
    AddObject('Mars', Pointer(TCipher_Mars));
    AddObject('Misty 1', Pointer(TCipher_Misty));
    AddObject('Q128', Pointer(TCipher_Q128));
    AddObject('RC2', Pointer(TCipher_RC2));
    AddObject('RC4', Pointer(TCipher_RC4));
    AddObject('RC5', Pointer(TCipher_RC5));
    AddObject('RC6', Pointer(TCipher_RC6));
    AddObject('Rijndael', Pointer(TCipher_Rijndael));
    AddObject('SAFER-K40', Pointer(TCipher_SAFER_K40));
    AddObject('SAFER-SK40', Pointer(TCipher_SAFER_SK40));
    AddObject('SAFER-K64', Pointer(TCipher_SAFER_K64));
    AddObject('SAFER-SK64', Pointer(TCipher_SAFER_SK64));
    AddObject('SAFER-K128', Pointer(TCipher_SAFER_K128));
    AddObject('SAFER-SK128', Pointer(TCipher_SAFER_SK128));
    AddObject('SCOP', Pointer(TCipher_SCOP));
    AddObject('Shark', Pointer(TCipher_Shark));
    AddObject('Skipjack', Pointer(TCipher_Skipjack));
    AddObject('TEA', Pointer(TCipher_TEA));
    AddObject('TEAN', Pointer(TCipher_TEAN));
    AddObject('Twofish', Pointer(TCipher_Twofish));
  end;
  with FHashList do
  begin
    AddObject('Message Digest 4', Pointer(THash_MD4));
    AddObject('Message Digest 5', Pointer(THash_MD5));
    AddObject('Secure Hash Algorithm 1', Pointer(THash_SHA1));
    AddObject('Ripe Message Digest 128', Pointer(THash_RipeMD128));
    AddObject('Ripe Message Digest 160', Pointer(THash_RipeMD160));
    AddObject('Ripe Message Digest 256', Pointer(THash_RipeMD256));
    AddObject('Ripe Message Digest 320', Pointer(THash_RipeMD320));
    AddObject('Haval-256', Pointer(THash_Haval256));
    AddObject('Haval-224', Pointer(THash_Haval224));
    AddObject('Haval-192', Pointer(THash_Haval192));
    AddObject('Haval-160', Pointer(THash_Haval160));
    AddObject('Haval-128', Pointer(THash_Haval128));
    AddObject('Snefru-256', Pointer(THash_Snefru));
    AddObject('Tiger', Pointer(THash_Tiger));
    AddObject('XOR-16', Pointer(THash_XOR16));
    AddObject('XOR-32', Pointer(THash_XOR32));
    AddObject('CRC-32', Pointer(THash_CRC32));
  end;
finalization
  FCipherList.Free;
  FHashList.Free;
end.
