unit XWMod01Intf;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus, XWModExpt, xwViewFrm, ComCtrls, Mod01MgrFrm;

type

  TXWMod01Interface = class(TXWModuleInterface)
  private
    FViewCaptions: TStringList;
    FModuleIcon: TIcon;
    FModuleToolBar: TToolBar;
    FModuleMgrForm: TMod01MgrForm;
  private
    procedure ModuleViewsMenuOnClick(Sender: TObject);

  protected
    function GetModuleCaption: String; override;
    function GetModuleDescription: String; override;
    function GetModuleFileName: String; override;
    function GetModuleIcon: TIcon; override;
    function GetModuleToolBar: TToolBar; override;
    function GetModuleLevelMenu: TPopupMenu; override;

    // View Methods

    function GetViewCount: Integer; override;
    function GetViewDescriptions: TStringList; override;
    function GetViewCaptions: TStringList; override;
    function GetCurrentView: TViewForm; override;

  public
    constructor Create(AParentWindow: TWinControl; AModuleLevelMenuParent: TMenuItem;
      AModuleLevelTBParent: TPanel); override;
    destructor Destroy; override;

    function CanSwitchModules: Boolean; override;
    function ModuleLogOn(const AUserName, APassword: String;
       const LenderID: Integer): Boolean; override;
    function GetView(const AIndex: Integer; AWithParent: Boolean): TViewForm; override; 
    procedure CloseView; override;
  end;

implementation
uses DemoFrm1, DemoFrm2;

const
  {: Specify a valid form count }
  FViewCount = 2;

{ TXWMod01Interface }

function TXWMod01Interface.CanSwitchModules: Boolean;
begin
  // pass true as a default for this example.
  Result := True;
end;

procedure TXWMod01Interface.CloseView;
begin
  if FCurrentView <> nil then
  begin
    UnloadViewLevelMenu;
    FCurrentView.Free;
    FCurrentView := nil;
  end;
end;

constructor TXWMod01Interface.Create(AParentWindow: TWinControl;
  AModuleLevelMenuParent: TMenuItem; AModuleLevelTBParent: TPanel);
var
  MenuItem: TMenuItem;
  i: integer;
begin
  inherited Create(AParentWindow, AModuleLevelMenuParent, AModuleLevelTBParent);
  FCurrentView := nil;

  FModuleIcon := TIcon.Create;
  // The icon is stored in the resource file for the module.  Retrieve it.
  FModuleIcon.Handle := LoadIcon(hInstance, 'MODULE1');

  // Create a TStringlist of Captions for each view in the system
  FViewCaptions := TStringList.Create;
  FViewCaptions.Add('Module 1 Demo Form 1');
  FViewCaptions.Add('Module 1 Demo Form 2');

  FModuleLevelMenu := TPopupMenu.Create(nil);

  // Add Menu items for each module.
  for i := 0 to FViewCaptions.Count - 1 do
  begin
    MenuItem := TMenuItem.Create(FModuleLevelMenu);
    MenuItem.Caption := FViewCaptions[i];
    MenuItem.OnClick := ModuleViewsMenuOnClick;
    FModuleLevelMenu.Items.Add(MenuItem);
  end;

  { Create the managment form used for centralizing some logic and for design-time
    manipulation of visual components used with this module. Otherwise, I'd have to
    programatically create these component and set there properties.  It's easier to
    just use them off this hidden form. }

  FModuleMgrForm := TMod01MgrForm.Create(Application);

end;

destructor TXWMod01Interface.Destroy;
begin
  FModuleLevelMenu.Free;
  FModuleIcon.Free;
  FViewCaptions.Free;
  if FCurrentView <> nil then
    FCurrentView.Free;
  FModuleMgrForm.Free;
  inherited;
end;

function TXWMod01Interface.GetModuleCaption: String;
begin
  Result := Format('Module %s', [ClassName]);
end;

function TXWMod01Interface.GetModuleDescription: String;
begin
  Result := 'TXWMod01Interface Module Description';
end;

function TXWMod01Interface.GetModuleFileName: String;
begin
  Result := 'XWMod01Intf.pas';
end;

function TXWMod01Interface.GetModuleIcon: TIcon;
begin
  Result := FModuleIcon;
end;

function TXWMod01Interface.GetModuleToolBar: TToolBar;
begin
  Result := FModuleMgrForm.ToolBar1;
end;

function TXWMod01Interface.GetModuleLevelMenu: TPopupMenu;
begin
  Result := FModuleLevelMenu;
end;

function TXWMod01Interface.GetViewCaptions: TStringList;
begin
  Result := FViewCaptions;
end;

function TXWMod01Interface.GetViewCount: Integer;
begin
  Result := FViewCount;
end;

function TXWMod01Interface.GetViewDescriptions: TStringList;
begin
  Result := nil;
end;

function TXWMod01Interface.ModuleLogOn(const AUserName, APassword: String;
  const LenderID: Integer): Boolean;
begin
  // This could provide logic to perform a database logon. 
  Result := True;
end;

procedure TXWMod01Interface.ModuleViewsMenuOnClick(Sender: TObject);
begin
  ShowMessage(Sender.ClassName);
end;

function TXWMod01Interface.GetView(const AIndex: Integer;
  AWithParent: Boolean): TViewForm;
begin
  if ((AIndex >= FViewCount) or (AIndex < 0)) then
     raise Exception.Create('Invalid view index');

  if FCurrentView <> nil then
    FCurrentView.Free;

  { This method will create the view as both a child window and as a regular form
    depending on the value of AWithParent }
  if (AWithParent = True) and (FViewParentWindow <> nil) then
  begin
    case AIndex of
      0: FCurrentView := TDemoForm1.Create(Application, FViewParentWindow);
      1: FCurrentView := TDemoForm2.Create(Application, FViewParentWindow);
    end; // case
  end
  else begin
    case AIndex of
      0: FCurrentView := TDemoForm1.Create(Application);
      1: FCurrentView := TDemoForm2.Create(Application);
    end; // case
  end;
  Result := FCurrentView;
end;

function TXWMod01Interface.GetCurrentView: TViewForm;
begin
  Result := FCurrentView;
end;

initialization

  RegisterClass(TXWMod01Interface);

finalization

  UnRegisterClass(TXWMod01Interface);

end.
