unit XWMod02Intf;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, XWModExpt, xwViewFrm, DemoDataMod, ComCtrls, Mod02MgrFrm;

type

  TXWMod02Interface = class(TXWModuleInterface)
  private
    FViewCaptions: TStringList;
    FModuleIcon: TIcon;
    FModDataModule: TDataModule1;
    FModuleMgrForm: TMod02MgrForm;
  protected
    function GetModuleCaption: String; override;
    function GetModuleDescription: String; override;
    function GetModuleFileName: String; override;
    function GetModuleIcon: TIcon; override;
    function GetModuleToolBar: TToolBar; override;

    function GetViewCount: Integer; override;
    function GetViewDescriptions: TStringList; override;
    function GetViewCaptions: TStringList; override;
    function GetCurrentView: TViewForm; override;
  public
    constructor Create(AParentWindow: TWinControl; AModuleLevelMenuParent: TMenuItem;
      AModuleLevelTBParent: TPanel); override;
    destructor Destroy; override;

    function CanSwitchModules: Boolean; override;
    function ModuleLogOn(const AUserName, APassword: String;
       const LenderID: Integer): Boolean; override;
    function GetView(const AIndex: Integer; AWithParent: Boolean): TViewForm; override;
    procedure CloseView; override;

  end;

implementation
uses DemoV1Frm, DemoV2Frm, DemoV3Frm;

{ TXWMod02Interface }


const
  {: Specify a valid form count }
  FViewCount = 3;

function TXWMod02Interface.CanSwitchModules: Boolean;
begin
 Result := True;
end;

constructor TXWMod02Interface.Create(AParentWindow: TWinControl;
  AModuleLevelMenuParent: TMenuItem; AModuleLevelTBParent: TPanel);

begin
  inherited Create(AParentWindow, AModuleLevelMenuParent,
    AModuleLevelTBParent);
  FCurrentView := nil;

  FModuleIcon := TIcon.Create;
  FModuleIcon.Handle := LoadIcon(hInstance, 'MODULE2');

  // Create a TStringlist of Captions for each view in the system
  FViewCaptions := TStringList.Create;
  FViewCaptions.Add('Module 2 Demo Form 1');
  FViewCaptions.Add('Module 2 Demo Form 2');
  FViewCaptions.Add('Module 2 Demo Form 3');

  FModDataModule := TDataModule1.Create(nil);

  FModuleMgrForm := TMod02MgrForm.Create(Application);
end;

destructor TXWMod02Interface.Destroy;
begin
  FModuleIcon.Free;
  FViewCaptions.Free;
  FModDataModule.Free;
  if FCurrentView <> nil then
    FCurrentView.Free;

  FModuleMgrForm.Free;

  inherited;
end;

function TXWMod02Interface.GetModuleCaption: String;
begin
  Result := Format('Module %s', [ClassName]);
end;

function TXWMod02Interface.GetModuleDescription: String;
begin
  Result := 'TXWMod02Interface Module Description';
end;

function TXWMod02Interface.GetModuleFileName: String;
begin
  Result := 'XWMod02Intf.pas';
end;

function TXWMod02Interface.GetModuleIcon: TIcon;
begin
  Result := FModuleIcon;
end;

function TXWMod02Interface.GetModuleToolBar: TToolBar;
begin
  Result := FModuleMgrForm.ToolBar1;
end;

function TXWMod02Interface.GetView(const AIndex: Integer;
  AWithParent: Boolean): TViewForm;
begin

  if ((AIndex >= FViewCount) or (AIndex < 0)) then
     raise Exception.Create('Invalid view index');

  if FCurrentView <> nil then
    FCurrentView.Free;

  if (AWithParent = True) and (FViewParentWindow <> nil) then
  begin
    case AIndex of
      0: FCurrentView := TDemo1Form.Create(Application, FViewParentWindow);
      1: FCurrentView := TDemo2Form.Create(Application, FViewParentWindow);
      2: FCurrentView := TDemo3Form.Create(Application, FViewParentWindow);
    end; // case
  end
  else begin
    case AIndex of
      0: FCurrentView := TDemo1Form.Create(Application);
      1: FCurrentView := TDemo2Form.Create(Application);
      2: FCurrentView := TDemo3Form.Create(Application);
    end; // case
  end;
  Result := FCurrentView;
end;


procedure TXWMod02Interface.CloseView;
begin
  if FCurrentView <> nil then
  begin
    FCurrentView.Free;
    FCurrentView := nil;
  end;
end;

function TXWMod02Interface.GetViewCaptions: TStringList;
begin
  Result := FViewCaptions;
end;

function TXWMod02Interface.GetViewCount: Integer;
begin
  Result := FViewCount;
end;

function TXWMod02Interface.GetViewDescriptions: TStringList;
begin
  Result := nil;
end;

function TXWMod02Interface.ModuleLogOn(const AUserName, APassword: String;
  const LenderID: Integer): Boolean;
begin
  Result := True;
end;

function TXWMod02Interface.GetCurrentView: TViewForm;
begin
  Result := FCurrentView;
end;
initialization

  RegisterClass(TXWMod02Interface);

finalization

  UnRegisterClass(TXWMod02Interface);

end.
