unit xwViewFrm;
{: Defines the view descendant for each module. Each view shall return:

   Caption     - The caption to be displayed no the Tab
   Icon        - An icon for display
   Description - A descripton of each view
   Menu        - A TPopupMenu containing View specific information.
   Toolbar     - A View level Toolbar for the shell application 

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Menus;

type
  TViewForm = class(TForm)
  private
    { Private declarations }
    FAsChild: Boolean;        // Form created as child indicator
    FTempParent: TWinControl; // Temporary parent window
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
    function CanChange: Boolean; virtual;
    function GetViewDescription: String; virtual;
    function GetViewMenu: TPopupMenu; virtual;
    function GetViewToolBar: TToolBar; virtual;
  public
    { Public declarations }
    // Constructor to create as a normal form.
    constructor Create(AOwner: TComponent); overload; override;
    // Constructor to create as a child form.
    constructor Create(AOwner: TComponent; AParent: TWinControl); reintroduce; overload;
    property ViewDescription: String read GetViewDescription;
    property ViewMenu: TPopupMenu read GetViewMenu;
    property ViewToolbar: TToolBar read GetViewToolBar;
  end;

implementation

{$R *.DFM}

{ TViewForm }

constructor TViewForm.Create(AOwner: TComponent);
begin
  FAsChild := False;
  inherited Create(AOwner);
end;

constructor TViewForm.Create(AOwner: TComponent; AParent: TWinControl);
begin
  FAsChild := True;
  FTempParent := aParent;
  inherited Create(AOwner);
end;

procedure TViewForm.Loaded;
begin
  { Set parameters to allow this form to appear as a child form to another window. }
  inherited;
  if FAsChild then
  begin
    align := alClient;
    BorderStyle := bsNone;
    BorderIcons := [];
    Parent := FTempParent;
    Position := poDefault;
  end;
end;

procedure TViewForm.CreateParams(var Params: TCreateParams);
begin
  Inherited CreateParams(Params);
  if FAsChild then
    Params.Style := Params.Style or WS_CHILD;
end;

function TViewForm.GetViewMenu: TPopupMenu;
begin
  Result := nil;
end;

function TViewForm.GetViewToolBar: TToolBar;
begin
  Result := nil; // Default
end;

function TViewForm.CanChange: Boolean;
begin
  Result := True; // Default
end;

function TViewForm.GetViewDescription: String;
begin
  Result := EmptyStr; // Default
end;

end.


