
{*******************************************************}
{                                                       }
{       AnimatedMenus/2000                              }
{       T_AM2000_MenuDesigner Component Unit            }
{                                                       }
{       Copyright (c) 1998-1999 by Andrew Cher          }
{                                                       }
{*******************************************************}


unit am2000designer;

interface


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, CommCtrl, ComCtrls, ExtCtrls, DsgnIntf, Buttons, Menus, Registry,
  LibHelp,
  {$IFNDEF VER90} ExtDlgs, {$ENDIF}
  {$IFDEF VER120} ImgList, {$ENDIF}
  am2000menuitem, am2000popupmenu, am2000mainmenu, am2000, am2000cache;

type
  // menu designer
  T_AM2000_MenuDesigner = class(TPropertyEditor)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;

  end;

  // component editor
  T_AM2000_ComponentEditor = class(TComponentEditor)
  public
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): String; override;
    procedure ExecuteVerb(Index: Integer); override;
    procedure Edit; override;

  end;

  // dialog for menu designer
  T_AM2000_MenuDesignerDlg = class(TForm)
    MenuTree: TTreeView;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    ButtonNewMenuItem: TSpeedButton;
    ButtonNewSubMenu: TSpeedButton;
    Bevel1: TBevel;
    ButtonDelete: TSpeedButton;
    Bevel2: TBevel;
    ButtonLevelUp: TSpeedButton;
    ButtonMoveUp: TSpeedButton;
    ButtonMoveDown: TSpeedButton;
    ButtonLevelDown: TSpeedButton;
    PopupMenu20001: TPopupMenu2000;
    DefaultBitmaps1: TMenuItem2000;
    N2: TMenuItem2000;
    None1: TMenuItem2000;
    ButtonArray1: TMenuItem2000;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    NewItem1: TMenuItem2000;
    NewSubmenuItem1: TMenuItem2000;
    MenuItem3: TMenuItem2000;
    ImageListBitmaps1: TMenuItem2000;
    None2: TMenuItem2000;
    ButtonArray2: TMenuItem2000;
    DefaultBitmapsMenu: TPopupMenu2000;
    ImageListMenu: TPopupMenu2000;
    LoadBitmapFromFile1: TMenuItem2000;
    SaveBitmapToFile1: TMenuItem2000;
    AM2000MenuDesigner4: TMenuItem2000;
    ExpandAll1: TMenuItem2000;
    CollapseAll1: TMenuItem2000;
    AM2000MenuDesigner8: TMenuItem2000;
    Open1: TMenuItem2000;
    SaveAs1: TMenuItem2000;
    InsStdMnu: TMenuItem2000;
    AM2000MenuDesigner2: TMenuItem2000;
    DefaultBitmaps: TImageList;
    AM2000MenuDesigner5: TMenuItem2000;
    Delete1: TMenuItem2000;
    Bevel3: TBevel;
    ButtonStayOnTop: TSpeedButton;
    procedure MenuTreeChange(Sender: TObject; Node: TTreeNode);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure CollapseAll1Click(Sender: TObject);
    procedure ExpandAll1Click(Sender: TObject);
    procedure ButtonNewMenuItemClick(Sender: TObject);
    procedure ButtonNewSubMenuClick(Sender: TObject);
    procedure MenuTreeEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure ButtonDeleteClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ButtonMoveDownClick(Sender: TObject);
    procedure ButtonMoveUpClick(Sender: TObject);
    procedure ButtonLevelUpClick(Sender: TObject);
    procedure ButtonLevelDownClick(Sender: TObject);
    procedure LoadFromFile2Click(Sender: TObject);
    procedure SaveToFile2Click(Sender: TObject);
    procedure SaveAsTemplate1Click(Sender: TObject);
    procedure MenuTreeMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LoadBitmapFromFileClick(Sender: TObject);
    procedure PopupMenu20001Popup(Sender: TObject);
    procedure SaveBitmapToFileClick(Sender: TObject);
    procedure None1Click(Sender: TObject);
    procedure ButtonArray1Click(Sender: TObject);
    procedure ButtonArray2Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AM2000MenuDesigner6Click(Sender: TObject);
    procedure AM2000MenuDesigner7Click(Sender: TObject);
    procedure MenuTreeEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure ButtonStayOnTopClick(Sender: TObject);
    procedure ImageListMenuPopup(Sender: TObject);

  private
    TempFolder: String;
    FMenu: TMenu;

{$IFNDEF VER90}
    OpenPictureDialog1: TOpenPictureDialog;
    SavePictureDialog1: TSavePictureDialog;
{$ELSE}
    OpenPictureDialog1: TOpenDialog;
    SavePictureDialog1: TSaveDialog;
{$ENDIF}

    procedure CheckConstraints;
    procedure InsertTemplateMenuClick(Sender: TObject);
    procedure SetMenu(const Value: TMenu);

  protected
    procedure Loaded; override;

  public
{$IFNDEF VER120}
    Designer: TFormDesigner;
{$ELSE}
    Designer: IFormDesigner;
{$ENDIF}
    property Menu: TMenu read FMenu write SetMenu;

    procedure UpdateCaption(MenuItem: TMenuItem);

  end;

var
  MenuDesignerDlg : T_AM2000_MenuDesignerDlg;

implementation

uses
  Consts, TypInfo,
  {$IFDEF VER90} Ole2, {$ELSE} ShlObj, ActiveX, {$ENDIF}
  am2000menubar, am2000utils, am2000const;

{$R *.DFM}

const
  MENU_REGISTRY_KEY : String = 'Software\AnimatedMenus.com\AnimatedMenus/2000\Folders';


{$IFDEF VER90}

type
  TSHItemID = record
    cb: Word;                         
    abID: array[0..0] of Byte;        
  end;

  PItemIDList = ^TItemIDList;
  TItemIDList = record
     mkid: TSHItemID;
   end;


  TBrowseInfo = record
    hwndOwner: HWND;
    pidlRoot: PItemIDList;
    pszDisplayName: PAnsiChar;  
    lpszTitle: PAnsiChar;      
    ulFlags: UINT;           
    lpfn: Pointer;
    lParam: LPARAM;         
    iImage: Integer;        
  end;


  function SHBrowseForFolder(var lpbi: TBrowseInfo): PItemIDList;
    stdcall; external 'shell32.dll' name 'SHBrowseForFolderA';
  function SHGetPathFromIDList(pidl: PItemIDList; pszPath: PChar): BOOL;
    stdcall; external 'shell32.dll' name 'SHGetPathFromIDListA';

{$ENDIF}

procedure DrawImageList(Item: TMenuItem2000; ImageList: TImageList);
var
  DX, DY, X, Y: Integer;
begin
  DX:= ImageList.Width +4;
  DY:= ImageList.Height +4;

  with Item.AsButtonArray do begin
    Count:= ImageList.Count;
    Columns:= 9;
    Rows:= Count div Columns;
    if (Count mod Columns) <> 0 then Rows:= Rows +1;

    Bitmap.Width:= Columns * DX;
    Bitmap.Height:= Rows * DY;

    with Bitmap.Canvas do begin
      Brush.Color:= clFuchsia;
      Brush.Style:= bsSolid;
      FillRect(ClipRect);
    end;

    for X:= 0 to Columns do
      for Y:= 0 to Rows do
        ImageList_Draw(ImageList.Handle, X + Y * Columns, Bitmap.Canvas.Handle,
          X * DX +2, Y * DY +2, ild_Transparent);
  end;
end;


{ T_AM2000_MenuDesigner }

procedure T_AM2000_MenuDesigner.Edit;
begin
  if not Assigned(MenuDesignerDlg) then
    MenuDesignerDlg:= T_AM2000_MenuDesignerDlg.Create(Application);

  MenuDesignerDlg.Designer:= Designer;
  MenuDesignerDlg.Menu:= TMenu(GetComponent(0));
  MenuDesignerDlg.Show;
end;

function T_AM2000_MenuDesigner.GetValue: string;
begin
  Result:= '(Menu2000)';
end;

function T_AM2000_MenuDesigner.GetAttributes: TPropertyAttributes;
begin
  Result:= [paDialog, paReadOnly];
end;


{ T_AM2000_ComponentEditor }

procedure T_AM2000_ComponentEditor.Edit;
begin
  if not Assigned(MenuDesignerDlg) then
    MenuDesignerDlg:= T_AM2000_MenuDesignerDlg.Create(Application);

  MenuDesignerDlg.Designer:= Designer;
  MenuDesignerDlg.Menu:= TMenu(Component);
  MenuDesignerDlg.Show;
end;

function T_AM2000_ComponentEditor.GetVerbCount: Integer;
begin
  Result:= inherited GetVerbCount + 1;
end;

function T_AM2000_ComponentEditor.GetVerb(Index: Integer): String;
begin
  if Index = 0
  then Result:= 'AM/2000 Menu Designer...'
  else Result:= inherited GetVerb(Index -1);
end;

procedure T_AM2000_ComponentEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then Edit;
end;


{ TMenuDesignerDlg }

procedure T_AM2000_MenuDesignerDlg.CheckConstraints;
begin
  if not Assigned(MenuTree.Selected)
  then Exit;

  with MenuTree.Selected do begin
    ButtonDelete.Enabled:= Parent <> nil;
    ButtonLevelUp.Enabled:= (Parent <> nil) and (Parent.Parent <> nil);
    ButtonMoveUp.Enabled:= (GetPrevSibling <> nil)
      or ((Parent <> nil) and (Parent.GetPrevSibling <> nil));
    ButtonMoveDown.Enabled:=  (GetNextSibling <> nil)
      or ((Parent <> nil) and (Parent.GetNextSibling <> nil));
    ButtonLevelDown.Enabled:= Parent <> nil;
    DefaultBitmaps1.Enabled:= Parent <> nil;
    ImageListBitmaps1.Enabled:= (Parent <> nil) and
      (not ButtonArray2.AsButtonArray.Bitmap.Empty);
    LoadBitmapFromFile1.Enabled:= Parent <> nil;
    SaveBitmapToFile1.Enabled:= Parent <> nil;
  end;

  if ActiveMenuBar <> nil
  then ActiveMenuBar.UpdateMenuBar(False);
end;

procedure T_AM2000_MenuDesignerDlg.MenuTreeChange(Sender: TObject;
  Node: TTreeNode);
var
  M: TMenuItem2000;
  S: String;
  B: Boolean;
begin
  if not (Assigned(Node) and Assigned(Node.Data)) then Exit;

  CheckConstraints;

  try
    S:= '';
    if TObject(Node.Data) is TMenuItem2000 then begin
      M:= TMenuItem2000(Node.Data);
      while Assigned(M) and (M.Caption <> '') do begin
        if S <> ''then S:= ' | ' + S;
        S:= M.Caption + S;
        M:= M.Parent;
      end;
    end;

    StatusBar.SimpleText:= ' ' + S;

    // select menu item into object inspector
    if Designer <> nil then
      if Node.Parent <> nil
      then Designer.SelectComponent(TMenuItem2000(Node.Data))
      else Designer.SelectComponent(Menu);

    // set item index at ButtonArray1
    if Assigned(Node.Data) then
      with TMenuItem2000(Node.Data) do begin
        IgnoreRepaintFloating:= True;
        ButtonArray1.AsButtonArray.ItemIndex:= DefaultIndex;
        ButtonArray2.AsButtonArray.ItemIndex:= ImageIndex;
        IgnoreRepaintFloating:= False;

        B:= (DefaultIndex = -1) and (ImageIndex = -1);
        None1.AsButton.Down:= B;
        None2.AsButton.Down:= B;
      end;

  except
  end;
end;

procedure T_AM2000_MenuDesignerDlg.Loaded;
var
  Reg: TRegistry;
  SI: TSearchRec;
  I, Res: Integer;
  mici: T_AM2000_MenuItemCacheItem;
  BlankColor: ColorRef;
  hbmp: HBitmap;
  B: Boolean;
  lpbi: TBrowseInfo;
  pid2: PItemIDList;
  z: array [0..MAX_PATH] of Char;
begin
  inherited;

  // load autobitmaps & autohints
  BlankColor:= ColorToRGB(clFuchsia);
  with ButtonArray1.AsButtonArray do
    for I:= 0 to MenuItemCache.Count -1 do begin
      mici:= T_AM2000_MenuItemCacheItem(TStringList(MenuItemCache).Objects[I]);
      if (mici.Bitmap <> 0)
      and (mici.IsDefault)
      then begin
        hbmp:= CopyImage(mici.Bitmap, Image_Bitmap, 0, 0, lr_CopyReturnOrg);
        ImageList_AddMasked(DefaultBitmaps.Handle, hbmp, BlankColor);
        DeleteObject(hbmp);
        if mici.Hint <> ''
        then Hints.Add(mici.Hint)
        else Hints.Add(TStringList(MenuItemCache)[I]);
      end;
    end;

  // draw image list
  DrawImageList(ButtonArray1, DefaultBitmaps);


{$IFNDEF VER90}
  // set flat property
  ButtonNewMenuItem.Flat:= True;
  ButtonNewSubMenu.Flat:= True;
  ButtonDelete.Flat:= True;
  ButtonLevelUp.Flat:= True;
  ButtonMoveUp.Flat:= True;
  ButtonMoveDown.Flat:= True;
  ButtonLevelDown.Flat:= True;
  ButtonStayOnTop.Flat:= True;
{$ENDIF}


  // load menu templates
  Reg:= TRegistry.Create;
  B:= Reg.OpenKey(MENU_REGISTRY_KEY, False);
  if not B then begin
    FillChar(lpbi, SizeOf(lpbi), 0);
    lpbi.hwndOwner:= Application.MainForm.Handle;
    lpbi.lpszTitle:= SBrowseForMenuTemplateFolder;
    pid2:= SHBrowseForFolder(lpbi);
    if pid2 = nil then Exit;

    Reg.OpenKey(MENU_REGISTRY_KEY, True);
    SHGetPathFromIDList(pid2, z);
    TempFolder:= StrPas(z);
    Reg.WriteString('MenuTemplatesFolder', TempFolder);
    CoTaskMemFree(pid2);
  end
  else
    TempFolder:= Reg.ReadString('MenuTemplatesFolder');

  // scan template files
  if (Length(TempFolder) > 0)
  and (TempFolder[Length(TempFolder)] <> '\')
  then AppendStr(TempFolder, '\');

  Res:= FindFirst(TempFolder + '*.mnu', faAnyFile, SI);
  while Res = 0 do begin
    InsStdMnu.Add(NewItem(ChangeFileExt(SI.Name, ''), 0,
      False, True, InsertTemplateMenuClick, 0, ''));

    Res:= FindNext(SI);
  end;
  FindClose(SI);

  Reg.Free;
end;

procedure T_AM2000_MenuDesignerDlg.FormShow(Sender: TObject);
begin
  // create and initialize dialogs
{$IFNDEF VER90}
  OpenPictureDialog1:= TOpenPictureDialog.Create(Self);
  SavePictureDialog1:= TSavePictureDialog.Create(Self);
{$ELSE}
  OpenPictureDialog1:= TOpenDialog.Create(Self);
  SavePictureDialog1:= TSaveDialog.Create(Self);
{$ENDIF}

  with OpenPictureDialog1 do begin
    DefaultExt:= 'bmp';
    Filter:= SBitmapDialogFilter;
    Options:= [ofHideReadOnly, ofPathMustExist, ofFileMustExist];
    Title:= SOpenBitmapDialogTitle;
  end;

  with SavePictureDialog1 do begin
    DefaultExt:= 'bmp';
    FileName:= 'Untitled-1.bmp';
    Filter:= SBitmapDialogFilter;
    Options:= [ofOverwritePrompt, ofHideReadOnly, ofPathMustExist];
    Title:= SSaveBitmapDialogTitle;
  end;
end;

procedure T_AM2000_MenuDesignerDlg.FormHide(Sender: TObject);
begin
  Designer:= nil;
end;

procedure T_AM2000_MenuDesignerDlg.FormDestroy(Sender: TObject);
begin
  MenuDesignerDlg:= nil;
end;

procedure T_AM2000_MenuDesignerDlg.CollapseAll1Click(Sender: TObject);
begin
  MenuTree.FullCollapse;
end;

procedure T_AM2000_MenuDesignerDlg.ExpandAll1Click(Sender: TObject);
begin
  MenuTree.FullExpand;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonNewMenuItemClick(Sender: TObject);
var
  N, N1: TTreeNode;
  M, M1: TMenuItem2000;
begin
  N:= MenuTree.Selected;
  if (not Assigned(N))
  or (not Assigned(N.Parent)) then Exit;

  M:= TMenuItem2000(N.Data);
  M1:= TMenuItem2000.Create(Menu.Owner);
  M1.Caption:= SDefaultMenuItemCaption;

  if Designer <> nil
  then M1.Name:= Designer.UniqueName('MenuItem');

  if (N.GetNextSibling <> nil) then begin
    N1:= MenuTree.Items.Insert(N.GetNextSibling, SDefaultMenuItemCaption);
    M.Parent.Insert(M.MenuIndex +1, M1);
  end
  else begin
    N1:= MenuTree.Items.Add(N, SDefaultMenuItemCaption);
    M.Parent.Add(M1);
  end;

  if Designer <> nil then begin
    Designer.Modified;
    Designer.SelectComponent(M1);
  end;

  N1.Selected:= True;
  N1.Data:= M1;
  N1.EditText;
  CheckConstraints;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonNewSubMenuClick(Sender: TObject);
var
  N: TTreeNode;
  M: TMenuItem2000;
begin
  if not Assigned(MenuTree.Selected) then Exit;
  N:= MenuTree.Items.AddChild(MenuTree.Selected, SDefaultMenuItemCaption);

  M:= TMenuItem2000.Create(Menu.Owner);
  M.Caption:= SDefaultMenuItemCaption;
  
  if Designer <> nil
  then M.Name:= Designer.UniqueName('MenuItem');

  TMenuItem2000(MenuTree.Selected.Data).Add(M);
  MenuTree.Selected.Expand(False);

  if Designer <> nil then begin
    Designer.SelectComponent(M);
    Designer.Modified;
  end;

  N.Selected:= True;
  N.Data:= M;
  N.EditText;
  CheckConstraints;
end;

procedure T_AM2000_MenuDesignerDlg.MenuTreeEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
var
  M: TMenuItem;
  S1, S2: String;
begin
//  try
    if Node.Parent <> nil then begin
      // generate menu item capion
      M:= TMenuItem(Node.Data);
      M.Caption:= S;
      S1:= S;

      while (M.Parent <> nil) do begin
        M:= M.Parent;
        S1:= M.Caption + S1;
      end;

      if Designer <> nil then begin
        // set menu item name
        S1:= GetValidName(S1);
        if S1 = '' then S1:= 'N';
        
        S2:= Designer.UniqueName(S1);
        if (S1[1] = 'T') and (S2[1] <> 'T')
        then S2:= 'T' + S2;

        TMenuItem(Node.Data).Name:= S2;
        Designer.Modified;
        Designer.SelectComponent(TMenuItem(Node.Data));
      end;
    end;

//  except
//    S:= Node.Text;

//  end;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonDeleteClick(Sender: TObject);
begin
  if not Assigned(MenuTree.Selected) then Exit;
  TMenuItem2000(MenuTree.Selected.Data).Free;
  MenuTree.Selected.Free;
  CheckConstraints;

{  if (Menu.Items.Count = 0)
  and (Menu.Owner is TForm)
  and (TForm(Menu.Owner).Menu = Menu)
  then TForm(Menu.Owner).Menu:= nil;}
end;

procedure T_AM2000_MenuDesignerDlg.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  S: String;
begin
  if MenuTree.IsEditing
  then
    case Key of
      vk_Return:
        ButtonNewMenuItemClick(nil);
      vk_Escape:
        if MenuTree.Selected.Text = SDefaultMenuItemCaption
        then ButtonDeleteClick(nil);
    end
  else
    case Key of
      vk_Insert:
        ButtonNewMenuItemClick(nil);
      vk_Delete:
        ButtonDeleteClick(nil);
      vk_F2:
        if ssCtrl in Shift
        then begin
          S:= TMenuItem2000(MenuTree.Selected.Data).Caption;
          MenuTreeEdited(nil, MenuTree.Selected, S);
        end
        else MenuTree.Selected.EditText;
      vk_Up:
        if ssCtrl in Shift
        then ButtonMoveUpClick(nil);
      vk_Down:
        if ssCtrl in Shift
        then ButtonMoveDownClick(nil);
      vk_Right:
        if ssCtrl in Shift
        then ButtonLevelDownClick(nil);
      vk_Left:
        if ssCtrl in Shift
        then ButtonLevelUpClick(nil);
    end;
  CheckConstraints;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonMoveDownClick(Sender: TObject);
var
  N, N1: TTreeNode;
  M, M1: TMenuItem2000;
begin
  N:= MenuTree.Selected;
  if (not Assigned(N))
  or (not Assigned(N.Parent)) then Exit;

  N1:= N.GetNextSibling;

  MenuTree.Items.BeginUpdate;

  if Assigned(N1)
  then begin
    N1:= N1.GetNextSibling;

    if Assigned(N1)
    then N.MoveTo(N1, naInsert)
    else N.MoveTo(N.Parent, naAddChild);

    with TMenuItem2000(N.Data) do MenuIndex:= MenuIndex +1;

  end
  else begin
    N1:= N.Parent.GetNextSibling;
    if Assigned(N1) then begin
      N.MoveTo(N1, naAddChildFirst);

      M:= TMenuItem2000(N.Data);
      M1:= M.Parent.Parent.Items[M.Parent.MenuIndex +1];
      M.Parent.Remove(M);
      M1.Insert(0, M);
    end
  end;

  MenuTree.Items.EndUpdate;
  CheckConstraints;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonMoveUpClick(Sender: TObject);
var
  N, N1: TTreeNode;
  M, M1: TMenuItem2000;
begin
  N:= MenuTree.Selected;
  if (not Assigned(N))
  or (not Assigned(N.Parent)) then Exit;

  N1:= N.GetPrevSibling;
  if N1 = N.Parent then Exit;

  MenuTree.Items.BeginUpdate;

  if Assigned(N1) then begin
    N.MoveTo(N1, naInsert);
    with TMenuItem2000(N.Data) do MenuIndex:= MenuIndex -1;
  end
  else begin
    N1:= N.Parent.GetPrevSibling;
    if Assigned(N1) then begin
      N.MoveTo(N1, naAddChild);

      M:= TMenuItem2000(N.Data);
      M1:= M.Parent.Parent.Items[M.Parent.MenuIndex -1];
      M.Parent.Remove(M);
      M1.Add(M);

    end;
  end;

  MenuTree.Items.EndUpdate;
  CheckConstraints;

end;

procedure T_AM2000_MenuDesignerDlg.ButtonLevelUpClick(Sender: TObject);
var
  N, N1: TTreeNode;
  M, M1: TMenuItem2000;
  I: Integer;
begin
  N:= MenuTree.Selected;
  if (not Assigned(N))
  or (not Assigned(N.Parent))
  or (not Assigned(N.Parent.Parent)) then Exit;

  N1:= N.Parent.GetNextSibling;
  MenuTree.Items.BeginUpdate;

  M:= TMenuItem2000(N.Data);
  M1:= M.Parent.Parent;
  I:= M.Parent.MenuIndex;
  M.Parent.Remove(M);

  if Assigned(N1) then begin
    N.MoveTo(N1, naInsert);
    M1.Insert(I +1, M);

  end
  else begin
    N.MoveTo(N.Parent.Parent, naAddChild);
    M1.Add(M);

  end;

  MenuTree.Items.EndUpdate;
  CheckConstraints;

end;

procedure T_AM2000_MenuDesignerDlg.ButtonLevelDownClick(Sender: TObject);
var
  N, N1: TTreeNode;
  M, M1, M2: TMenuItem2000;
  I: Integer;
begin
  N:= MenuTree.Selected;
  if (not Assigned(N))
  or (not Assigned(N.Parent)) then Exit;

  // treeview...
  N1:= N.GetPrevSibling;
  MenuTree.Items.BeginUpdate;

  // menu...
  M:= TMenuItem2000(N.Data);
  M1:= M.Parent;
  I:= M.MenuIndex;
  M.Parent.Remove(M);

  if Assigned(N1)
  then begin
    N.MoveTo(N1, naAddChild);
    M1[I -1].Add(M);
  end
  else begin
    N1:= MenuTree.Items.AddChildFirst(N.Parent, SDefaultMenuItemCaption);
    N.MoveTo(N1, naAddChild);

    M2:= TMenuItem2000.Create(Menu.Owner);
    M2.Caption:= SDefaultMenuItemCaption;

    if Designer <> nil
    then M2.Name:= Designer.UniqueName('MenuItem');

    M2.Add(M);
    M1.Insert(0, M2);

    N1.Data:= M2;

  end;

  MenuTree.Items.EndUpdate;
  CheckConstraints;

end;

procedure T_AM2000_MenuDesignerDlg.UpdateCaption(MenuItem: TMenuItem);
var
  I: Integer;
begin
  if Assigned(MenuTree.Selected)
  and (MenuTree.Selected.Data = MenuItem)
  then begin
    if MenuTree.Selected.Text <> MenuItem.Caption
    then MenuTree.Selected.Text:= MenuItem.Caption;

    Exit;
  end;

  for I:= 0 to MenuTree.Items.Count -1 do
    if MenuTree.Items[I].Data = MenuItem then begin
      MenuTree.Items[I].Text:= MenuItem.Caption;
      Exit;
    end;
end;


procedure T_AM2000_MenuDesignerDlg.LoadFromFile2Click(Sender: TObject);
begin
  ShowDoesntSupport(SFileOperations);
end;

procedure T_AM2000_MenuDesignerDlg.InsertTemplateMenuClick(Sender: TObject);
  // loads a template menu
var
  S: String;
  M, M1: TMenuItem2000;
begin
  S:= TempFolder + TMenuItem2000(Sender).Caption + '.mnu';

  if (MenuTree.Selected = nil)
  or (not FileExists(S))
  then Exit;

  M:= TMenuItem2000(MenuTree.Selected.Data);
  M1:= TMenuItem2000.Create(Menu.Owner);
  ReadComponentResFile(S, M1);

  M.Insert(0, M1);
  Menu:= Menu;

  if Designer <> nil
  then Designer.Modified;
end;

procedure T_AM2000_MenuDesignerDlg.SaveToFile2Click(Sender: TObject);
  // saves menu to a file
begin
  ShowDoesntSupport(SFileOperations);
end;

procedure T_AM2000_MenuDesignerDlg.SaveAsTemplate1Click(Sender: TObject);
  // saves menu as an template
begin
  ShowDoesntSupport(SFileOperations);
end;

procedure T_AM2000_MenuDesignerDlg.MenuTreeMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  // pops up a right-click menu
var
  N: TTreeNode;
begin
  if (Button <> mbRight)
  then Exit;

  N:= MenuTree.GetNodeAt(X, Y);
  if Assigned(N) then begin
    MenuTree.Selected:= N;
    with MenuTree.ClientToScreen(Point(X, Y)) do
      PopupMenu20001.Popup(X, Y);
  end
  else begin
  end;
end;

procedure T_AM2000_MenuDesignerDlg.LoadBitmapFromFileClick(Sender: TObject);
begin
  if not OpenPictureDialog1.Execute
  then Exit;

  with TMenuItem2000(MenuTree.Selected.Data) do
    Bitmap.LoadFromFile(OpenPictureDialog1.Filename);

  if Designer <> nil
  then Designer.Modified;
end;

procedure T_AM2000_MenuDesignerDlg.SaveBitmapToFileClick(Sender: TObject);
begin
  if not SavePictureDialog1.Execute
  then Exit;

  with TMenuItem2000(MenuTree.Selected.Data) do
    Bitmap.SaveToFile(SavePictureDialog1.Filename);
end;

procedure T_AM2000_MenuDesignerDlg.PopupMenu20001Popup(Sender: TObject);
  // restore bitmap indexes
var
  il: TImageList;
begin
  if Assigned(MenuTree.Selected) then
    with TMenuItem2000(MenuTree.Selected.Data) do begin
      IgnoreRepaintFloating:= True;
      ButtonArray1.AsButtonArray.ItemIndex:= DefaultIndex;
      IgnoreRepaintFloating:= False;
      None1.AsButton.Down:= (not IsBitmapAssigned);
    end;

  // load bitmaps from menu's imagelist
  il:= nil;
  if Menu is TMainMenu2000 then il:= TImageList(TCustomMainMenu2000(Menu).Images);
  if Menu is TPopupMenu2000 then il:= TImageList(TCustomPopupMenu2000(Menu).Images);
  ImageListBitmaps1.Enabled:= (il <> nil) and (il.Count > 0);
end;

procedure T_AM2000_MenuDesignerDlg.None1Click(Sender: TObject);
  // remove the bitmap
begin
  if None1.AsButton.Down then Exit;

  None1.AsButton.Down:= True;
  None2.AsButton.Down:= True;
  with MenuTree.Selected, TMenuItem2000(Data) do begin
    DefaultIndex:= -1;
    ImageIndex:= -1;
    if IsBitmapAssigned then begin
      Bitmap.Handle:= 0;
    end;

    if Designer <> nil then begin
      Designer.Modified;
      Designer.SelectComponent(TMenuItem2000(Data));
    end;
  end;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonArray1Click(Sender: TObject);
  // assigns default bitmaps
begin
  with ButtonArray1.AsButtonArray, MenuTree.Selected, TMenuItem2000(Data)
  do begin
    DefaultIndex:= ItemIndex;
    ImageIndex:= -1;
    None2.AsButton.Down:= False;
    ButtonArray2.AsButtonArray.ItemIndex:= -1;

    if Designer <> nil then begin
      Designer.SelectComponent(TMenuItem2000(Data));
      Designer.Modified;
    end;
  end;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonArray2Click(Sender: TObject);
  // assigns imagelist's bitmap
begin
  with ButtonArray2.AsButtonArray, MenuTree.Selected, TMenuItem2000(Data)
  do begin
    DefaultIndex:= -1;
    ImageIndex:= ItemIndex;
    None1.AsButton.Down:= False;
    ButtonArray1.AsButtonArray.ItemIndex:= -1;

    if Designer <> nil then begin
      Designer.SelectComponent(TMenuItem2000(Data));
      Designer.Modified;
    end;
  end;
end;

procedure T_AM2000_MenuDesignerDlg.SetMenu(const Value: TMenu);

  procedure AddMenu(M: TMenuItem; T: TTreeNodes; N: TTreeNode);
  var
    I: Integer;
    N1: TTreeNode;
  begin
    for I:= 0 to M.Count -1 do begin
      N1:= T.AddChild(N, M[I].Caption);
      N1.Data:= M[I];
      if M[I].Count > 0 then AddMenu(M[I], T, N1);
    end;
  end;

var
  N: TTreeNode;
begin
  // init menu tree items
  MenuTree.Items.Clear;

  FMenu:= Value;
  if FMenu = nil then Exit;

  try
    MenuTree.Items.BeginUpdate;

    N:= MenuTree.Items.Add(nil, 'AnimatedMenus/2000');
    N.Data:= Menu.Items;

    AddMenu(Menu.Items, MenuTree.Items, N);

    MenuTree.Items[0].Expand(False);
    MenuTree.Items.EndUpdate;
  except
  end;
end;

procedure T_AM2000_MenuDesignerDlg.AM2000MenuDesigner6Click(
  Sender: TObject);
begin
  MenuTree.Items.BeginUpdate;
  MenuTree.FullExpand;
  MenuTree.Items.EndUpdate;
end;

procedure T_AM2000_MenuDesignerDlg.AM2000MenuDesigner7Click(
  Sender: TObject);
begin
  MenuTree.Items.BeginUpdate;
  MenuTree.FullCollapse;
  MenuTree.Items.EndUpdate;
end;

procedure T_AM2000_MenuDesignerDlg.MenuTreeEditing(Sender: TObject;
  Node: TTreeNode; var AllowEdit: Boolean);
begin
  AllowEdit:= Node.Parent <> nil;
end;

procedure T_AM2000_MenuDesignerDlg.ButtonStayOnTopClick(Sender: TObject);
begin
  if ButtonStayOnTop.Down
  then FormStyle:= fsStayOnTop
  else FormStyle:= fsNormal;
end;

procedure T_AM2000_MenuDesignerDlg.ImageListMenuPopup(Sender: TObject);
var
  il: TImageList;
begin
  // load bitmaps from menu's imagelist
  il:= nil;
  if Menu is TMainMenu2000 then il:= TImageList(TCustomMainMenu2000(Menu).Images);
  if Menu is TPopupMenu2000 then il:= TImageList(TCustomPopupMenu2000(Menu).Images);

  if (il = nil) or (il.Count = 0)
  then begin
    ImageListBitmaps1.Enabled:= False;
    ButtonArray2.AsButtonArray.Bitmap.Width:= 0;
  end
  else begin
    DrawImageList(ButtonArray2, il);
    ImageListBitmaps1.Enabled:= True;
  end;
end;

end.
