
{*******************************************************}
{                                                       }
{       AnimatedMenus/2000                              }
{       TMenuItem2000                                   }
{                                                       }
{       Copyright (c) 1998-1999 by Andrew Cher          }
{                                                       }
{*******************************************************}

unit am2000menuitem;

interface

uses
  Windows, SysUtils, Classes, Controls, Graphics, Menus,
  am2000options, am2000button, am2000buttonarray;

type

  // options for menu item
  T_AM2000_ControlType = (ctlNone, ctlButton, ctlButtonArray {, ctlEditbox, ctlCombobox});

  // descendant of TPopupMenu
  T_AM2000_PopupMenu = class(TPopupMenu)
  end;


  // TMenuItem98
  TMenuItem2000 = class(TMenuItem)
  private
    // improvements
    FPopupMenu      : T_AM2000_PopupMenu;
    FControl        : T_AM2000_ControlType;
    FControlOptions : T_AM2000_ControlOptions;
    FHidden         : Boolean;
//    FNewBreak       : TMenuBreak;
    FAttachMenu     : T_AM2000_PopupMenu;
    FOptions        : T_AM2000_MenuOptions;
    FShortCut       : String;
    FDefaultBitmapIndex : Integer;
{$IFNDEF VER120}
    FBitmap         : TBitmap;
    FImageIndex     : Integer;
{$ENDIF}

    function GetItem(Index: Integer): TMenuItem2000;
    function GetCaption: String;
    function GetEnabled: Boolean;
    function GetVisible: Boolean;
    function GetParent: TMenuItem2000;
    function GetBitmap:   TBitmap;

    procedure SetCaption(const Value: String);
    procedure SetEnabled(Value: Boolean);
    procedure SetVisible(Value: Boolean);
    procedure SetControl(Value: T_AM2000_ControlType);
    procedure SetBitmap(Value: TBitmap);

    function IsBitmapStored:   Boolean;

    function IsOptionsStored:  Boolean;
    function GetOptions: T_AM2000_MenuOptions;
    procedure SetOptions(Value: T_AM2000_MenuOptions);
    function IsHintStored:     Boolean;
    function IsShortCutStored: Boolean;
    function GetShortCut: String;
    function GetHint:     String;
    procedure SetShortCut(Value: String);
    procedure SetHint(Value: String);

  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
    property Parent        : TMenuItem2000  read GetParent;
    property Items[Index: Integer]: TMenuItem2000 read GetItem; default;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Insert(Index: Integer; Item: TMenuItem2000);
    function IndexOf(Item: TMenuItem2000): Integer;
    procedure Add(Item: TMenuItem2000);
    procedure Remove(Item: TMenuItem2000);
    procedure Delete(Index: Integer);
    procedure TurnSiblingsOff;

    function IsTopLevelItem   : Boolean;
    function IsBitmapAssigned : Boolean;
    function IsOptionsAssigned: Boolean;
    function AsButtonArray    : T_AM2000_ButtonArrayOptions;
    function AsButton         : T_AM2000_ButtonOptions;

    function GetWidth(Canvas: TCanvas): Integer; 
    function GetHeight(ItemHeight: Integer): Integer;

    procedure Assign(Source: TPersistent); override;
    procedure Popup(X, Y: Integer); virtual;

  published
    property Bitmap        : TBitmap
      read GetBitmap write SetBitmap stored IsBitmapStored;
//    property Break         : TMenuBreak
//      read FNewBreak stored False;
    property Caption       : String
      read GetCaption write SetCaption;
    property Control       : T_AM2000_ControlType
      read FControl write SetControl default ctlNone;
    property ControlOptions: T_AM2000_ControlOptions
      read FControlOptions write FControlOptions stored True;
    property Enabled       : Boolean
      read GetEnabled write SetEnabled default True;
    property Hidden        : Boolean
      read FHidden write FHidden default False;
    property PopupMenu     : T_AM2000_PopupMenu
      read FPopupMenu write FPopupMenu;
    property Visible       : Boolean
      read GetVisible write SetVisible default True;
    property DefaultIndex  : Integer
      read FDefaultBitmapIndex write FDefaultBitmapIndex default -1;
    property AttachMenu    : T_AM2000_PopupMenu
      read FAttachMenu write FAttachMenu;
    property Options       : T_AM2000_MenuOptions
      read GetOptions write SetOptions stored IsOptionsStored;
    property Hint          : String
      read GetHint write SetHint stored IsHintStored;
    property ShortCut      : String
      read GetShortCut write SetShortCut stored IsShortCutStored;
{$IFNDEF VER120}
    property ImageIndex    : Integer
      read FImageIndex write FImageIndex default -1;
{$ENDIF}

  end;


  // TEditableMenuItem98
  // This class was designed because of error in Delphi:
  // you cannot register another menu designer when you already have default designer
  // This class are incompatible with TMenuItem nor TMenuItem2000 but acts like layer
  // between your application and the TMenuItem2000 components stored in FItems property
  // It has only minimal set of properties and methods so please verify your applications
  // for bugs
  // Also you can use Items98 property of TMainMenu98 and TPopupMenu98 components
  // to direct access to the real TMenuItem2000 components.
  TEditableMenuItem2000 = class(TComponent)
  private
    function GetHandle: HMenu;
    function GetCount: Integer;
    function GetItem(Index: Integer): TMenuItem2000;

  public                                  
    property Handle        : HMenu        read GetHandle;
    property Count         : Integer      read GetCount;
    property Items[Index: Integer]: TMenuItem2000 read GetItem; default;

    procedure Insert(Index: Integer; Item: TMenuItem2000);
    procedure Delete(Index: Integer);
    function IndexOf(Item: TMenuItem2000): Integer;
    procedure Add(Item: TMenuItem2000);
    procedure Remove(Item: TMenuItem2000);

  end;


// resource name conversion function
//var
//  ResNameConvertFunc : function(Caption: String): String;


// compatibility
function NewMenu(Owner: TComponent; const AName: string; Items: array of TMenuItem): TMainMenu;
function NewPopupMenu(Owner: TComponent; const AName: string;
  Alignment: TPopupAlignment; AutoPopup: Boolean; Items: array of TMenuItem): TPopupMenu;
function NewSubMenu(const ACaption: string; hCtx: Word; const AName: string;
  Items: array of TMenuItem2000): TMenuItem2000;
function NewItem(const ACaption: string; AShortCut: TShortCut;
  AChecked, AEnabled: Boolean; AOnClick: TNotifyEvent; hCtx: Word;
  const AName: string): TMenuItem2000;
function NewLine: TMenuItem2000;

function NewItem2000(const ACaption, AShortCut: string;
  AChecked, AEnabled: Boolean; AOnClick: TNotifyEvent; hCtx: Word;
  const AName: string): TMenuItem2000;
function NewLine2000: TMenuItem2000;

function GetMainShortCut(const S: String): String;
procedure SetState(var its: T_AM2000_ItemState; Value: T_AM2000_its; Condition: Boolean);


// drawing routines - both for TMenuItem and TMenuItem2000

// draws menu item when TMenuItem or TMenuItem2000 component doesn't present
// (like Window menu's menu items in Delphi 2 & 3)
procedure DrawMenuItemWin32(DrawRect: P_AM2000_DrawMenuItemRect);

// draws TMenuItem2000 with ControlStyle = ctlNone and TMenuItem
procedure DrawMenuItem(DrawRect: P_AM2000_DrawMenuItemRect);


{ Drawing routines }

procedure DrawCaption(Canvas: TCanvas;
            Caption: String;
            Alignment: T_AM2000_Alignment;
            Rect: TRect);

procedure DrawSubmenuTriangle(Canvas: TCanvas;
            Draw: Boolean;
            Alignment: T_AM2000_Alignment;
            Rect: TRect);

procedure DrawSeparator(Canvas: TCanvas;
            State: T_AM2000_ItemState;
            Options: T_AM2000_BaseOptions;
            Rect: TRect);

procedure DrawBackground(Canvas: TCanvas;
            State: T_AM2000_ItemState;
            Options: T_AM2000_BaseOptions;
            Rect: TRect);

procedure DrawBitmap(Canvas: TCanvas;
            Bitmap: HBitmap;
            Bitmap2: HBitmap;
            State: T_AM2000_ItemState;
            Options: T_AM2000_BaseOptions;
            Rect: TRect;
            Images: TImageList);

procedure DrawPatternBackground(Canvas: TCanvas;
            Rect: TRect);

function CreatePattern(Color1, Color2: TColor): TBitmap;


implementation

uses
  CommCtrl, DsgnIntf,
  am2000utils, am2000menubar, am2000designer, am2000const, am2000cache;


{ Routines }

function NewMenu(Owner: TComponent; const AName: string; Items: array of TMenuItem): TMainMenu;
begin
  raise EMenuError.Create('You cannot use NewMenu function with AnimatedMenus/98');
end;

function NewPopupMenu(Owner: TComponent; const AName: string;
  Alignment: TPopupAlignment; AutoPopup: Boolean; Items: array of TMenuItem): TPopupMenu;
begin
  raise EMenuError.Create('You cannot use NewPopupMenu function with AnimatedMenus/98');
end;

function NewSubMenu(const ACaption: string; hCtx: Word; const AName: string;
  Items: array of TMenuItem2000): TMenuItem2000;
var
  I: Integer;
begin
  Result := TMenuItem2000.Create(nil);
  for I := Low(Items) to High(Items) do
    Result.Add(Items[I]);
  Result.Caption := ACaption;
  Result.HelpContext := hCtx;
  Result.Name := AName;
end;

function NewItem(const ACaption: string; AShortCut: TShortCut;
  AChecked, AEnabled: Boolean; AOnClick: TNotifyEvent; hCtx: Word;
  const AName: string): TMenuItem2000;
begin
  Result:= TMenuItem2000.Create(nil);
  with Result do begin
    Caption:= ACaption;
    ShortCut:= ShortCutToText(AShortCut);
    OnClick:= AOnClick;
    HelpContext:= hCtx;
    Checked:= AChecked;
    Enabled:= AEnabled;
    Name:= AName;
  end;
end;

function NewLine: TMenuItem2000;
begin
  Result:= TMenuItem2000.Create(nil);
  Result.Caption:= '-';
end;

function NewItem2000(const ACaption, AShortCut: string;
  AChecked, AEnabled: Boolean; AOnClick: TNotifyEvent; hCtx: Word;
  const AName: string): TMenuItem2000;
begin
  Result:= TMenuItem2000.Create(nil);
  with Result do begin
    Caption:= ACaption;
    ShortCut:= AShortCut;
    OnClick:= AOnClick;
    HelpContext:= hCtx;
    Checked:= AChecked;
    Enabled:= AEnabled;
    Name:= AName;
  end;
end;

function NewLine2000: TMenuItem2000;
begin
  Result:= NewLine;
end;

function GetMainShortCut(const S: String): String;
var
  I: Integer;
begin
  I:= Pos(';', S);
  if I < 1 then I:= Length(S) +1;
  Result:= Trim(Copy(S, 1, I -1));
end;


{ SetState }

procedure SetState(var its: T_AM2000_ItemState; Value: T_AM2000_its; Condition: Boolean);
begin
  if Condition
  then Include(its, Value)
  else Exclude(its, Value);
end;



{ Drawing routines }

{ Drawing routines }

procedure DrawCaption(
  Canvas: TCanvas;
  Caption: String;
  Alignment: T_AM2000_Alignment;
  Rect: TRect);
var
  Flags: Integer;
begin
  Flags:= dt_DrawTextFlags;

  // caption alignment
  if (Alignment = taCenter)
  then Flags:= Flags or dt_Center;
  if (Alignment = taRightJustify) or (Alignment = taHebrew)
  then Flags:= Flags or dt_Right;

  // draw caption
  DrawText(Canvas.Handle, PChar(Caption), -1, Rect, Flags);
end;

procedure DrawSubmenuTriangle(
  Canvas: TCanvas;
  Draw: Boolean;
  Alignment: T_AM2000_Alignment;
  Rect: TRect);
var
  DX: Integer;
begin
  if (not Draw)
  or (Rect.Left = Rect.Right)
  then Exit;

  Canvas.Pen.Color:= Canvas.Font.Color;
  Canvas.Brush.Color:= Canvas.Font.Color;
  Canvas.Brush.Style:= bsSolid;

  DX:= (Rect.Top + Rect.Bottom - 5) div 2;
  if Alignment <> taHebrew then
    Canvas.PolyGon([
      Point(Rect.Right -8, DX),
      Point(Rect.Right -8, DX +6),
      Point(Rect.Right -5, DX +3)])
  else
    Canvas.PolyGon([
      Point(Rect.Left  +8, DX),
      Point(Rect.Left  +8, DX +6),
      Point(Rect.Left  +5, DX +3)])
end;

procedure DrawSeparator(
  Canvas: TCanvas;
  State: T_AM2000_ItemState;
  Options: T_AM2000_BaseOptions;
  Rect: TRect);
var
  Y, DX: Integer;
begin
  Exclude(State, isSelected);

  // hide separator
  if (isHiddenPrev in State) and (isHiddenSucc in State)
  then Include(State, isHidden);

  DrawBackground(Canvas, State, Options, Rect);

  Y:= (Rect.Top + Rect.Bottom) div 2;

  if mfNoShortDividers in Options.Flags
  then DX:= 2
  else DX:= 14;

  if Options.Colors.Line <> clNone then begin
    Canvas.Pen.Color:= Options.Colors.Line;
    Canvas.PolyLine([Point(Rect.Left + DX, Y), Point(Rect.Right - DX, Y)]);
  end;

  if Options.Colors.LineShadow <> clNone then begin
    Canvas.Pen.Color:= Options.Colors.LineShadow;
    Canvas.PolyLine([Point(Rect.Left + DX, Y +1), Point(Rect.Right - DX, Y +1)]);
  end;
end;

procedure DrawPatternBackground(
  Canvas: TCanvas;
  Rect: TRect);
begin
  if not Assigned(Pattern)
  then Pattern:= CreatePattern(clBtnHighlight, clBtnFace);
  Canvas.Brush.Bitmap:= Pattern;
  Canvas.FillRect(Rect);
end;

procedure DrawSolidBackground(
  Canvas: TCanvas;
  Rect: TRect;
  Color: TColor);
begin
  // set brush style
  if Canvas.Brush.Style <> bsSolid
  then Canvas.Brush.Style:= bsSolid;

  // erase bitmap
  if Canvas.Brush.Bitmap <> nil
  then Canvas.Brush.Bitmap:= nil;

  // set canvas color
  if Canvas.Brush.Color <> Color
  then Canvas.Brush.Color:= Color;

  // fill
  Canvas.FillRect(Rect);
end;

procedure DrawBackground(
  Canvas: TCanvas;
  State: T_AM2000_ItemState;
  Options: T_AM2000_BaseOptions;
  Rect: TRect);
begin
  // no background on graphic backround
  if isGraphBack in State
  then Exit;

  if isSelected in State then begin
    // for hidden menu items
    if isHidden in State then InflateRect(Rect, 1, -1);
    DrawSolidBackground(Canvas, Rect, Options.Colors.Highlight);
//    if Canvas.Font.Color <>  Options.Colors.HighlightText
//    then Canvas.Font.Color:= Options.Colors.HighlightText;
  end
  else
    if isHidden in State then begin
      // draw pattern background
      InflateRect(Rect, 2, 0);
      DrawPatternBackground(Canvas, Rect);
      InflateRect(Rect, 0, -1);

      // draw borders
      if not (isHiddenPrev in State) then begin
        Canvas.Pen.Color:= Options.Colors.Line;
        Canvas.Polygon([Point(Rect.Left, Rect.Top -1), Point(Rect.Right -1, Rect.Top -1)]);
      end;
      if not (isHiddenSucc in State) then begin
        Canvas.Pen.Color:= Options.Colors.LineShadow;
        Canvas.Polygon([Point(Rect.Left, Rect.Bottom), Point(Rect.Right -1, Rect.Bottom)]);
      end;
    end
    else
      DrawSolidBackground(Canvas, Rect, Options.Colors.Menu);
end;

procedure DrawBitmap(
  Canvas: TCanvas;
  Bitmap: HBitmap;
  Bitmap2: HBitmap;
  State: T_AM2000_ItemState;
  Options: T_AM2000_BaseOptions;
  Rect: TRect;
  Images: TImageList);
var
  bmprect, markrect: TRect;

  procedure DrawBmp(Canvas: TCanvas; X, Y: Integer; Bitmap: HBitmap);
    // draws bitmap
  begin
    // is it a bitmap?
    if GetObject(Bitmap, 0, nil) <> 0
    // yes
    then TransBlt(Canvas, bmprect.Left +1, bmprect.Top +1, Bitmap)
    // no -- it's ImageIndex in the Images
    else ImageList_Draw(Images.Handle, Bitmap -1, Canvas.Handle, X, Y, ild_Transparent);
  end;

begin
  if (Rect.Left = Rect.Right)
  then Exit;

  // draw bitmap background
  if Canvas.Brush.Style <> bsSolid
  then Canvas.Brush.Style:= bsSolid;

  if Canvas.Brush.Color <> Options.Colors.Menu
  then Canvas.Brush.Color:= Options.Colors.Menu;

  Dec(Rect.Right);

  // calculate bmprect and markrect
  bmprect:= Rect;
  markrect:= Rect;
  if mfShowCheckMark in Options.Flags then begin
    markrect.Right:= (markrect.Right - markrect.Left) div 2 + markrect.Left;
    bmprect.Left:= markrect.Right +1;
  end;

  // draw solid background
  if (not (isHidden in State)) or (isSelected in State) then begin
    if (Bitmap <> 0) then begin
      Inc(bmprect.Right);
      Canvas.FillRect(bmprect);
      Dec(bmprect.Right);
    end;

    if isChecked in State then begin
      Inc(markrect.Right);
      Canvas.FillRect(markrect);
      Dec(markrect.Right);
    end;
  end;

  // draw background and rect
  if isSelected in State then begin
    if (Bitmap <> 0) and (not (mfNoBitmapRect in Options.Flags))
    then DrawEdge(Canvas.Handle, bmprect, bdr_RaisedInner, bf_Rect);
    if isChecked in State
    then DrawEdge(Canvas.Handle, markrect, bdr_SunkenOuter, bf_Rect);
  end
  else
    if isChecked in State then begin
      DrawPatternBackground(Canvas, markrect);
      DrawEdge(Canvas.Handle, markrect, bdr_SunkenOuter, bf_Rect);
    end;

  Inc(Rect.Right);
  if isChecked in State then Inc(Rect.Left);

  // draw bitmaps
  if not (isDisabled in State)
  then begin
    if Bitmap <> 0
    then DrawBmp(Canvas, bmprect.Left +1, bmprect.Top +1, Bitmap);

    if (isChecked in State)
    and ((mfShowCheckMark in Options.Flags) or (Bitmap = 0))
    then TransBlt(Canvas, markrect.Left +1, markrect.Top +1, Bitmap2);
  end
  else begin
    if Bitmap <> 0
    then NewDisabledBlt(Canvas, bmprect.Left +1, bmprect.Top +1, Options.Colors.DisabledText,
      Options.Colors.DisabledShadow, Bitmap);

    if (isChecked in State)
    and ((mfShowCheckMark in Options.Flags) or (Bitmap = 0))
    then
      NewDisabledBlt(Canvas, markrect.Left +1, markrect.Top +1, Options.Colors.DisabledText,
        Options.Colors.DisabledShadow, Bitmap2);
  end;
end;

function CreatePattern(Color1, Color2: TColor): TBitmap;
var
  X, Y: Integer;
begin
  Result:= TBitmap.Create;
  Result.Width:= 8;
  Result.Height:= 8;
  for Y:= 0 to 7 do
    for X:= 0 to 7 do
      if (Y mod 2) = (X mod 2)
      then Result.Canvas.Pixels[X, Y]:= Color1
      else Result.Canvas.Pixels[X, Y]:= Color2;
end;


procedure DrawTextItem(
  Canvas: TCanvas;
  Options: T_AM2000_BaseOptions;
  Caption: String;
  Shortcut: String;
  Bitmap: HBitmap;
  State: T_AM2000_ItemState;
  MouseState: T_AM2000_MouseState;
  mir: T_AM2000_MenuItemRect;
  Images: TImageList);
var
  OldFontStyle: TFontStyles;
  ScAlign: T_AM2000_Alignment;
  P: Integer;
  Bitmap2: HBitmap;
begin
  DrawBackground(Canvas, State, Options, mir.LineRect);

  if isHidden in State then begin
    Inc(mir.Top);
    Dec(mir.Height, 2);
  end;

  // draw bitmap
  if isRadio in State
  then Bitmap2:= bmpRadioItem
  else Bitmap2:= bmpCheckMark;

  DrawBitmap(Canvas, Bitmap, Bitmap2, State, Options, mir.BitmapRect, Images);

  // draw caption
  OldFontStyle:= Canvas.Font.Style;
  if (isDefault in State) and (not (fsBold in OldFontStyle)) then
    Canvas.Font.Style:= Canvas.Font.Style + [fsBold];

  // shortcut's alignment
  if Options.Alignment = taHebrew
  then ScAlign:= taLeftJustify
  else ScAlign:= taRightJustify;

  Canvas.Brush.Style:= bsClear;

  // little interesting thing!
  P:= Pos(#9, Caption);
  if (P <> 0)
  and (ShortCut = '')
  then begin
    Shortcut:= Copy(Caption, P +1, MaxInt);
    Caption:= Copy(Caption, 1, P -1);
  end;

  // draw the caption
  if not (isDisabled in State)
  then               // enabled
    if isSelected in State
    then Canvas.Font.Color:= Options.Colors.HighlightText
    else Canvas.Font.Color:= Options.Colors.MenuText
  else begin         // disabled
    // draw the shadow if not selected
    if not (isSelected in State) then begin
      Canvas.Font.Color:= Options.Colors.DisabledShadow;
      mir.IncreaseOffset;
      DrawCaption(Canvas, Caption, Options.Alignment, mir.ItemRect);
      DrawCaption(Canvas, Shortcut, ScAlign, mir.ShortcutRect);
      DrawSubmenuTriangle(Canvas, isSubmenu in State, Options.Alignment, mir.TriangleRect);
      mir.DecreaseOffset;
      Canvas.Brush.Style:= bsClear;
    end;

    Canvas.Font.Color:= Options.Colors.DisabledText;
  end;

  // caption, shortcut & triangle
  DrawCaption(Canvas, Caption, Options.Alignment, mir.ItemRect);
  DrawCaption(Canvas, Shortcut, ScAlign, mir.ShortcutRect);
  DrawSubmenuTriangle(Canvas, isSubmenu in State, Options.Alignment, mir.TriangleRect);

  // disabled menu items
  if Canvas.Font.Style <> OldFontStyle then
    Canvas.Font.Style:= OldFontStyle;

end;


{ Main drawing functions }

procedure DrawMenuItemWin32;
var
  Caption: String;
  Bitmap: HBitmap;
begin
  with DrawRect^ do begin
    mii.fMask:= $3F;
    mii.dwTypeData:= PChar(@Z);
    mii.cch:= SizeOf(Z) -1;
    if not GetMenuItemInfo(Handle, Index, True, mii) then Exit;

    // separator
    if mii.fType and mft_Separator <> 0 then begin
      DrawSeparator(Canvas, State, Options, mir.LineRect);
      Exit;
    end;

    // set caption
    if mii.fState and mfs_Checked <> 0
    then Bitmap:= mii.hbmpChecked
    else Bitmap:= mii.hbmpUnchecked;
    Caption:= StrPas(Z);

    // load bitmap
    case mii.wID of
      sc_Restore:  Bitmap:= MenuItemCache['SYSTEMRESTORE' ].Bitmap;
      sc_Move:     Bitmap:= MenuItemCache['SYSTEMMOVE'    ].Bitmap;
      sc_Size:     Bitmap:= MenuItemCache['SYSTEMSIZE'    ].Bitmap;
      sc_Minimize: Bitmap:= MenuItemCache['SYSTEMMINIMIZE'].Bitmap;
      sc_Maximize: Bitmap:= MenuItemCache['SYSTEMMAXIMIZE'].Bitmap;
      sc_Close:    Bitmap:= MenuItemCache['SYSTEMCLOSE'   ].Bitmap;
    end;

    // draw item
    SetState(State, isChecked, mii.fState and mfs_Checked <> 0);
    SetState(State, isDisabled, mii.fState and (mfs_Disabled + mfs_Grayed) <> 0);
    SetState(State, isDefault, mii.fState and mfs_Default <> 0);
    SetState(State, isRadio, mii.fType and mft_RadioCheck <> 0);
    SetState(State, isSubmenu, mii.hSubmenu <> 0);

    DrawTextItem(Canvas, Options, Caption, '', Bitmap, State, MouseState, mir, nil);
  end;
end;

procedure DrawMenuItem;
var
  Caption, Shortcut: String;
  Bitmap: HBitmap;
  Item2000: TMenuItem2000;
begin
  with DrawRect^ do begin
    Item2000:= TMenuItem2000(Item);

    // separator
    if Item.Caption = '-' then begin
      DrawSeparator(Canvas, State, Options, mir.LineRect);
      Exit;
    end;

    // get caption
    Caption:= Item.Caption;

    // get shortcut
    if Item is TMenuItem2000
    then ShortCut:= GetMainShortCut(Item2000.ShortCut)
    else Shortcut:= ShortcutToText(Item.ShortCut);

    // get user bitmap

    // advanced bitmap
    if (Item is TMenuItem2000)
    and (Item2000.IsBitmapAssigned)
    then Bitmap:= Item2000.Bitmap.Handle


    // ordinal bitmap's ImageIndex
    else
    if
    (Images <> nil) and
{$IFNDEF VER120}
    (Item is TMenuItem2000) and (Item2000.ImageIndex <> -1)
{$ELSE}
    (Item.ImageIndex <> -1)
{$ENDIF}
    then Bitmap:= Item2000.ImageIndex +1

    // no advanced bitmap
    else
{$IFDEF VER120}
      Bitmap:= Item.Bitmap.Handle;
{$ELSE}
      Bitmap:= 0;
{$ENDIF}

    // draw item
    SetState(State, isChecked, Item.Checked);
    SetState(State, isDisabled, not Item.Enabled);
    SetState(State, isDefault, Item.Default);
    SetState(State, isRadio, Item.RadioItem);
    SetState(State, isSubmenu, (Item.Count > 0) or ((Item is TMenuItem2000) and (Item2000.AttachMenu <> nil)));
    DrawTextItem(Canvas, Options, Caption, Shortcut, Bitmap, State, MouseState, mir, Images);
  end;
end;


{ TMenuItem2000 }

constructor TMenuItem2000.Create(AOwner: TComponent);
begin
  inherited;

{$IFNDEF VER120}
  FImageIndex:= -1;
  FBitmap:= TBitmap.Create;
{$ENDIF}
  FControlOptions:= T_AM2000_ControlOptions.Create(Self);
  FDefaultBitmapIndex:= -1;
end;

destructor TMenuItem2000.Destroy;
begin
  if Assigned(Parent) then
    Parent.Remove(Self);

  while Count > 0 do Items[0].Free;

  // if this menu item uses AutoBitmap, it should
  // release it before destroying the bitmap
  if
{$IFNDEF VER120}
  Assigned(FBitmap) and
  (not FBitmap.Empty) and
{$ENDIF}
     (Caption <> '')
  or (DefaultIndex <> -1) 
  and (Bitmap.Handle = MenuItemCache[Caption].Bitmap)
  then Bitmap.ReleaseHandle;

{$IFNDEF VER120}
  FBitmap.Free;
{$ENDIF}
  FOptions.Free;
  FControlOptions.Free;

  inherited;
end;

procedure TMenuItem2000.Insert(Index: Integer; Item: TMenuItem2000);
begin
  inherited Insert(Index, Item);

  // update menu bar
  if (Parent = nil)
  and (ActiveMenuBar <> nil)
  and (ActiveMenuBar.HandleAllocated)
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 1, 0);
end;

function TMenuItem2000.IndexOf(Item: TMenuItem2000): Integer;
begin
  Result:= inherited IndexOf(Item);
end;

procedure TMenuItem2000.Add(Item: TMenuItem2000);
begin
  inherited Add(Item);

  // update menu bar
  if (Parent = nil)
  and (ActiveMenuBar <> nil)
  and (ActiveMenuBar.HandleAllocated)
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 1, 0);
end;

procedure TMenuItem2000.Remove(Item: TMenuItem2000);
begin
  inherited Remove(Item);

  // update menu bar
  if (Parent = nil)
  and (ActiveMenuBar <> nil)
  and (ActiveMenuBar.HandleAllocated)
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 1, 0);
end;

procedure TMenuItem2000.Delete(Index: Integer);
begin
  inherited Delete(Index);

  // update menu bar
  if (ActiveMenuBar <> nil)
  and (ActiveMenuBar.HandleAllocated)
  and (Parent = nil)
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 1, 0);
end;

procedure TMenuItem2000.Assign(Source: TPersistent);
begin
  inherited;
end;

function TMenuItem2000.IsTopLevelItem: Boolean;
begin
  Result:= (Parent = nil) or (Parent.Parent = nil);
end;

function TMenuItem2000.IsBitmapAssigned : Boolean;
  // checks if menu item has any bitmap anywhere
begin
  Result:=
{$IFNDEF VER120}
    (Assigned(FBitmap) and (not FBitmap.Empty))
{$ELSE}
    (not Bitmap.Empty)
{$ENDIF}
    or (DefaultIndex <> -1)
    or ((Caption <> '')
    and (MenuItemCache[Caption].Bitmap <> 0));
end;

function TMenuItem2000.IsOptionsAssigned: Boolean;
begin
  Result:= Assigned(FOptions);
end;

procedure TMenuItem2000.Popup(X, Y: Integer);
begin
end;

function TMenuItem2000.GetParent: TMenuItem2000;
begin
  Result:= TMenuItem2000(inherited Parent);
end;

function TMenuItem2000.IsHintStored: Boolean;
begin
  Result:= Hint <> MenuItemCache[Caption].Hint;
end;

function TMenuItem2000.IsShortCutStored: Boolean;
begin
  Result:= FShortCut <> MenuItemCache[Caption].ShortCuts;
end;

function TMenuItem2000.IsBitmapStored: Boolean;
var
  B: Boolean;
begin
  B:=
{$IFNDEF VER120}
    (FBitmap = nil) or
{$ENDIF}
    (Bitmap.Handle = 0)
    or (DefaultIndex <> -1)
    or (Bitmap.Handle = MenuItemCache[Caption].Bitmap);

  // necessary for optimization purposes
  Result:= not B;
end;

function TMenuItem2000.GetBitmap: TBitmap;
begin
{$IFNDEF VER120}
  if FBitmap = nil then FBitmap:= TBitmap.Create;
  Result:= FBitmap;
{$ELSE}
  Result:= inherited Bitmap;
{$ENDIF}

  if (not (csDesigning in ComponentState))
  then begin
    if DefaultIndex <> -1
    then begin
      Result.ReleaseHandle;
      Result.Handle:= MenuItemCache.GetDefaultBitmap(DefaultIndex);
    end;

    if (Caption <> '')
    and (Result.Handle = 0)
    then Result.Handle:= MenuItemCache[Caption].Bitmap;
  end;
end;

function TMenuItem2000.GetShortCut: String;
begin
  Result:= '';

  // inherited shortcut
  if inherited ShortCut <> 0
  then
    Result:= ShortCutToText(inherited ShortCut)
    
  else begin
    Result:= FShortCut;

    if not (csWriting in ComponentState) then begin
      if (Result = '') then Result:= MenuItemCache[Caption].ShortCuts;
      if (Result = #1) then Result:= '';
    end;
  end;
end;

function TMenuItem2000.GetHint: String;
begin
  Result:= inherited Hint;

  if not (csWriting in ComponentState) then begin
    if (Result = '') then Result:= MenuItemCache[Caption].Hint;
    if (Result = #1) then Result:= '';
  end;
end;

procedure TMenuItem2000.SetBitmap(Value: TBitmap);
begin
{$IFNDEF VER120}
  if FBitmap = nil then FBitmap := TBitmap.Create;
  FBitmap.Assign(Value);
{$ELSE}
  inherited Bitmap:= Value;
{$ENDIF}
end;

procedure TMenuItem2000.TurnSiblingsOff;
var
  I: Integer;
  SaveIRF: Boolean;
begin
  SaveIRF:= IgnoreRepaintFloating;
  IgnoreRepaintFloating:= True;
  for I:= 0 to Parent.Count -1 do
    if (Parent.Items[I] <> Self)
    and (Parent.Items[I].GroupIndex = GroupIndex)
    then
      case Parent.Items[I].Control of
        ctlNone:        Parent.Items[I].Checked:= False;
        ctlButton:      Parent.Items[I].AsButton.Down:= False;
        ctlButtonArray: Parent.Items[I].AsButtonArray.ItemIndex:= -1;
      end;
  IgnoreRepaintFloating:= SaveIRF;
end;

procedure TMenuItem2000.SetHint(Value: String);
begin
  if (Value = '')
  and (MenuItemCache[Caption].Hint <> '')
  then inherited Hint:= #1
  else inherited Hint:= Value;
end;

procedure TMenuItem2000.SetShortCut(Value: String);
var
  I: Integer;
  S, S1: String;
  SC: TShortCut;
begin
  inherited ShortCut:= 0;

  // if shortcut is empty
  if (Value = '') or (Value = #1)
  then FShortCut:= #1

  else begin
    S:= '';
    while Value <> '' do begin
      I:= Pos(';', Value);
      if I < 1 then I:= Length(Value) +1;
      S1:= Trim(Copy(Value, 1, I -1));
      System.Delete(Value, 1, I);

      if S1 = '' then System.Break;

      SC:= TextToShortCut(S1);
      if SC = 0
      then raise Exception.Create(SInvalidShortCut + S1 +'''');

      if S <> '' then AppendStr(S, ';');
      AppendStr(S, ShortCutToText(SC));
    end;
    FShortCut:= S;
  end;
end;

function TMenuItem2000.AsButtonArray: T_AM2000_ButtonArrayOptions;
begin
  if FControl = ctlButtonArray
  then Result:= FControlOptions as T_AM2000_ButtonArrayOptions
  else raise EMenuError.Create(Name + SThisIsNotAButtonArray);
end;

function TMenuItem2000.AsButton: T_AM2000_ButtonOptions;
begin
  if FControl = ctlButton
  then Result:= FControlOptions as T_AM2000_ButtonOptions
  else raise EMenuError.Create(Name + SThisIsNotAButton);
end;

function TMenuItem2000.GetItem(Index: Integer): TMenuItem2000;
var
  R: TMenuItem;
begin
  R:= inherited GetItem(Index);

  if Assigned(R)
  then Result:= TMenuItem2000(R)
  else raise EMenuError.Create(SMenuNotFound);
end;

procedure TMenuItem2000.SetCaption(const Value: String);
begin
  inherited Caption:= Value;

  if (csDesigning in ComponentState)
  and (not (csLoading in ComponentState))
  and Assigned(MenuDesignerDlg)
  then MenuDesignerDlg.UpdateCaption(Self);

  if IsTopLevelItem
  and (ActiveMenuBar <> nil)
  and ActiveMenuBar.HandleAllocated
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 1, 0);
end;

procedure TMenuItem2000.SetEnabled(Value: Boolean);
begin
  inherited Enabled:= Value;

  if IsTopLevelItem
  and Assigned(ActiveMenuBar)
  and ActiveMenuBar.HandleAllocated
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 0, 0);
end;

procedure TMenuItem2000.SetVisible(Value: Boolean);
begin
  inherited Visible:= Value;

  if IsTopLevelItem
  and Assigned(ActiveMenuBar)
  and ActiveMenuBar.HandleAllocated
  then PostMessage(ActiveMenuBar.Handle, wm_UpdateMenuBar, 1, 0);
end;

procedure TMenuItem2000.SetControl(Value: T_AM2000_ControlType);
begin
  if FControl <> Value then begin
    FControl:= Value;

    FControlOptions.Free;
    case FControl of
      ctlButton      :
        FControlOptions:= T_AM2000_ButtonOptions.Create(Self);
      ctlButtonArray :
        FControlOptions:= T_AM2000_ButtonArrayOptions.Create(Self);
//      ctlEditbox     :
//        FControlOptions:= T_AM2000_EditboxOptions.Create(Self);
//      ctlCombobox    :
//        FControlOptions:= T_AM2000_ComboboxOptions.Create(Self);
//      ctlListbox     :
//        FControlOptions:= T_AM2000_ListboxOptions.Create(Self);
      else
        FControlOptions:= T_AM2000_ControlOptions.Create(Self);
    end;

    if (csDesigning in ComponentState)
    and (not (csLoading in ComponentState))
    and Assigned(MenuDesignerDlg)
    and Assigned(MenuDesignerDlg.Designer)
    then
      with MenuDesignerDlg.Designer do begin
        SelectComponent(Self);
        Modified;
      end;
  end;
end;

function TMenuItem2000.IsOptionsStored: Boolean;
begin
  Result:= Count > 0;
end;

function TMenuItem2000.GetOptions: T_AM2000_MenuOptions;
begin
  if Assigned(FOptions)
  and (not (csLoading in ComponentState))
  and (((FOptions.ClassType = T_AM2000_MenuOptions) and ((Count > 0) or (AttachMenu <> nil)))
  or ((FOptions.ClassType = T_AM2000_BaseOptions) and (Count = 0)))
  then begin
    FOptions.Free;
    FOptions:= nil;
  end;

  if not Assigned(FOptions)
  then
    if (Count > 0)
    or (csLoading in ComponentState)
    or (not (csDesigning in ComponentState))
    then FOptions:= T_AM2000_BaseOptions.Create
    else FOptions:= T_AM2000_MenuOptions.Create;

  Result:= FOptions;  
end;

procedure TMenuItem2000.SetOptions(Value: T_AM2000_MenuOptions);
begin
  if not Assigned(FOptions)
  then
    if Count > 0
    then FOptions:= T_AM2000_BaseOptions.Create
    else FOptions:= T_AM2000_MenuOptions.Create;

  FOptions.Assign(Value);
end;

function TMenuItem2000.GetCaption: String;
begin
  Result:= inherited Caption;
end;

function TMenuItem2000.GetEnabled: Boolean;
begin
  Result:= inherited Enabled;
end;

function TMenuItem2000.GetVisible: Boolean;
begin
  Result:= inherited Visible;
end;

function TMenuItem2000.GetWidth(Canvas: TCanvas): Integer;
begin
  Result:= FControlOptions.GetWidth(Canvas);
end;

function TMenuItem2000.GetHeight(ItemHeight: Integer): Integer;
begin
  Result:= FControlOptions.GetHeight(ItemHeight);
end;

procedure TMenuItem2000.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then begin
    if (AComponent = AttachMenu) then FAttachMenu:= nil;
    if (AComponent = PopupMenu) then FPopupMenu:= nil;
  end;
end;


{ TEditableMenuItem2000 }

function TEditableMenuItem2000.GetHandle: HMenu;
begin
  Result:= TMenu(Owner).Items.Handle;
end;

function TEditableMenuItem2000.GetCount: Integer;
begin
  Result:= TMenu(Owner).Items.Count;
end;

function TEditableMenuItem2000.GetItem(Index: Integer): TMenuItem2000;
begin
  // it's safe because AnimatedMenus/98 doesn't use TMenuItem anymore
  // Only TMenuItem2000
  Result:= TMenuItem2000(TMenu(Owner).Items[Index]);
end;

procedure TEditableMenuItem2000.Insert(Index: Integer; Item: TMenuItem2000);
begin
  TMenu(Owner).Items.Insert(Index, Item);
end;

procedure TEditableMenuItem2000.Delete(Index: Integer);
begin
  TMenu(Owner).Items.Delete(Index);
end;

function TEditableMenuItem2000.IndexOf(Item: TMenuItem2000): Integer;
begin
  Result:= TMenu(Owner).Items.IndexOf(Item);
end;

procedure TEditableMenuItem2000.Add(Item: TMenuItem2000);
begin
  TMenu(Owner).Items.Add(Item);
end;

procedure TEditableMenuItem2000.Remove(Item: TMenuItem2000);
begin
  TMenu(Owner).Items.Remove(Item);
end;


initialization
  RegisterClasses([TMenuItem2000, TEditableMenuItem2000]);

finalization
  Pattern.Free;

end.

