unit mcPhoneEdit;

{ =======================================================================================

  Copyright 1998, eMCee Software
  mrichter@emceesoftware.com

  Internal development, Delphi 3, Delphi 4 (not tested under previous versions)

    +--------- Internal development
    | +------- Development
    | | +----- Design / Prototype
    | | |  +-- Build
    v v v  v
  v 0.0.0.19 (Internal development, design phase)

  =======================================================================================

  Purpose:

    This unit defines a TEditBox descendant that provides for entering and displaying telephone
    numbers.

  Function:

    This editbox component displays a formatted input area.  It supports Area Code "A", Exchange "E",
    Number "N", and Extension "X" in the following format:  '(AAA) EEE-NNNN x-XXXX'

    The two properties, PhoneNumber and FormattedPhoneNumber provide unformatted and formatted
    representations of the phone number, respectively.

    This component also supports the use of the Insert key to toggle between 'insert' and 'overstrike'
    modes.  However, there is currently no support for displaying which mode is active.

  Known Issues:

    This component currently does not:
    - support alternate phone number arrangements other than US
    - allow the formatting to be changed by the user

  =======================================================================================

    This component is offered as "freeware" and is released into the public domain with the
    exception that no copyright may be sought by anyone other than eMCee Software.

    You may use this component and its source code in any manner you wish with the
    understanding that it is provided "as is" without warranty of any kind.  In no manner,
    and under no circumstances, will eMCee Software be held liable for any damages which
    may arise from the use of this component.

    If you have any problems with the component, or have any suggestions for improvement, please
    contact eMCee Software (mrichter@emceesoftware.com).

  =======================================================================================

    Particular thanks to the members of TeamB, without whom, the knowledge of these techniques
    would be extremely difficult to find.

  ======================================================================================= }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TmcPhoneEdit = class(TEdit)
  private
    fTextEditActive : Boolean;
(*    fPhoneNumberEditActive : Boolean;
    fFormattedPhoneNumber : String;
    fPhoneNumber : String;
*)
    { Private declarations }
    function GetPhoneNumber : String;
    procedure SetPhoneNumber(const Value: String);
    function GetFormattedPhoneNumber : String;
  protected
    { Protected declarations }
(*    property Text read GetText write SetText;
*)
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure DoExit; override;
    procedure DoEnter; override;
    procedure Change; override;
  public
    { Public declarations }
    function PackPhoneNumber(PhoneNumber: String) : String;
    function UnPackPhoneNumber(PhoneNumber : String) : String;
    function DisplayMask(PhoneNumber : String) : String;
    property PhoneNumber : String Read GetPhoneNumber Write SetPhoneNumber;
    property FormattedPhoneNumber : String Read GetFormattedPhoneNumber Write SetPhoneNumber;
  published
    { Published declarations }
(*    property PhoneNumberEditActive : Boolean Read fPhoneNumberEditActive Write fPhoneNumberEditActive;
    property TextEditActive : Boolean Read fTextEditActive Write fTextEditActive;
*)    
  end;

procedure Register;

implementation

{ ============================================================================= }
procedure Register;
begin
  RegisterComponents('eMCee', [TPhoneEdit]);
end;

const
  MaxPhoneNumberSize = 14;
  InsertMode : Boolean = True;

{ == supporting routines == }

{ ============================================================================= }
function InsertCharacter(StartingText, TextToInsert : String; StartingPosition, NumberOfCharacters : integer) : String;
begin
  { insert the character and stick the rest of the text back on }
  Result := Copy(StartingText, 1, StartingPosition) +
            TextToInsert +
            Copy(StartingText, StartingPosition+NumberOfCharacters+1, Length(StartingText));
end;


{ == component methods == }

{ ============================================================================= }
function TPhoneEdit.PackPhoneNumber(PhoneNumber : string) : string;
var
  i : integer;
begin
  i := 1;
  while i <= length(PhoneNumber)
  do
  begin
    if (PhoneNumber[i] < '0') or
       (PhoneNumber[i] > '9')
    then
      Delete(PhoneNumber, i, 1)
    else
      i := i + 1;
  end;
  Result := PhoneNumber;
end;

{ ============================================================================= }
function TPhoneEdit.UnPackPhoneNumber(PhoneNumber : string) : string;
var
  PackedNumber : string;
begin
  PackedNumber := PackPhoneNumber(PhoneNumber);
  if PackedNumber = ''
  then
    Result := ''
  else
  begin
    if Length(PackedNumber) > 10
    then
    begin
      Result := '('+Copy(PackedNumber, 1, 3)+') '+Copy(PackedNumber, 4, 3)+'-'+Copy(PackedNumber, 7, 4)+
                ' x'+Copy(PackedNumber, 11, 255);
    end
    else
    if Length(PackedNumber) > 7
    then
    begin
      while Length(PackedNumber) < 10
      do
        PackedNumber := ' '+PackedNumber;
      Result := '('+Copy(PackedNumber, 1, 3)+') '+Copy(PackedNumber, 4, 3)+'-'+Copy(PackedNumber, 7, 4);
    end
    else
    if Length(PackedNumber) > 4
    then
    begin
      while Length(PackedNumber) < 7
      do
        PackedNumber := ' '+PackedNumber;
      Result := Trim(Copy(PackedNumber, 1, 3)+'-'+Copy(PackedNumber, 4, 4));
    end
    else
      Result := PackedNumber;
  end;
end;

{ ============================================================================= }
function TPhoneEdit.DisplayMask(PhoneNumber : string) : string;
var
  PackedNumber : string;
begin
  PackedNumber := PackPhoneNumber(PhoneNumber);
(*  if PackedNumber = ''
  then
    Result := ''
  else
  begin
*)
    while Length(PackedNumber) < 10
    do
      PackedNumber := ' '+PackedNumber;
    while Length(PackedNumber) < 14
    do
      PackedNumber := PackedNumber + ' ';
    Result := '('+Copy(PackedNumber, 1, 3)+') '+Copy(PackedNumber, 4, 3)+'-'+Copy(PackedNumber, 7, 4)+
              ' x'+Copy(PackedNumber, 11, 255);
(*  end;
*)
end;

{ == property editor procedures == }

{ ============================================================================= }
procedure TPhoneEdit.SetPhoneNumber(const Value: String);
begin
(*  if not fTextEditActive
  then
  begin
    fPhoneNumberEditActive := True;
    fPhoneNumber := PackPhoneNumber(Value);
    fFormattedPhoneNumber := UnPackPhoneNumber(fPhoneNumber);
*)
    Text := DisplayMask(Value);
(*  end;
  fPhoneNumberEditActive := False;
*)
end;

{ ============================================================================= }
function TPhoneEdit.GetPhoneNumber : String;
begin
  Result := PackPhoneNumber(Text);
end;

{ ============================================================================= }
function TPhoneEdit.GetFormattedPhoneNumber : String;
begin
  Result := UnPackPhoneNumber(Text);
end;

{ == event handlers == }

{ ============================================================================= }
procedure TPhoneEdit.Change;
begin
(*  if not fPhoneNumberEditActive
  then
  begin
    fPhoneNumber := PackPhoneNumber(Text);
    fFormattedPhoneNumber := UnPackPhoneNumber(Text);
  else
    fTextEditActive := True;
*)
  if not fTextEditActive
  then
  begin
    fTextEditActive := True;
    Text := DisplayMask(Text);
    inherited Change;
    fTextEditActive := False;
  end;
(*  fTextEditActive := False;
*)
end;

{ ============================================================================= }
procedure TPhoneEdit.DoExit;
begin

  { convert the formatted edit text into a 'packed' 14 digit phone number }

  Inherited DoExit;

end;

{ ============================================================================= }
procedure TPhoneEdit.DoEnter;
begin

  InsertMode := True;

  Inherited DoEnter;

end;

{ ============================================================================= }
procedure TPhoneEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
  KeyChar : char;
begin

  case Key
  of
    { insert key }
    VK_Insert :
      begin
        InsertMode := not InsertMode;
      end;
    { delete key }
    VK_Delete :
      begin
        KeyChar := chr(vk_Delete);
        KeyPress(KeyChar);
        Key := 0;
      end;
    { left arrow key }
    { right arrow key }
  end;

  Inherited KeyDown(Key, Shift);

end;

{ ============================================================================= }
procedure TPhoneEdit.KeyPress(var Key: Char);
var
  PackedNumber : string;
  NewSelStart : integer;
  NewSelLength : integer;
  i : integer;
begin

  if (Key = chr(13)) or
     (Key = chr(0))
  then
  begin
    Key := Chr(0);
    Exit;
  end;

  if ((Key >= '0') and (key <= '9')) or
     (key = chr(vk_Delete)) or
     (key = chr(vk_Back))
  then
  begin
    { pack the number and see the results of the cursor position }
    PackedNumber := PackPhoneNumber(InsertCharacter(Text, '', SelStart, SelLength));
    { determine whether we can modify the current number }
    if { the number is less than 14 digits }
       (Length(PackedNumber) < MaxPhoneNumberSize) or
       { or, if we have 14 digits, the 'insert mode' is not on, and the cursor is not at the end }
       ((Length(PackedNumber) = MaxPhoneNumberSize) and
        (not InsertMode) and
        (SelStart < GetTextLen)) or
       { or, if we have 14 digits, the [delete] key was pressed, and the cursor is not at the end }
       ((Length(PackedNumber) = MaxPhoneNumberSize) and
        (Key = chr(vk_Delete)) and
        (SelStart < GetTextLen)) or
       { or, the [backspace] key was pressed, and the cursor is not at the beginning }
       ((SelStart > 0) and
        (key = chr(vk_Back)))
    then
    begin
      { -- locate the new 'relative' cursor position (based entirely on digits, not formatting characters) }
      NewSelStart := 0;
      NewSelLength := SelLength;
      for i := 1 to SelStart
      do
      begin
        { if the current character is a digit, increment the cursor position }
        if (Text[i] >= '0') and
           (Text[i] <= '9')
        then
          NewSelStart := NewSelStart + 1;
      end;
      { -- adjust the cursor information if special keys have been pressed }
      { if the [delete] key was pressed, but no text is highlighted, highlight the character to the right }
      if (NewSelLength = 0) and
         (key = chr(vk_Delete))
      then
        NewSelLength := 1;
      { if the [backspace] key was pressed, but no text is highlighted, highlight the character to the left }
      if (key = chr(vk_Back)) and
         (NewSelStart > 0) and
         (NewSelLength = 0)
      then
      begin
        NewSelStart := NewSelStart - 1;
        NewSelLength := 1;
      end;
      { if 'overstrike' has been selected, but no text is highlighted }
      if (NewSelLength = 0) and
         (not InsertMode)
      then
        NewSelLength := 1;
      { if the [delete] or [backspace] keys were pressed, set key to the empty string }
      if (Key = chr(vk_Delete)) or
         (Key = chr(vk_Back))
      then
        Key := chr(0);
      { update the text to reflect the key pressed }
      PackedNumber := PackPhoneNumber(Text);
      PackedNumber := InsertCharacter(PackedNumber, Key, NewSelStart, NewSelLength);
      Text := DisplayMask(PackedNumber);
      { move the cursor forward if necessary }
      if key <> chr(0)
      then
        NewSelStart := NewSelStart + 1;
      if NewSelStart > 0
      then
      begin
        { reposition the actual cursor }
        for i := 1 to Length(Text)
        do
        begin
          if (Text[i] >= '0') and
             (Text[i] <= '9')
          then
            NewSelStart := NewSelStart - 1;
          if NewSelStart = 0
          then
            Break;
        end;
        SelStart := i;
      end
      else
      begin
        { reposition the actual cursor }
        for i := 1 to Length(Text)
        do
        begin
          if (Text[i] >= '0') and
             (Text[i] <= '9')
          then
            Break;
        end;
        SelStart := i-1;
      end;
    end;
  end;

  Key := chr(0);

end;

end.
